/* OdLib_Io - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.io.odtable;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.opendocument.io.OdXML;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class CellStyleNameEngine {

    private final static String COLUMNSTYLE_PREFIX = "co";
    private final static String CELLSTYLE_PREFIX = "ce";
    private final static String DATASTYLE_PREFIX = "N";
    private final static String ISODATE_DATASTYLE_NAME = "N101";
    private final static String DEFAULT_CELLSTYLE_NAME = "Default";
    private final List<String> columnStyleNameList = new ArrayList<String>();
    private final Map<String, TableCellStyleNameProvider> tableInfoMap = new HashMap<String, TableCellStyleNameProvider>();
    private final Map<OdCellKey, CellStyle> cellStyleNameMap = new LinkedHashMap<OdCellKey, CellStyle>();
    private final Map<Integer, Integer> currencyNumerMap = new LinkedHashMap<Integer, Integer>();
    private boolean withISODate = false;
    private int columnStyleNumber = 1;
    private int cellStyleNumber = 1;
    private int currencyStyleNumber = 102;

    public CellStyleNameEngine(Map<String, OdTableDef> odTableDefMap) {
        for (Map.Entry<String, OdTableDef> entry : odTableDefMap.entrySet()) {
            OdTableDef odTableDef = entry.getValue();
            TableCellStyleNameProvider tableCellStyleNameProvider = new TableCellStyleNameProvider();
            int count = odTableDef.getOdColumnDefCount();
            for (int i = 0; i < count; i++) {
                tableCellStyleNameProvider.addColumn(odTableDef.getOdColumnDef(i));
            }
            tableInfoMap.put(entry.getKey(), tableCellStyleNameProvider);
        }
    }

    public CellStyleNameProvider getCellStyleNameProvider(String tableName) {
        return tableInfoMap.get(tableName);
    }

    public void addAtTableStart(String tableName, XMLWriter xmlWriter) throws IOException {
        TableCellStyleNameProvider tableInfo = tableInfoMap.get(tableName);
        if (tableInfo == null) {
            return;
        }
        for (OdColumn odColumn : tableInfo.odColumnList) {
            xmlWriter.startOpenTag("table:table-column");
            xmlWriter.addAttribute("table:style-name", odColumn.getStyleName());
            xmlWriter.addAttribute("table:default-cell-style-name", odColumn.getDefaultCellStyle().getName());
            xmlWriter.closeEmptyTag();
        }
    }

    /**
     * Doit être invoqué après l'utilisation de getCellStyleNameProvider car son contenu n'est pas
     * fixé entièrement à l'avance
     */
    public void addInAutomaticStyles(XMLWriter xmlWriter) throws IOException {
        for (String columnStyleName : columnStyleNameList) {
            OdXML.openStyle(xmlWriter, columnStyleName, "table-column", null);
            xmlWriter.startOpenTag("style:table-column-properties");
            xmlWriter.addAttribute("fo:break-before", "auto");
            xmlWriter.addAttribute("style:column-width", "3.5cm");
            xmlWriter.closeEmptyTag();
            OdXML.closeStyle(xmlWriter);
        }
        if (withISODate) {
            xmlWriter.startOpenTag("number:date-style");
            xmlWriter.addAttribute("style:name", ISODATE_DATASTYLE_NAME);
            xmlWriter.endOpenTag();
            xmlWriter.startOpenTag("number:year");
            xmlWriter.addAttribute("number:style", "long");
            xmlWriter.closeEmptyTag();
            xmlWriter.addSimpleElement("number:text", "-");
            xmlWriter.startOpenTag("number:month");
            xmlWriter.addAttribute("number:style", "long");
            xmlWriter.closeEmptyTag();
            xmlWriter.addSimpleElement("number:text", "-");
            xmlWriter.startOpenTag("number:day");
            xmlWriter.addAttribute("number:style", "long");
            xmlWriter.closeEmptyTag();
            xmlWriter.closeTag("number:date-style");
        }
        for (Map.Entry<Integer, Integer> entry : currencyNumerMap.entrySet()) {
            int currencyInteger = entry.getKey();
            int number = entry.getValue();
            String dataStyleName = DATASTYLE_PREFIX + number;
            int subunitLength = CurrencyInteger.getSubunitLength(currencyInteger);
            String symbol = getSymbol(currencyInteger);
            boolean before = ((currencyInteger == CurrencyInteger.USDOLLAR) || (currencyInteger == CurrencyInteger.POUND));

            xmlWriter.startOpenTag("number:currency-style");
            xmlWriter.addAttribute("style:name", dataStyleName + "P0");
            xmlWriter.addAttribute("style:volatile", "true");
            xmlWriter.endOpenTag();
            if (before) {
                xmlWriter.addSimpleElement("number:currency-symbol", symbol);
            }
            xmlWriter.startOpenTag("number:number");
            xmlWriter.addAttribute("number:decimal-places", subunitLength);
            xmlWriter.addAttribute("number:min-integer-digits", "1");
            xmlWriter.addAttribute("number:grouping", "true");
            xmlWriter.closeEmptyTag();
            if (!before) {
                xmlWriter.addSimpleElement("number:text", " ");
                xmlWriter.addSimpleElement("number:currency-symbol", symbol);
            }
            xmlWriter.closeTag("number:currency-style");

            xmlWriter.startOpenTag("number:currency-style");
            xmlWriter.addAttribute("style:name", dataStyleName);
            xmlWriter.endOpenTag();
            xmlWriter.startOpenTag("style:text-properties");
            xmlWriter.addAttribute("fo:color", "#ff0000");
            xmlWriter.closeEmptyTag();
            xmlWriter.addSimpleElement("number:text", "-");
            if (before) {
                xmlWriter.addSimpleElement("number:currency-symbol", symbol);
            }
            xmlWriter.startOpenTag("number:number");
            xmlWriter.addAttribute("number:decimal-places", subunitLength);
            xmlWriter.addAttribute("number:min-integer-digits", "1");
            xmlWriter.addAttribute("number:grouping", "true");
            xmlWriter.closeEmptyTag();
            if (!before) {
                xmlWriter.addSimpleElement("number:text", " ");
                xmlWriter.addSimpleElement("number:currency-symbol", symbol);
            }
            xmlWriter.startOpenTag("style:map");
            xmlWriter.addAttribute("style:condition", "value()>=0");
            xmlWriter.addAttribute("style:apply-style-name", dataStyleName + "P0");
            xmlWriter.closeEmptyTag();
            xmlWriter.closeTag("number:currency-style");
        }
        for (CellStyle cellStyle : cellStyleNameMap.values()) {
            String parentStyleName = cellStyle.getParentStyleName();
            if (parentStyleName == null) {
                parentStyleName = DEFAULT_CELLSTYLE_NAME;
            }
            OdXML.startStyleOpenTag(xmlWriter, cellStyle.getName(), "table-cell", parentStyleName);
            xmlWriter.addAttribute("style:data-style-name", cellStyle.getDataStyleName());
            xmlWriter.closeEmptyTag();
        }
    }

    private CellStyle initCellStyle(OdCellKey odCellKey) {
        CellStyle cellStyle = cellStyleNameMap.get(odCellKey);
        if (cellStyle != null) {
            return cellStyle;
        }
        String name = CELLSTYLE_PREFIX + cellStyleNumber;
        cellStyleNumber++;
        String dataStyleName = null;
        switch (odCellKey.getType()) {
            case OdColumnDef.ISODATE_TYPE:
                withISODate = true;
                dataStyleName = ISODATE_DATASTYLE_NAME;
                break;
            case OdColumnDef.CURRENCY_TYPE:
                int currencyInteger = odCellKey.getCurrencyInteger();
                int number;
                if (currencyNumerMap.containsKey(currencyInteger)) {
                    number = currencyNumerMap.get(currencyInteger);
                } else {
                    number = currencyStyleNumber;
                    currencyStyleNumber++;
                    currencyNumerMap.put(currencyInteger, number);
                }
                dataStyleName = DATASTYLE_PREFIX + number;
                break;
        }
        cellStyle = new CellStyle(name, dataStyleName, odCellKey.getStyleName());
        cellStyleNameMap.put(odCellKey, cellStyle);
        return cellStyle;
    }

    private static String getSymbol(int currencyInteger) {
        if (currencyInteger == CurrencyInteger.EURO) {
            return "€";
        }
        if (currencyInteger == CurrencyInteger.USDOLLAR) {
            return "$";
        }
        if (currencyInteger == CurrencyInteger.POUND) {
            return "£";
        }
        return CurrencyInteger.toString(currencyInteger);
    }


    private class TableCellStyleNameProvider implements CellStyleNameProvider {

        private final List<OdColumn> odColumnList = new ArrayList<OdColumn>();

        private TableCellStyleNameProvider() {
        }

        @Override
        public String getCellStyleName(int columnNumber, short type, int currencyInteger, String parentStyleName) {
            OdColumn odColumn = odColumnList.get(columnNumber - 1);
            if (parentStyleName == null) {
                OdCellKey defaultOdCellKey = odColumn.defaultOdCellKey;
                if ((defaultOdCellKey.getType() == type) && (defaultOdCellKey.getCurrencyInteger() == currencyInteger)) {
                    return null;
                }
            }
            OdCellKey odCellKey = OdCellKey.newInstance(type, currencyInteger, parentStyleName);
            CellStyle cellStyle = initCellStyle(odCellKey);
            return cellStyle.getName();
        }

        private void addColumn(OdColumnDef columnDef) {
            short type = columnDef.getType();
            int currencyInteger = columnDef.getCurrencyInteger();
            String styleName = COLUMNSTYLE_PREFIX + String.valueOf(columnStyleNumber);
            columnStyleNumber++;
            columnStyleNameList.add(styleName);
            OdCellKey defaultOdCellKey = OdCellKey.newInstance(type, currencyInteger, null);
            CellStyle defaultCellStyle = initCellStyle(defaultOdCellKey);
            odColumnList.add(new OdColumn(styleName, defaultOdCellKey, defaultCellStyle));
        }

    }


    private static class CellStyle {

        private final String name;
        private final String dataStyleName;
        private final String parentStyleName;

        public CellStyle(String name, String dataStyleName, String parentStyleName) {
            this.name = name;
            this.dataStyleName = dataStyleName;
            this.parentStyleName = parentStyleName;
        }

        private String getName() {
            return name;
        }

        private String getDataStyleName() {
            return dataStyleName;
        }

        private String getParentStyleName() {
            return parentStyleName;
        }

    }


    private static class OdColumn {

        private final String styleName;
        private final OdCellKey defaultOdCellKey;
        private final CellStyle defaultCellStyle;

        private OdColumn(String styleName, OdCellKey defaultOdCellKey, CellStyle defaultCellStyle) {
            this.styleName = styleName;
            this.defaultOdCellKey = defaultOdCellKey;
            this.defaultCellStyle = defaultCellStyle;
        }

        public String getStyleName() {
            return styleName;
        }

        public OdCellKey getDefaultOdCellKey() {
            return defaultOdCellKey;
        }

        public CellStyle getDefaultCellStyle() {
            return defaultCellStyle;
        }

    }

}
