/* Scrutari - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin;

import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.SctContextConstants;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.servlets.admin.contentstreams.LexiesOdsStream;
import fr.exemole.sctserver.servlets.admin.contentstreams.MotsclesOdsStream;
import fr.exemole.sctserver.servlets.admin.contentstreams.URITxtStream;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.ErrorPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.FichesPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.IndexPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.JournalPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.JournauxPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.LexiesPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.MotsclesPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.StatsPage;
import fr.exemole.sctserver.servlets.selection.SelectionManager;
import fr.exemole.sctserver.servlets.selection.SelectionUnit;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.request.RequestConstants;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class AdminRequest implements AdminConstants {

    private final ScrutariSession scrutariSession;
    private final SctServer sctServer;
    private final SelectionManager selectionManager;
    private boolean notFound = false;
    private short responseHandlerType;
    private Parameters parameters;
    private LocalisationInfo localisationInfo = new LocalisationInfo(LangInteger.build("fr"));
    private CommandMessage commandMessage;
    private int journalDateInteger;

    private AdminRequest(SctContext sctContext, String pathInfo, RequestMap requestMap) {
        this.sctServer = sctContext.getSctServer();
        this.scrutariSession = sctContext.getSctServer().getScrutariSession();
        this.selectionManager = (SelectionManager) sctContext.getContextObject(SctContextConstants.SELECTIONMANAGER_NAME);
        if (pathInfo.equals("/")) {
            initAndDoCommand(requestMap);
        } else if (pathInfo.equals("/index.html")) {
            responseHandlerType = INDEX_PAGE;
        } else if (pathInfo.equals("/stats.html")) {
            responseHandlerType = STATS_PAGE;
        } else if (pathInfo.equals("/fiches.html")) {
            responseHandlerType = FICHES_PAGE;
        } else if (pathInfo.equals("/lexies.html")) {
            responseHandlerType = LEXIES_PAGE;
        } else if (pathInfo.equals("/uri.txt")) {
            responseHandlerType = URI_STREAM;
        } else if (pathInfo.equals("/journaux.html")) {
            responseHandlerType = JOURNAUX_PAGE;
        } else if (pathInfo.startsWith("/motscles")) {
            parameters = new Parameters();
            if (!parameters.init(pathInfo.substring(9), false)) {
                setNotFound();
                return;
            }
            String extension = parameters.getExtension();
            if (extension.equals("html")) {
                responseHandlerType = MOTSCLES_PAGE;
            } else if (extension.equals("ods")) {
                responseHandlerType = MOTSCLES_STREAM;
            } else {
                setNotFound();
            }
        } else if (pathInfo.startsWith("/lexies_")) {
            parameters = new Parameters();
            if (!parameters.init(pathInfo.substring(8), true)) {
                setNotFound();
                return;
            }
            String extension = parameters.getExtension();
            if (extension.equals("html")) {
                responseHandlerType = LEXIES_PAGE;
            } else if (extension.equals("ods")) {
                responseHandlerType = LEXIES_STREAM;
            } else {
                setNotFound();
            }
        } else if (pathInfo.startsWith("/journal_")) {
            JournalParameters journalParameters = new JournalParameters();
            if (!journalParameters.init(pathInfo.substring(9))) {
                setNotFound();
                return;
            }
            String extension = journalParameters.getExtension();
            if (extension.equals("html")) {
                responseHandlerType = JOURNAL_PAGE;
                journalDateInteger = journalParameters.getDateInteger();
            } else {
                setNotFound();
            }
        } else {
            setNotFound();
        }
    }

    public static AdminRequest init(SctContext sctContext, String pathInfo, RequestMap requestMap) {
        AdminRequest adminRequest = new AdminRequest(sctContext, pathInfo, requestMap);
        if (adminRequest.notFound) {
            return null;
        } else {
            return adminRequest;
        }
    }

    public ResponseHandler getResponseHandler() {
        switch (responseHandlerType) {
            case LEXIES_STREAM:
                return new LexiesOdsStream(scrutariSession, localisationInfo, parameters.getPageLangInteger(), parameters.getSelectionUnit());
            case URI_STREAM:
                return new URITxtStream(scrutariSession);
            case MOTSCLES_STREAM:
                return new MotsclesOdsStream(scrutariSession, localisationInfo, parameters.getSelectionUnit());
            default:
                AdminHtmlPage page = getAdminHtmlPage();
                page.setLocalisationInfo(localisationInfo);
                page.setCurrentPage(responseHandlerType);
                page.setCommandMessage(commandMessage);
                return page;

        }
    }

    private AdminHtmlPage getAdminHtmlPage() {
        switch (responseHandlerType) {
            case INDEX_PAGE:
                return new IndexPage(sctServer);
            case ERROR_PAGE:
                return new ErrorPage(sctServer);
            case STATS_PAGE:
                return new StatsPage(sctServer, selectionManager);
            case FICHES_PAGE:
                return new FichesPage(sctServer);
            case MOTSCLES_PAGE:
                return new MotsclesPage(sctServer, selectionManager);
            case LEXIES_PAGE:
                return new LexiesPage(sctServer, selectionManager, (parameters != null) ? parameters.getPageLangInteger() : 0);
            case JOURNAUX_PAGE:
                return new JournauxPage(sctServer);
            case JOURNAL_PAGE:
                return new JournalPage(sctServer, journalDateInteger);
            default:
                throw new SwitchException("responseHandlerType=" + responseHandlerType);
        }
    }

    private void initAndDoCommand(RequestMap requestMap) {
        String cmd = requestMap.getParameter(RequestConstants.COMMAND_PARAMETER);
        if (cmd == null) {
            responseHandlerType = INDEX_PAGE;
        } else {
            Command command = Command.getCommandInstance(cmd, sctServer);
            if (command == null) {
                responseHandlerType = ERROR_PAGE;
                commandMessage = CommandMessage.error("error_unknowncommand", cmd);
            } else {
                responseHandlerType = INDEX_PAGE;
                commandMessage = command.doCommand(requestMap);
            }
        }
    }

    public static String pageToString(short pageType) {
        switch (pageType) {
            case INDEX_PAGE:
                return "index";
            case STATS_PAGE:
                return "stats";
            case FICHES_PAGE:
                return "fiches";
            case MOTSCLES_PAGE:
                return "motscles";
            case LEXIES_PAGE:
                return "lexies";
            case JOURNAUX_PAGE:
                return "journaux";
            default:
                return null;
        }
    }

    private void setNotFound() {
        notFound = true;
    }


    private class JournalParameters {

        private String extension;
        private int dateInteger;

        JournalParameters() {
        }

        boolean init(String pathInfo) {
            int idx = pathInfo.lastIndexOf(".");
            if (idx == -1) {
                return false;
            }
            extension = pathInfo.substring(idx + 1);
            pathInfo = pathInfo.substring(0, idx);
            try {
                dateInteger = DateInteger.parse(pathInfo);
            } catch (DateIntegerException ide) {
                return false;
            }
            return true;
        }

        public String getExtension() {
            return extension;
        }

        public int getDateInteger() {
            return dateInteger;
        }

    }


    private class Parameters {

        private String extension;
        private SelectionUnit selectionUnit;
        private int pageLangInteger;

        Parameters() {
        }

        boolean init(String pathInfo, boolean testLangue) {
            int idx = pathInfo.lastIndexOf(".");
            if (idx == -1) {
                return false;
            }
            extension = pathInfo.substring(idx + 1);
            pathInfo = pathInfo.substring(0, idx);
            int idx2 = pathInfo.lastIndexOf("-");
            if (idx2 != -1) {
                String suname = pathInfo.substring(idx2 + 1);
                selectionUnit = selectionManager.getSelectionUnit(suname);
                if (getSelectionUnit() == null) {
                    return false;
                }
                pathInfo = pathInfo.substring(0, idx2);
            }
            if (testLangue) {
                try {
                    pageLangInteger = LangInteger.parse(pathInfo);
                } catch (LangIntegerException lie) {
                    return false;
                }
            }
            return true;
        }

        public String getExtension() {
            return extension;
        }

        public SelectionUnit getSelectionUnit() {
            return selectionUnit;
        }

        public int getPageLangInteger() {
            return pageLangInteger;
        }

    }

}
