/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.contentstreams;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.servlets.selection.SelectionUnit;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdXML;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.ZipEntrySource;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.data.BaseData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.stats.CountStats;
import net.scrutari.db.stats.MotcleCountStats;
import net.scrutari.db.tools.util.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesOdsStream implements ResponseHandler {

    private final ScrutariSession scrutariSession;
    private final SelectionUnit selectionUnit;
    private final int workingLangInteger;

    public MotsclesOdsStream(ScrutariSession scrutariSession, LocalisationInfo localisationInfo, SelectionUnit selectionUnit) {
        this.scrutariSession = scrutariSession;
        this.selectionUnit = selectionUnit;
        this.workingLangInteger = localisationInfo.getWorkingLangInteger();
    }

    @Override
    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        odZip.setContentZipEntrySource(new ContentZipEntrySource());
        odZip.zip(outputStream);
        outputStream.close();
    }


    private class ContentZipEntrySource implements ZipEntrySource {

        private String currentBaseLibelle;
        private String currentThesaurusLibelle;

        private ContentZipEntrySource() {
        }

        @Override
        public void writeEntry(OutputStream outputStream) throws IOException {
            BufferedWriter buf = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(buf);
            xmlWriter.appendXMLDeclaration();
            MotsclesXMLPart motsclesXMLPart = new MotsclesXMLPart(xmlWriter);
            motsclesXMLPart.start();
            motsclesXMLPart.addMotscles();
            motsclesXMLPart.end();
            buf.flush();
        }


        private class MotsclesXMLPart extends XMLPart {

            private final ScrutariDB scrutariDB;
            private final int[] motcleLangArray;
            private final DataAccess dataAccess;
            private final CountStats countStats;

            private MotsclesXMLPart(XMLWriter xmlWriter) {
                super(xmlWriter);
                scrutariDB = scrutariSession.getScrutariDB();
                dataAccess = scrutariDB.openDataAccess();
                motcleLangArray = scrutariDB.getLangStats().getAllMotcleLangArray();
                countStats = scrutariDB.getCountStats();
            }

            private void start() throws IOException {
                OdXML.openDocumentContent(this);
                OdXML.openBody(this);
                OdXML.openSpreadsheet(this);
                OdXML.openTable(this, "motscles");
                addLibRow();
            }

            private void addLibRow() throws IOException {
                OdXML.openTableRow(this);
                int length = motcleLangArray.length;
                for (int i = 0; i < length; i++) {
                    OdXML.addStringTableCell(this, LangInteger.toString(motcleLangArray[i]));
                }
                OdXML.addStringTableCell(this, "occurence");
                OdXML.addStringTableCell(this, "thesaurus");
                OdXML.addStringTableCell(this, "base");
                OdXML.closeTableRow(this);
            }

            private void addMotscles() throws IOException {
                int baseCount = dataAccess.getBaseCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseData baseData = dataAccess.getBaseDataByIndex(i);
                    addBaseData(baseData);
                }
            }

            private void addBaseData(BaseData baseData) throws IOException {
                currentBaseLibelle = LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT, workingLangInteger, "?");
                int thesaurusCount = baseData.getThesaurusCount();
                for (int i = 0; i < thesaurusCount; i++) {
                    ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
                    addThesaurusData(thesaurusData);
                }
            }

            private void addThesaurusData(ThesaurusData thesaurusData) throws IOException {
                if ((selectionUnit != null) && (!selectionUnit.contains(thesaurusData.getThesaurusURI()))) {
                    return;
                }
                currentThesaurusLibelle = LibelleUtils.getLibelleString(thesaurusData, workingLangInteger, "?");
                int motcleCount = thesaurusData.getMotcleCount();
                for (int i = 0; i < motcleCount; i++) {
                    addMotcleData(dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
                }
            }

            private void addMotcleData(MotcleData motcleData) throws IOException {
                OdXML.openTableRow(this);
                int length = motcleLangArray.length;
                for (int i = 0; i < length; i++) {
                    Libelle libelle = motcleData.getLibelleByLang(motcleLangArray[i]);
                    if (libelle != null) {
                        OdXML.addStringTableCell(this, libelle.getLibelleString());
                    } else {
                        OdXML.addEmptyTableCell(this);
                    }
                }
                MotcleCountStats motcleCountStats = countStats.getMotcleCountStats(motcleData.getCodeMotcle());
                OdXML.addNumberTableCell(this, motcleCountStats.getFicheCount());
                OdXML.addStringTableCell(this, currentThesaurusLibelle);
                OdXML.addStringTableCell(this, currentBaseLibelle);
                OdXML.closeTableRow(this);
            }

            private void end() throws IOException {
                OdXML.closeTable(this);
                OdXML.closeSpreadsheet(this);
                OdXML.closeBody(this);
                OdXML.closeDocumentContent(this);
                dataAccess.close();
            }

        }

    }

}
