/* Scrutari - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.ConfState;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.api.conf.ConfConstants;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.servlets.admin.ConfUpdateCommand;
import fr.exemole.sctserver.servlets.admin.RestartCommand;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.AttributesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.CorpusClassesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.SourcesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.UriCodesPrinter;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.CommandBoxParametersBuilder;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlConstants;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.scrutari.db.api.AttributeKeyCoder;


/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends AdminHtmlPage implements ConfConstants {

    private final static HTMLAttributes ERROR_ATTR = HA.classes(ERRORMESSAGE_SPANCLASS);
    private ConfState confState;

    public IndexPage(SctServer sctServer) {
        init(sctServer);
        confState = sctServer.getConfState();
        CommandBoxParametersBuilder commandBoxParametersBuilder = new CommandBoxParametersBuilder();
        commandBoxParametersBuilder.setCommandKeyDivAttributes(HA.classes("d_GLB_CommandKey"));
        commandBoxParametersBuilder.setFormAttributes(HA.action(".").method(HtmlConstants.GET_METHOD));
        commandBoxParametersBuilder.setSubmitDivAttributes(HA.classes("d_GLB_Submit"));
        setCommandBoxParameters(commandBoxParametersBuilder.toCommandBoxParameters());
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        String dataLog = sctServer.getSctServerStorage().getLogStorage().getLog(SctLogConstants.UPDATE_LOG, SctLogConstants.DATA_UPDATE_LOG);
        if ((dataLog != null) && (!dataLog.isEmpty())) {
            startP();
            startSpan(ERROR_ATTR);
            localize("info_updateprocesslog");
            endSpan();
            endP();
            startPre(null);
            escape(dataLog, true);
            endPre();
        }

        printRestartForm();

        printConfState();

        end();
    }

    private void printConfState() {
        startH2(null);
        localize("title_conf_files");
        endH2();
        short state = confState.getGlobalState();
        boolean printConfDetails = false;
        switch (state) {
            case ConfState.GLOBAL_CONFIO_ERROR:
                startP();
                startSpan(ERROR_ATTR);
                localize("error_global_confio");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_UNDEFINED:
                startP();
                startSpan(ERROR_ATTR);
                localize("error_global_confnull");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_OK:
                startUl();
                printConfFileLi(SOURCES_NAME);
                printConfFileLi(ATTRIBUTES_NAME);
                printConfFileLi(CORPUS_CLASSES_NAME);
                printConfFileLi(URI_CODES_NAME);
                printConfFileLi(FEED_NAME);
                endUl();
                printConfDetails = true;
                break;
        }
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(ConfUpdateCommand.COMMANDNAME);
        commandBox.setCommandKey(ConfUpdateCommand.COMMANDKEY);
        startCommandBox(commandBox);
        endCommandBox("submit_update");
        if (printConfDetails) {
            startH2(null);
            localize("title_conf_details");
            endH2();
            printSourcesDetails();
            printAttributesDetails();
            printCorpusClassesDetails();
            printUriCodesDetails();
            printFeedDetails();
        }
    }

    private void printConfFileLi(String name) {
        startLi();
        printFileInfo(name, true);
        endLi();
    }

    private void printSourcesDetails() {
        startH3(HA.id(SOURCES_NAME));
        localize("title_conf_sources");
        append(" / ");
        printFileInfo(SOURCES_NAME, false);
        endH3();
        printConfLog(SOURCES_NAME);
        SourcesPrinter.printSourceInfoList(this);
    }

    private void printAttributesDetails() {
        startH3(HA.id(ATTRIBUTES_NAME));
        localize("title_conf_attributes");
        append(" / ");
        printFileInfo(ATTRIBUTES_NAME, false);
        endH3();
        printConfLog(ATTRIBUTES_NAME);
        AttributeKeyCoder attributeKeyCoder = sctServer.getScrutariSession().getScrutariDB().getAttributeKeyCoder();
        if (attributeKeyCoder.getAttributeDefCount() == 0) {
            startP();
            startSpan(HA.classes(INFO_SPANCLASS));
            localize("info_attributes_none");
            endSpan();
            endP();
            return;
        }
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.PRIMARY_GROUP), "title_conf_attributes_primary");
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.SECONDARY_GROUP), "title_conf_attributes_secondary");
    }

    private void printAttributesList(AttributeDefList attributeDefList, String titleKey) {
        if (attributeDefList.getAttributeDefCount() == 0) {
            return;
        }
        startH4(null);
        localize(titleKey);
        endH4();
        AttributesPrinter.printAttributesList(this, attributeDefList);
    }

    private void printCorpusClassesDetails() {
        startH3(HA.id(CORPUS_CLASSES_NAME));
        localize("title_conf_corpusclasses");
        append(" / ");
        printFileInfo(CORPUS_CLASSES_NAME, false);
        endH3();
        printConfLog(CORPUS_CLASSES_NAME);
        CorpusClassesPrinter.printCorpusClasses(this);
    }

    private void printUriCodesDetails() {
        startH3(HA.id(URI_CODES_NAME));
        localize("title_conf_uricodes");
        append(" / ");
        printFileInfo(URI_CODES_NAME, false);
        endH3();
        printConfLog(URI_CODES_NAME);
        UriCodesPrinter.printUriCodes(this);
    }

    private void printFeedDetails() {
        startH3(HA.id(FEED_NAME));
        localize("title_conf_feed");
        append(" / ");
        printFileInfo(FEED_NAME, false);
        endH3();
        printConfLog(FEED_NAME);
    }

    private void printRestartForm() throws IOException {
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(RestartCommand.COMMANDNAME);
        commandBox.setCommandKey(RestartCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startP();
        HtmlPrinterUtils.labelInput(this, HA.type(HtmlConstants.CHECKBOX_TYPE).name(RestartCommand.FORCE_PARAMNAME).value("1"), "lib_restart_force", null);
        endP();
        endCommandBox("submit_restart");
    }

    private void printConfLog(String name) {
        short fileState = confState.getFileState(name);
        switch (fileState) {
            case FILE_STATE_UNKNOWN:
            case FILE_MISSING:
            case FILE_OK:
                return;
        }
        String logContent = sctServer.getSctServerStorage().getLogStorage().getLog(SctLogConstants.CONF_LOG, name);
        if (logContent == null) {
            return;
        }
        startP();
        localize("title_log");
        endP();
        startPre(HA.classes(LOG_PRECLASS));
        escape(logContent, true);
        endPre();
    }

    private void printFileInfo(String name, boolean withLink) {
        String[] array = getKeys(confState.getFileState(name));
        if (withLink) {
            startA(HA.href("#" + name));
        }
        append(name);
        append(".xml");
        if (withLink) {
            endA();
        }
        append(" (");
        startSpan(HA.classes(array[1]));
        localize("file_" + array[0]);
        endSpan();
        append(")");
    }

    private String[] getKeys(short fileState) {
        String spanClass = null;
        String key = null;
        switch (fileState) {
            case FILE_STATE_UNKNOWN:
                key = "state_unknown";
                spanClass = NEUTRAL_SPANCLASS;
                break;
            case FILE_MISSING:
                key = "missing";
                spanClass = NEUTRAL_SPANCLASS;
                break;
            case FILE_XML_MALFORMED:
                key = "xml_malformed";
                spanClass = ERROR_SPANCLASS;
                break;
            case FILE_XML_ERROR:
                key = "xml_error";
                spanClass = ERROR_SPANCLASS;
                break;
            case FILE_XML_WARNING:
                key = "xml_warning";
                spanClass = WARNING_SPANCLASS;
                break;
            case FILE_OK:
                key = "ok";
                spanClass = OK_SPANCLASS;
                break;
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
        String[] array = new String[2];
        array[0] = key;
        array[1] = spanClass;
        return array;
    }

}
