/* ScrutariDataURI - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri.tree.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.MotcleNode;
import net.scrutari.datauri.tree.ThesaurusNode;
import net.scrutari.datauri.tree.URINode;
import net.scrutari.datauri.tree.URITree;


/**
 *
 * @author Vincent Calame
 */
public class TsvReader {

    private InternalURITree uriTree = new InternalURITree();
    private InternalBaseNode currentBaseNode;

    private TsvReader() {
    }

    private URITree toURITree() {
        return uriTree;
    }

    public static URITree readURITree(Reader reader) throws IOException {
        BufferedReader bufreader = new BufferedReader(reader);
        TsvReader tsvReader = new TsvReader();
        String ligne = null;
        while ((ligne = bufreader.readLine()) != null) {
            tsvReader.readLine(ligne);
        }
        return tsvReader.toURITree();
    }

    private void readLine(String line) {
        if (line.length() < 2) {
            return;
        }
        char caracInitial = line.charAt(0);
        char caracActive = line.charAt(1);
        boolean active = true;
        if (caracActive == ':') {
            active = false;
        }
        int idx = line.indexOf('\t');
        String associatedObject = null;
        String uriString = null;
        if (idx != -1) {
            associatedObject = line.substring(idx + 1);
            uriString = line.substring(2, idx);
        } else {
            uriString = line.substring(2);
        }
        AbstractURINode uriNode = null;
        switch (caracInitial) {
            case 'b':
                uriNode = readBase(uriString);
                break;
            case 'c':
                uriNode = readCorpus(uriString);
                break;
            case 't':
                uriNode = readThesaurus(uriString);
                break;
            case 'f':
                uriNode = readFiche(uriString);
                break;
            case 'm':
                uriNode = readMotcle(uriString);
                break;
            default:
                return;
        }
        if (uriNode != null) {
            uriNode.setActive(active);
            uriNode.setAssociatedObject(associatedObject);
        }
    }

    private InternalBaseNode readBase(String uriString) {
        int idx = uriString.indexOf('/');
        if (idx == -1) {
            return null;
        }
        String authorityUUID = uriString.substring(0, idx);
        String baseName = uriString.substring(idx + 1);
        BaseURI baseURI = new BaseURI(authorityUUID, baseName);
        InternalBaseNode baseNode = new InternalBaseNode(baseURI);
        uriTree.addBaseNode(baseNode);
        currentBaseNode = baseNode;
        return baseNode;
    }

    private InternalCorpusNode readCorpus(String uriString) {
        BaseURI baseURI = currentBaseNode.getBaseURI();
        CorpusURI corpusURI = new CorpusURI(baseURI, uriString);
        InternalCorpusNode corpusNode = new InternalCorpusNode(corpusURI);
        currentBaseNode.addCorpusNode(corpusNode);
        return corpusNode;
    }

    private InternalThesaurusNode readThesaurus(String uriString) {
        BaseURI baseURI = currentBaseNode.getBaseURI();
        ThesaurusURI thesaurusURI = new ThesaurusURI(baseURI, uriString);
        InternalThesaurusNode thesaurusNode = new InternalThesaurusNode(thesaurusURI);
        currentBaseNode.addThesaurusNode(thesaurusNode);
        return thesaurusNode;
    }

    private InternalFicheNode readFiche(String uriString) {
        InternalCorpusNode corpusNode = currentBaseNode.currentCorpusNode;
        CorpusURI corpusURI = corpusNode.getCorpusURI();
        FicheURI ficheURI = new FicheURI(corpusURI, uriString);
        InternalFicheNode ficheNode = new InternalFicheNode(ficheURI);
        corpusNode.addFicheNode(ficheNode);
        return ficheNode;
    }

    private InternalMotcleNode readMotcle(String uriString) {
        InternalThesaurusNode thesaurusNode = currentBaseNode.currentThesaurusNode;
        ThesaurusURI thesaurusURI = thesaurusNode.getThesaurusURI();
        MotcleURI motcleURI = new MotcleURI(thesaurusURI, uriString);
        InternalMotcleNode ficheNode = new InternalMotcleNode(motcleURI);
        thesaurusNode.addMotcleNode(ficheNode);
        return ficheNode;
    }

    private static class InternalURITree implements URITree {

        private List<InternalBaseNode> baseNodeList = new ArrayList<InternalBaseNode>();

        private InternalURITree() {
        }

        public int getBaseCount() {
            return baseNodeList.size();
        }

        public BaseNode getBaseNode(int i) {
            return baseNodeList.get(i);
        }

        private void addBaseNode(InternalBaseNode baseNode) {
            baseNodeList.add(baseNode);
        }

    }


    private static abstract class AbstractURINode implements URINode {

        private boolean active = false;
        private Object associatedObject = null;

        public boolean isActive() {
            return active;
        }

        protected void setActive(boolean active) {
            this.active = active;
        }

        public Object getAssociatedObject() {
            return associatedObject;
        }

        protected void setAssociatedObject(Object associatedObject) {
            this.associatedObject = associatedObject;
        }

    }


    private static class InternalBaseNode extends AbstractURINode implements BaseNode {

        private BaseURI baseURI;
        private InternalCorpusNode currentCorpusNode;
        private InternalThesaurusNode currentThesaurusNode;
        private List<InternalCorpusNode> corpusNodeList = new ArrayList<InternalCorpusNode>();
        private List<InternalThesaurusNode> thesaurusNodeList = new ArrayList<InternalThesaurusNode>();

        private InternalBaseNode(BaseURI baseURI) {
            this.baseURI = baseURI;
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

        public int getCorpusCount() {
            return corpusNodeList.size();
        }

        public CorpusNode getCorpusNode(int i) {
            return corpusNodeList.get(i);
        }

        public int getThesaurusCount() {
            return thesaurusNodeList.size();
        }

        public ThesaurusNode getThesaurusNode(int i) {
            return thesaurusNodeList.get(i);
        }

        private void addCorpusNode(InternalCorpusNode corpusNode) {
            corpusNodeList.add(corpusNode);
            currentCorpusNode = corpusNode;
        }

        private void addThesaurusNode(InternalThesaurusNode thesaurusNode) {
            thesaurusNodeList.add(thesaurusNode);
            currentThesaurusNode = thesaurusNode;
        }

    }

    private static class InternalCorpusNode extends AbstractURINode implements CorpusNode {

        private CorpusURI corpusURI;
        private List<InternalFicheNode> ficheNodeList = new ArrayList<InternalFicheNode>();

        private InternalCorpusNode(CorpusURI corpusURI) {
            this.corpusURI = corpusURI;
        }

        public CorpusURI getCorpusURI() {
            return corpusURI;
        }

        public int getFicheCount() {
            return ficheNodeList.size();
        }

        public FicheNode getFicheNode(int i) {
            return ficheNodeList.get(i);
        }

        private void addFicheNode(InternalFicheNode ficheNode) {
            ficheNodeList.add(ficheNode);
        }

    }


    private static class InternalThesaurusNode extends AbstractURINode implements ThesaurusNode {

        private ThesaurusURI thesaurusURI;
        private List<InternalMotcleNode> motcleNodeList = new ArrayList<InternalMotcleNode>();

        private InternalThesaurusNode(ThesaurusURI thesaurusURI) {
            this.thesaurusURI = thesaurusURI;
        }

        public ThesaurusURI getThesaurusURI() {
            return thesaurusURI;
        }

        public int getMotcleCount() {
            return motcleNodeList.size();
        }

        public MotcleNode getMotcleNode(int i) {
            return motcleNodeList.get(i);
        }

        private void addMotcleNode(InternalMotcleNode motcleNode) {
            motcleNodeList.add(motcleNode);
        }

    }


    private static class InternalFicheNode extends AbstractURINode implements FicheNode {

        private FicheURI ficheURI;

        private InternalFicheNode(FicheURI ficheURI) {
            this.ficheURI = ficheURI;
        }

        public FicheURI getFicheURI() {
            return ficheURI;
        }

    }


    private static class InternalMotcleNode extends AbstractURINode implements MotcleNode {

        private MotcleURI motcleURI;

        private InternalMotcleNode(MotcleURI motcleURI) {
            this.motcleURI = motcleURI;
        }

        public MotcleURI getMotcleURI() {
            return motcleURI;
        }

    }

}
