/* ScrutariLib_DB - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.stats;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.primitives.PrimUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.db.tools.util.DataUtils;


/**
 *
 * @author Vincent Calame
 */
public class CountStats {

    private int ficheCount;
    private int motcleCount;
    private int thesaurusCount;
    private int corpusCount;
    private int baseCount;
    private int lexieCount;
    private int indexationCount;
    private Map<Integer, Object> countStatsMap = new HashMap<Integer, Object>();
    private Set<Long> indexationSet = new HashSet<Long>();

    public CountStats() {
    }

    public int getCorpusCount() {
        return corpusCount;
    }

    public int getThesaurusCount() {
        return thesaurusCount;
    }

    public int getMotcleCount() {
        return motcleCount;
    }

    public int getFicheCount() {
        return ficheCount;
    }

    public int getBaseCount() {
        return baseCount;
    }

    public int getLexieCount() {
        return lexieCount;
    }

    public int getIndexationCount() {
        return indexationCount;
    }

    public boolean hasIndexation(int codecorpus, int codethesaurus) {
        long lg = PrimUtils.toLong(codecorpus, codethesaurus);
        return indexationSet.contains(lg);
    }

    public BaseCountStats getBaseCountStats(int codebase) {
        return (BaseCountStats) countStatsMap.get(codebase);
    }

    public ThesaurusCountStats getThesaurusCountStats(int codethesaurus) {
        return (ThesaurusCountStats) countStatsMap.get(codethesaurus);
    }

    public CorpusCountStats getCorpusCountStats(int codecorpus) {
        return (CorpusCountStats) countStatsMap.get(codecorpus);
    }

    public MotcleCountStats getMotcleCountStats(int codemotcle) {
        return (MotcleCountStats) countStatsMap.get(codemotcle);
    }

    public void update(ScrutariDB scrutariDB) {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        corpusCount = dataAccess.getCorpusCount();
        thesaurusCount = dataAccess.getThesaurusCount();
        motcleCount = DataUtils.getMotcleCountByBase(dataAccess, null);
        ficheCount = DataUtils.getFicheCountByBase(dataAccess, null, null);
        baseCount = dataAccess.getBaseCount();
        LexieAccess lexieAccess = scrutariDB.openLexieAccess();
        lexieCount = lexieAccess.getLexieCount();
        lexieAccess.close();
        IndexationInfoMap infoMap = new IndexationInfoMap(dataAccess);
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            updateBase(dataAccess, infoMap, baseData);
        }
        this.indexationSet = infoMap.indexationSet;
        dataAccess.close();
    }

    private void updateBase(DataAccess dataAccess, IndexationInfoMap infoMap, BaseData baseData) {
        BaseCountStats baseCountStats = new BaseCountStats();
        int baseCorpusCount = baseData.getCorpusCount();
        int baseThesaurusCount = baseData.getThesaurusCount();
        int baseFicheCount = 0;
        int baseMotcleCount = 0;
        int baseIndexationCount = 0;
        baseCountStats.setCorpusCount(baseCorpusCount);
        baseCountStats.setThesaurusCount(baseThesaurusCount);
        for (int i = 0; i < baseCorpusCount; i++) {
            int codecorpus = baseData.getCodeCorpus(i);
            CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
            CorpusCountStats corpusCountStats = updateCorpus(infoMap.getIndexationInfo(codecorpus), corpusData);
            baseFicheCount = baseFicheCount + corpusCountStats.getFicheCount();
            baseIndexationCount = baseIndexationCount + corpusCountStats.getIndexationCount();
        }
        for (int i = 0; i < baseThesaurusCount; i++) {
            int codethesaurus = baseData.getCodeThesaurus(i);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
            ThesaurusCountStats thesaurusCountInfo = updateThesaurus(infoMap, thesaurusData);
            baseMotcleCount = baseMotcleCount + thesaurusCountInfo.getMotcleCount();
        }
        baseCountStats.setFicheCount(baseFicheCount);
        baseCountStats.setMotcleCount(baseMotcleCount);
        baseCountStats.setIndexationCount(baseIndexationCount);
        countStatsMap.put(baseData.getCodeBase(), baseCountStats);
        indexationCount = indexationCount + baseIndexationCount;
    }

    private ThesaurusCountStats updateThesaurus(IndexationInfoMap infoMap, ThesaurusData thesaurusData) {
        ThesaurusCountStats thesaurusCountStats = new ThesaurusCountStats();
        int codethesaurus = thesaurusData.getCodeThesaurus();
        int thesaurusMotcleCount = thesaurusData.getMotcleCount();
        thesaurusCountStats.setMotcleCount(thesaurusMotcleCount);
        IndexationInfo info = infoMap.getIndexationInfo(codethesaurus);
        thesaurusCountStats.setIndexationCount(info.getIndexationCount());
        thesaurusCountStats.setFicheCount(info.getIndexedCount());
        for (int i = 0; i < thesaurusMotcleCount; i++) {
            int codemotcle = thesaurusData.getCodeMotcle(i);
            MotcleCountStats motcleCountStats = new MotcleCountStats();
            IndexationInfo infoMotcle = infoMap.getIndexationInfo(codemotcle);
            motcleCountStats.setFicheCount(infoMotcle.getIndexationCount());
            countStatsMap.put(codemotcle, motcleCountStats);
        }
        countStatsMap.put(codethesaurus, thesaurusCountStats);
        return thesaurusCountStats;
    }

    private CorpusCountStats updateCorpus(IndexationInfo info, CorpusData corpusData) {
        CorpusCountStats corpusCountStats = new CorpusCountStats();
        int codecorpus = corpusData.getCodeCorpus();
        corpusCountStats.setMaxnumber(corpusData.getComplementMaxNumber());
        corpusCountStats.setFicheCount(corpusData.getFicheCount());
        corpusCountStats.setIndexationCount(info.getIndexationCount());
        corpusCountStats.setMotcleCount(info.getIndexedCount());
        countStatsMap.put(codecorpus, corpusCountStats);
        return corpusCountStats;
    }


    private static class IndexationInfo {

        private int indexationCount = 0;
        private Set<Integer> codeSet;

        IndexationInfo() {
        }

        public int getIndexationCount() {
            return indexationCount;
        }

        public int getIndexedCount() {
            if (codeSet == null) {
                return 0;
            }
            return codeSet.size();
        }

        private void incrementCount() {
            indexationCount++;
        }

        private void addIndexedCode(int code) {
            if (codeSet == null) {
                codeSet = new HashSet<Integer>();
            }
            codeSet.add(code);
        }

    }


    private static class IndexationInfoMap {

        private Map<Integer, IndexationInfo> map = new HashMap<Integer, IndexationInfo>();
        private Set<Long> indexationSet = new HashSet<Long>();

        private IndexationInfoMap(DataAccess dataAccess) {
            int thesaurusCount = dataAccess.getThesaurusCount();
            CodeList corpusCodeList = dataAccess.getCorpusCodeList();
            int corpusCount = corpusCodeList.getCodeCount();
            for (int i = 0; i < thesaurusCount; i++) {
                ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
                int codethesaurus = thesaurusData.getCodeThesaurus();
                int motcleCount = thesaurusData.getMotcleCount();
                for (int j = 0; j < motcleCount; j++) {
                    int codemotcle = thesaurusData.getCodeMotcle(j);
                    for (int k = 0; k < corpusCount; k++) {
                        int codecorpus = corpusCodeList.getCode(k);
                        IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
                        int ficheCount = indexationCodeList.getCodeCount();
                        if (ficheCount > 0) {
                            indexationSet.add(PrimUtils.toLong(codecorpus, codethesaurus));
                            for (int l = 0; l < ficheCount; l++) {
                                int codefiche = indexationCodeList.getCode(l);
                                IndexationInfo corpusInfo = getOrCreate(codecorpus);
                                IndexationInfo thesaurusInfo = getOrCreate(codethesaurus);
                                IndexationInfo motcleInfo = getOrCreate(codemotcle);
                                IndexationInfo ficheInfo = getOrCreate(codemotcle);
                                motcleInfo.incrementCount();
                                ficheInfo.incrementCount();
                                corpusInfo.incrementCount();
                                thesaurusInfo.incrementCount();
                                corpusInfo.addIndexedCode(codemotcle);
                                thesaurusInfo.addIndexedCode(codefiche);
                            }
                        }
                    }
                }
            }
        }

        public IndexationInfo getIndexationInfo(int code) {
            IndexationInfo indexationInfo = map.get(code);
            if (indexationInfo == null) {
                return new IndexationInfo();
            }
            return indexationInfo;
        }

        private IndexationInfo getOrCreate(int code) {
            IndexationInfo indexationInfo = map.get(code);
            if (indexationInfo == null) {
                indexationInfo = new IndexationInfo();
                map.put(code, indexationInfo);
            }
            return indexationInfo;
        }

    }

}
