/* ScrutariLib_DB - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.MotcleNode;
import net.scrutari.datauri.tree.ThesaurusNode;
import net.scrutari.datauri.tree.URINode;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.datauri.tree.URITreeBuilder;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.codes.ScrutariDataURICodeList;


/**
 *
 * @author Vincent Calame
 */
public class DataCoderBuilder {

    private Set<Integer> usedCodeSet = new HashSet<Integer>();
    private Map<ScrutariDataURI, Integer> uriMap = new LinkedHashMap<ScrutariDataURI, Integer>();
    int maxInitialCode = 0;

    private DataCoderBuilder() {
    }

    public static DataCoder newInstance(ScrutariDataURICodeList initialCodeList, URITree initialURITree) {
        DataCoderBuilder builder = new DataCoderBuilder();
        builder.add(initialCodeList);
        builder.add(initialURITree);
        return new InternalDataCoder(builder.uriMap, builder.maxInitialCode + 1);
    }

    private void add(ScrutariDataURICodeList codeList) {
        if (codeList == null) {
            return;
        }
        int codeCount = codeList.getCodeCount();
        for (int i = 0; i < codeCount; i++) {
            int code = codeList.getCode(i);
            ScrutariDataURI scrutariDataURI = codeList.getScrutariDataURI(i);
            add(scrutariDataURI, code);
        }
    }

    private void add(URITree uriTree) {
        if (uriTree == null) {
            return;
        }
        int baseCount = uriTree.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseNode baseNode = uriTree.getBaseNode(i);
            add(baseNode.getBaseURI(), baseNode);
            int corpusCount = baseNode.getCorpusCount();
            for (int j = 0; j < corpusCount; j++) {
                CorpusNode corpusNode = baseNode.getCorpusNode(j);
                add(corpusNode.getCorpusURI(), corpusNode);
                int ficheCount = corpusNode.getFicheCount();
                for (int k = 0; k < ficheCount; k++) {
                    FicheNode ficheNode = corpusNode.getFicheNode(k);
                    add(ficheNode.getFicheURI(), ficheNode);
                }
            }
            int thesaurusCount = baseNode.getThesaurusCount();
            for (int j = 0; j < thesaurusCount; j++) {
                ThesaurusNode thesaurusNode = baseNode.getThesaurusNode(j);
                add(thesaurusNode.getThesaurusURI(), thesaurusNode);
                int motcleCount = thesaurusNode.getMotcleCount();
                for (int k = 0; k < motcleCount; k++) {
                    MotcleNode motcleNode = thesaurusNode.getMotcleNode(k);
                    add(motcleNode.getMotcleURI(), motcleNode);
                }
            }
        }
    }

    private void add(ScrutariDataURI scrutariDataURI, URINode uriNode) {
        if (!uriNode.isActive()) {
            return;
        }
        Object obj = uriNode.getAssociatedObject();
        if (obj == null) {
            return;
        }
        int code = 0;
        if (obj instanceof Integer) {
            code = ((Integer) obj).intValue();
        } else if (obj instanceof String) {
            try {
                code = Integer.parseInt((String) obj);
            } catch (NumberFormatException nfe) {
            }
        }
        if (code > 0) {
            add(scrutariDataURI, code);
        }
    }

    private void add(ScrutariDataURI scrutariDataURI, int code) {
        if ((!uriMap.containsKey(scrutariDataURI)) && (!usedCodeSet.contains(code))) {
            maxInitialCode = Math.max(maxInitialCode, code);
            uriMap.put(scrutariDataURI, code);
            usedCodeSet.add(code);
        }
    }


    private static class InternalDataCoder implements DataCoder {

        private Map<ScrutariDataURI, Integer> uriMap;
        private int maxCode = 1;
        private Map<ScrutariDataURI, Integer> newUriMap = new LinkedHashMap<ScrutariDataURI, Integer>();

        private InternalDataCoder(Map<ScrutariDataURI, Integer> uriMap, int maxCode) {
            this.uriMap = uriMap;
            this.maxCode = maxCode;
        }

        public int getCode(ScrutariDataURI scrutariDataURI) {
            Integer itg = uriMap.get(scrutariDataURI);
            if (itg != null) {
                return itg.intValue();
            }
            int newCode = maxCode;
            maxCode++;
            uriMap.put(scrutariDataURI, newCode);
            newUriMap.put(scrutariDataURI, newCode);
            return newCode;
        }

        public int checkCode(ScrutariDataURI scrutariDataURI) {
            Integer itg = uriMap.get(scrutariDataURI);
            if (itg == null) {
                return -1;
            }
            return itg.intValue();
        }

        public boolean hasNewURI() {
            return (!newUriMap.isEmpty());
        }

        public URITree toURITree() {
            URITreeBuilder builder = new URITreeBuilder();
            for (Map.Entry<ScrutariDataURI, Integer> entry : uriMap.entrySet()) {
                builder.addScrutariDataURI(entry.getKey(), entry.getValue());
            }
            return builder.toURITree();
        }

        public URITree getNewURITree() {
            if (newUriMap.isEmpty()) {
                return null;
            }
            URITreeBuilder builder = new URITreeBuilder();
            for (Map.Entry<ScrutariDataURI, Integer> entry : newUriMap.entrySet()) {
                builder.addScrutariDataURI(entry.getKey(), entry.getValue());
            }
            return builder.toURITree();
        }

    }

}
