/* ScrutariLib_Data - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.datauri.FicheURI;


/**
 *
 * @author Vincent Calame
 */
public final class FicheData implements Serializable {

    private final int codefiche;
    private final int codecorpus;
    private final int codebase;
    private final int langInteger;
    private final int dateInteger;
    private final String titre;
    private final String soustitre;
    private final String hrefPart1;
    private final String hrefPart2;
    private final String[] complements;
    private final FicheURI ficheURI;
    private final String ficheIcon;
    private final String latitude;
    private final String longitude;
    private final AttributeHolder attributeHolder;

    private FicheData(FicheURI ficheURI, int codefiche, int codecorpus, int codebase, int langInteger, int dateInteger, String titre, String soustitre, String hrefPart1, String hrefPart2, String[] complements, String ficheIcon, String latitude, String longitude, AttributeHolder attributeHolder) {
        this.codecorpus = codecorpus;
        this.codefiche = codefiche;
        this.codebase = codebase;
        this.langInteger = langInteger;
        this.dateInteger = dateInteger;
        this.titre = titre;
        this.soustitre = soustitre;
        this.hrefPart1 = hrefPart1;
        this.hrefPart2 = hrefPart2;
        this.complements = complements;
        this.ficheURI = ficheURI;
        this.ficheIcon = ficheIcon;
        this.latitude = latitude;
        this.longitude = longitude;
        this.attributeHolder = attributeHolder;
    }

    public int getCodeFiche() {
        return codefiche;
    }

    public int getCodeCorpus() {
        return codecorpus;
    }

    public int getCodeBase() {
        return codebase;
    }

    public int getLangInteger() {
        return langInteger;
    }

    public int getComplementMaxNumber() {
        if (complements == null) {
            return 0;
        }
        return complements.length;
    }

    public FicheURI getFicheURI() {
        return ficheURI;
    }

    /**
     * Peut avoir la valeur nulle mais n'est pas de longueur nulle ou composé
     * d'espace blanc.
     */
    public String getTitre() {
        return titre;
    }

    /**
     * Peut avoir la valeur nulle mais n'est pas de longueur nulle ou composé
     * d'espace blanc.
     */
    public String getSoustitre() {
        return soustitre;
    }

    public String getHref(int langUI) {
        if (hrefPart2 == null) {
            return hrefPart1;
        }
        StringBuilder buf = new StringBuilder(hrefPart1.length() + hrefPart2.length() + 10);
        buf.append(hrefPart1);
        LangInteger.appendString(buf, langUI);
        buf.append(hrefPart2);
        return buf.toString();
    }

    public String getRawHref() {
        if (hrefPart2 == null) {
            return hrefPart1;
        }
        return hrefPart1 + "$LANGUI" + hrefPart2;
    }

    public int getDateInteger() {
        return dateInteger;
    }

    public String getFicheIcon() {
        return ficheIcon;
    }

    public String getLatitude() {
        return latitude;
    }

    public String getLongitude() {
        return longitude;
    }

    /**
     * Attention, les compléments sont numérotés à partir de 1.
     */
    public String getComplementByNumber(int num) {
        if (complements == null) {
            throw new IndexOutOfBoundsException();
        }
        return complements[num - 1];
    }

    public AttributeHolder getAttributHolder() {
        return attributeHolder;
    }

    public static FicheData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        int codefiche = primitivesReader.readInt();
        int codecorpus = primitivesReader.readInt();
        int codebase = primitivesReader.readInt();
        int langInteger = primitivesReader.readInt();
        int dateInteger = primitivesReader.readInt();
        FicheURI ficheURI = (FicheURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        String titre = primitivesReader.readString();
        String soustitre = primitivesReader.readString();
        String hrefPart1 = primitivesReader.readString();
        String hrefPart2 = primitivesReader.readString();
        String ficheIcon = primitivesReader.readString();
        String latitude = primitivesReader.readString();
        String longitude = primitivesReader.readString();
        String[] complements = null;
        int length = primitivesReader.readInt();
        if (length > 0) {
            complements = new String[length];
            for (int i = 0; i < length; i++) {
                complements[i] = primitivesReader.readString();
            }
        }
        AttributeHolder attributeHolder = ScrutariAttributes.fromPrimitives(primitivesReader);
        return new FicheData(ficheURI, codefiche, codecorpus, codebase, langInteger, dateInteger, titre, soustitre, hrefPart1, hrefPart2, complements, ficheIcon, latitude, longitude, attributeHolder);
    }

    public static int getLangIntegerFromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        primitivesReader.readInt();
        primitivesReader.readInt();
        primitivesReader.readInt();
        return primitivesReader.readInt();
    }

    public static int getDateIntegerFromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        primitivesReader.readInt();
        primitivesReader.readInt();
        primitivesReader.readInt();
        primitivesReader.readInt();
        return primitivesReader.readInt();
    }

    public static void toPrimitives(FicheData ficheData, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeInt(ficheData.codefiche);
        primitivesWriter.writeInt(ficheData.codecorpus);
        primitivesWriter.writeInt(ficheData.codebase);
        primitivesWriter.writeInt(ficheData.langInteger);
        primitivesWriter.writeInt(ficheData.dateInteger);
        URIPrimitivesUtils.writeScrutariDataURI(ficheData.ficheURI, primitivesWriter);
        primitivesWriter.writeString(ficheData.titre);
        primitivesWriter.writeString(ficheData.soustitre);
        primitivesWriter.writeString(ficheData.hrefPart1);
        primitivesWriter.writeString(ficheData.hrefPart2);
        primitivesWriter.writeString(ficheData.ficheIcon);
        primitivesWriter.writeString(ficheData.latitude);
        primitivesWriter.writeString(ficheData.longitude);
        if (ficheData.complements == null) {
            primitivesWriter.writeInt(-1);
        } else {
            int length = ficheData.complements.length;
            primitivesWriter.writeInt(length);
            for (int i = 0; i < length; i++) {
                primitivesWriter.writeString(ficheData.complements[i]);
            }
        }
        ScrutariAttributes.toPrimitives(ficheData.attributeHolder, primitivesWriter);
    }


    public static class Builder {

        private final FicheURI ficheURI;
        private final List<String> complementsList;
        private int langInteger = LangInteger.UNDETERMINED;
        private int dateInteger = DateInteger.NULL;
        private int complementMaxNumber = 0;
        private String titre;
        private String soustitre;
        private String href;
        private String ficheIcon;
        private String latitude;
        private String longitude;
        private AppendableAttributeHolderBuilder attributeHolderBuilder;

        /**
         * Construit une instance de Builder pour la fiche d'URI
         * <code>ficheURI</code> en précisant le nombre maximum de champs
         * complémentaires qu'est susceptible de contenir la fiche.
         */
        public Builder(FicheURI ficheURI, int complementMaxNumber) {
            this.ficheURI = ficheURI;
            this.complementMaxNumber = complementMaxNumber;
            if (complementMaxNumber > 0) {
                this.complementsList = new ArrayList<String>();
            } else {
                this.complementsList = null;
            }
        }

        /**
         * Modifie l'entier de la langue de la fiche
         */
        public void setLangInteger(int langInteger) {
            this.langInteger = langInteger;
        }

        /**
         * Modifie l'entier de la date de la fiche.
         */
        public void setDateInteger(int dateInteger) {
            this.dateInteger = dateInteger;
        }

        /**
         * Modifie le titre de la fiche.
         */
        public void setTitre(String titre) {
            this.titre = checkString(titre);
        }

        /**
         * Modifie le sous-titre de la fiche
         */
        public void setSoustitre(String soustitre) {
            this.soustitre = checkString(soustitre);
        }

        /**
         * Modifie l'hyperlien de la fiche.
         */
        public void setHref(String href) {
            this.href = checkString(href);
        }

        public void setFicheIcon(String ficheIcon) {
            this.ficheIcon = checkString(ficheIcon);
        }

        public void setGeoloc(String latitude, String longitude) {
            latitude = checkString(latitude);
            longitude = checkString(longitude);
            if ((latitude != null) && (longitude != null)) {
                this.latitude = latitude;
                this.longitude = longitude;
            }
        }

        /**
         * Retourne l'entier de la langue de la fiche.
         */
        public int getLangInteger() {
            return langInteger;
        }

        /**
         * Retourne l'entier de la date de la fiche.
         */
        public int getDateInteger() {
            return dateInteger;
        }

        /**
         * Retourne l'URI de la fiche.
         */
        public FicheURI getFicheURI() {
            return ficheURI;
        }

        /**
         * Ajoute la valeur d'un champ complémentaire. L'ordre d'ajout des
         * champs détermine l'ordre final des champs dans l'instance de
         * FicheData issue de l'objet FicheInfoBuffer. Cette méthode retourne
         * false si le nombre maximum de champs est atteint.
         */
        public boolean addComplement(String s) {
            if (complementMaxNumber == 0) {
                return false;
            }
            if (complementsList.size() == complementMaxNumber) {
                return false;
            }
            complementsList.add(checkString(s));
            return true;
        }

        public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
            if (attributeHolderBuilder == null) {
                attributeHolderBuilder = new AppendableAttributeHolderBuilder();
            }
            attributeHolderBuilder.addAttribute(attributeKey, valueList);
        }

        /**
         * Construit une instance de FicheData à partir des données de l'objet
         * Builder.
         */
        public FicheData toFicheData(int codefiche, String hrefParent, int codecorpus, int codebase) {
            if ((href != null) && (hrefParent != null)) {
                if ((!(href.startsWith("http://"))) || (!(href.startsWith("https://")))) {
                    href = hrefParent + href;
                }
            }
            String hrefPart1 = null;
            String hrefPart2 = null;
            if (href != null) {
                int idx = href.indexOf("$LANGUI");
                if (idx == -1) {
                    hrefPart1 = href;
                    hrefPart2 = null;
                } else {
                    hrefPart1 = href.substring(0, idx);
                    hrefPart2 = href.substring(idx + 7);
                }
            }
            String[] complements = null;
            if (complementMaxNumber > 0) {
                complements = new String[complementMaxNumber];
                for (int i = 0; i < complementsList.size(); i++) {
                    complements[i] = complementsList.get(i);
                }
            }
            AttributeHolder attributeHolder = (attributeHolderBuilder != null) ? attributeHolderBuilder.toAttributeHolder() : AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
            return new FicheData(ficheURI, codefiche, codecorpus, codebase, langInteger, dateInteger, titre, soustitre, hrefPart1, hrefPart2, complements, ficheIcon, latitude, longitude, attributeHolder);
        }

        private String checkString(String s) {
            if (s == null) {
                return null;
            }
            s = StringUtils.cleanString(s);
            if (s.length() == 0) {
                return null;
            }
            return s;
        }

    }

}
