/* @ScrutariLib_Data - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefBuilder;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.CheckedKeyString;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public final class ScrutariAttributes {

    public final static String INLINE_TYPE = "inline";
    public final static String LIST_TYPE = "list";
    public final static String BLOCK_TYPE = "block";
    public final static CheckedKeyString SCT_NAMESPACE = CheckedKeyString.build("sct");
    public final static AttributeKey AUTHORS_KEY = AttributeKey.build(SCT_NAMESPACE, "authors");
    private final static Map<String, AttributeKey> attributeKeyMap = new HashMap<String, AttributeKey>();
    private final static Map<AttributeKey, AttributeDef> defaultDefMap = new HashMap<AttributeKey, AttributeDef>();

    static {
        initAttributeDef(AUTHORS_KEY, INLINE_TYPE);
    }

    private ScrutariAttributes() {
    }

    public static AttributeKey toAttributeKey(String nameSpace, String localKey) throws ParseException {
        String attributeKeyString = nameSpace + ":" + localKey;
        AttributeKey attributeKey = attributeKeyMap.get(attributeKeyString);
        if (attributeKey == null) {
            attributeKey = AttributeKey.parse(attributeKeyString);
            synchronized (attributeKeyMap) {
                attributeKeyMap.put(attributeKeyString, attributeKey);
            }
        }
        return attributeKey;
    }

    public static boolean isValidFormatType(String formatType) {
        if (formatType.length() == 0) {
            return true;
        }
        if (formatType.equals(INLINE_TYPE)) {
            return true;
        }
        if (formatType.equals(LIST_TYPE)) {
            return true;
        }
        if (formatType.equals(BLOCK_TYPE)) {
            return true;
        }
        return false;
    }

    public static AttributeDef getDefaultAttributeDef(AttributeKey attributeKey) {
        return defaultDefMap.get(attributeKey);
    }

    public static void toPrimitives(AttributeHolder attributeHolder, PrimitivesWriter primitivesWriter) throws IOException {
        int count = attributeHolder.getAttributeCount();
        primitivesWriter.writeInt(count);
        for (int i = 0; i < count; i++) {
            Attribute attribute = attributeHolder.getAttributeByIndex(i);
            AttributeKey attributeKey = attribute.getAttributeKey();
            primitivesWriter.writeString(attributeKey.toString());
            int valCount = attribute.getValueCount();
            primitivesWriter.writeInt(valCount);
            for (int j = 0; j < valCount; j++) {
                primitivesWriter.writeString(attribute.getValue(j));
            }
        }
    }

    public static AttributeHolder fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        int count = primitivesReader.readInt();
        if (count == 0) {
            return AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
        }
        if (count == 1) {
            Attribute attribute = readAttribute(primitivesReader);
            return AttributeUtils.toSingletonAttributeHolder(attribute);
        }
        Attribute[] array = new Attribute[count];
        for (int i = 0; i < count; i++) {
            array[i] = readAttribute(primitivesReader);
        }
        return new ArrayAttributeHolder(array);
    }

    private static Attribute readAttribute(PrimitivesReader primitivesReader) throws IOException {
        String attributeKeyString = primitivesReader.readString();
        AttributeKey attributeKey = attributeKeyMap.get(attributeKeyString);
        if (attributeKey == null) {
            synchronized (attributeKeyMap) {
                try {
                    attributeKey = AttributeKey.parse(attributeKeyString);
                    attributeKeyMap.put(attributeKeyString, attributeKey);
                } catch (ParseException pe) {
                    throw new IllegalStateException(pe);
                }
            }
        }
        int valCount = primitivesReader.readInt();
        if (valCount < 1) {
            throw new IllegalStateException("valCount < 1");
        }
        if (valCount == 1) {
            return new SingletonAttribute(attributeKey, primitivesReader.readString());
        }
        String[] array = new String[valCount];
        for (int i = 0; i < valCount; i++) {
            array[i] = primitivesReader.readString();
        }
        return new ArrayAttribute(attributeKey, array);
    }

    private static void initAttributeDef(AttributeKey attributeKey, String formatType) {
        AttributeDefBuilder builder = new AttributeDefBuilder(attributeKey);
        builder.setFormatType(formatType);
        try {
            InputStream is = ScrutariAttributes.class.getResourceAsStream("l10n/" + attributeKey.toString() + ".csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String ligne;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf(',');
                    if (idx != -1) {
                        int langInteger = LangInteger.parse(ligne.substring(0, idx));
                        CleanedString cs = CleanedString.newInstance(ligne.substring(idx + 1));
                        if (cs != null) {
                            builder.addLibelle(LibelleUtils.toLibelle(langInteger, cs));
                        }
                    }
                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch (LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
        defaultDefMap.put(attributeKey, builder.toAttributeDef());
    }


    private static class SingletonAttribute implements Attribute {

        private AttributeKey key;
        private String value;

        private SingletonAttribute(AttributeKey key, String value) {
            this.key = key;
            this.value = value;
        }

        @Override
        public AttributeKey getAttributeKey() {
            return key;
        }

        @Override
        public int getValueCount() {
            return 1;
        }

        @Override
        public String getValue(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return value;
        }

    }


    private static class ArrayAttribute implements Attribute {

        private AttributeKey key;
        private String[] valueArray;

        private ArrayAttribute(AttributeKey key, String[] valueArray) {
            this.key = key;
            this.valueArray = valueArray;
        }

        @Override
        public AttributeKey getAttributeKey() {
            return key;
        }

        @Override
        public int getValueCount() {
            return valueArray.length;
        }

        @Override
        public String getValue(int i) {
            return valueArray[i];
        }

    }


    private static class ArrayAttributeHolder implements AttributeHolder {

        private Attribute[] attributeArray;

        private ArrayAttributeHolder(Attribute[] attributeArray) {
            this.attributeArray = attributeArray;
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            int length = attributeArray.length;
            for (int i = 0; i < length; i++) {
                Attribute attribute = attributeArray[i];
                if (attribute.getAttributeKey().equals(key)) {
                    return attribute;
                }
            }
            return null;
        }

        @Override
        public int getAttributeCount() {
            return attributeArray.length;
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeArray[i];
        }

    }

}
