/* ScrutariLib_SearchEngine - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine;

import java.util.List;
import net.scrutari.data.CorpusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.tools.util.DataUtils;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.pertinence.FichePertinence;
import net.scrutari.searchengine.api.pertinence.MotclePertinence;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.seeker.SeekOptions;
import net.scrutari.searchengine.tools.SearchFilter;
import net.scrutari.searchengine.tools.operands.search.SearchOperationUtils;
import net.scrutari.searchengine.tools.pertinence.PertinenceUtils;
import net.scrutari.searchengine.tools.result.FicheSearchResultBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchSourceBuilder;
import net.scrutari.searchengine.tools.result.MotcleSearchResultBuilder;
import net.scrutari.searchengine.tools.seeker.SeekOptionsBuilder;
import net.scrutari.searchengine.tools.seeker.lexie.LexieSeeker;


/**
 *
 * @author Vincent Calame
 */
public final class FicheSearchEngine {

    private FicheSearchEngine() {
    }

    public static FicheSearchResult search(ScrutariDB scrutariDB, SearchOperation searchOperation, SearchOptions searchOptions, GlobalSearchOptions globalSearchOptions, QId qId) {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        FicheSearchResult ficheSearchResult = search(scrutariDB, dataAccess, searchOperation, searchOptions, globalSearchOptions, qId);
        dataAccess.close();
        return ficheSearchResult;
    }

    public static FicheSearchResult search(ScrutariDB scrutariDB, DataAccess dataAccess, SearchOperation searchOperation, SearchOptions searchOptions, GlobalSearchOptions globalSearchOptions, QId qId) {
        SeekOptions seekOptions = SeekOptionsBuilder.build(scrutariDB, dataAccess, searchOptions, true, true);
        int operandMaxNumber = searchOperation.getSearchTokenOperandMaxNumber();
        int[] acceptOperandArray = SearchOperationUtils.toAcceptOperandArray(searchOperation);
        SearchEngine searchEngine = new SearchEngine(operandMaxNumber, acceptOperandArray);
        LexieAccess lexieAccess = scrutariDB.openLexieAccess();
        LexieSeeker lexieSeeker = new LexieSeeker(seekOptions, searchEngine, lexieAccess);
        lexieSeeker.seek(searchOperation);
        lexieAccess.close();
        if (seekOptions.isWithMotscles()) {
            searchEngine.checkMotclePertinence(dataAccess, seekOptions);
        }
        return toResult(searchOperation, searchOptions, globalSearchOptions, qId, dataAccess, seekOptions, searchEngine);
    }

    private static FicheSearchResult toResult(SearchOperation searchOperation, SearchOptions searchOptions, GlobalSearchOptions globalSearchOptions, QId qId, DataAccess dataAccess, SeekOptions seekOptions, SearchEngine searchEngine) {
        int[] acceptOperandArray = SearchOperationUtils.toAcceptOperandArray(searchOperation);
        FicheSort engine = new FicheSort(globalSearchOptions, acceptOperandArray, searchOptions.getPertinencePonderation(), dataAccess);
        FicheSearchResultBuilder builder = new FicheSearchResultBuilder(FicheSearchSourceBuilder.build(qId, searchOperation.getQInfo(), searchOptions));
        int[] langIntegerArray = searchOptions.getLangIntegerArray();
        builder.setFicheMaximum(DataUtils.getFicheCountByCorpus(dataAccess, seekOptions.getCodecorpusArray(), langIntegerArray));
        SearchFilter searchFilter = new SearchFilter(searchOperation);
        if (SearchOperationUtils.hasMandatoryOperand(searchOperation)) {
            List<FichePertinence> list = searchEngine.getFichePertinenceList();
            for (FichePertinence fichePertinence : list) {
                if (searchFilter.accept(fichePertinence)) {
                    int dateInteger = dataAccess.getIntValue(DataAccess.FICHE_DATE_INTVALUE, fichePertinence.getCodefiche());
                    engine.addFichePertinence(fichePertinence, dateInteger);
                }
            }
        } else {
            int[] codecorpusArray = seekOptions.getCodecorpusArray();
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                int codecorpus = codecorpusArray[i];
                CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
                int ficheCount = corpusData.getFicheCount();
                for (int j = 0; j < ficheCount; j++) {
                    int codefiche = corpusData.getCodeFiche(j);
                    if (seekOptions.acceptFiche(codefiche)) {
                        FichePertinence fichePertinence = searchEngine.getFichePertinence(codefiche);
                        if ((fichePertinence == null) || (searchFilter.accept(fichePertinence))) {
                            int dateInteger = dataAccess.getIntValue(DataAccess.FICHE_DATE_INTVALUE, codefiche);
                            if (fichePertinence == null) {
                                engine.addFichePertinence(codecorpus, codefiche, dateInteger);
                            } else {
                                engine.addFichePertinence(fichePertinence, dateInteger);
                            }
                        }
                    }
                }
            }
        }
        engine.initFicheSearchResultGroup(builder);
        List<MotclePertinence> mtclPertList = searchEngine.getMotclePertinenceList();
        builder.setMotcleCount(mtclPertList.size());
        for (MotclePertinence mtclPert : mtclPertList) {
            InLangOccurrenceList[] array = PertinenceUtils.getInLangOccurrenceListArray(mtclPert, acceptOperandArray);
            builder.addMotcleSearchResultInfo(MotcleSearchResultBuilder.toMotcleSearchResultInfo(mtclPert.getCodemotcle(), array));
        }
        return builder.toFicheSearchResult();
    }

}
