/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.SubOperand;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.operands.search.SearchTokenOperand;
import net.scrutari.searchengine.api.pertinence.FichePertinence;


/**
 *
 * @author Vincent Calame
 */
public class SearchFilter {

    private final static int[] EMPTY_INTARRAY = new int[0];
    private final static OperandTest[] EMPTY_TESTARRAY = new OperandTest[0];
    private SearchOperation searchOperation;
    private OperandTest rootOperandTest;

    public SearchFilter(SearchOperation searchOperation) {
        this.searchOperation = searchOperation;
        this.rootOperandTest = init(searchOperation);
    }

    public boolean accept(FichePertinence fichePertinence) {
        try {
            return rootOperandTest.test(fichePertinence);
        } catch(NoPertinentException npe) {
            return false;
        }
    }

    public SearchOperation getSearchOperation() {
        return searchOperation;
    }

    private OperandTest init(SearchOperation searchOperation) {
        Operand rootOperand = searchOperation.getRootOperand();
        if (rootOperand instanceof SearchTokenOperand) {
            SearchTokenOperand searchTokenOperand = (SearchTokenOperand) rootOperand;
            int[] array = new int[1];
            array[0] = searchTokenOperand.getOperandNumber();
            if (searchTokenOperand.isAcceptMode()) {
                return new OperandTest(false, EMPTY_TESTARRAY, array, EMPTY_INTARRAY);
            } else {
                return new OperandTest(false, EMPTY_TESTARRAY, EMPTY_INTARRAY, array);
            }
        } else {
            return init((SubOperand) rootOperand);
        }
    }

    private OperandTest init(SubOperand subOperand) {
        boolean isUnionMode = (subOperand.getOperator() == LogicalOperationConstants.UNION_OPERATOR);
        OperandTestBuilder builder = new OperandTestBuilder(isUnionMode);
        int count = subOperand.getOperandCount();
        for (int i = 0; i < count; i++) {
            Operand operand = subOperand.getOperand(i);
            if (operand instanceof SearchTokenOperand) {
                SearchTokenOperand searchTokenOperand = (SearchTokenOperand) operand;
                int operandNumber = searchTokenOperand.getOperandNumber();
                if (searchTokenOperand.isAcceptMode()) {
                    builder.addAccept(operandNumber);
                } else {
                    builder.addReject(operandNumber);
                }
            } else {
                builder.addOperandTest(init((SubOperand) operand));
            }
        }
        return builder.toOperandTest();
    }


    private static class OperandTestBuilder {

        private boolean unionMode;
        private List<OperandTest> subList = new ArrayList<OperandTest>();
        private List<Integer> acceptList = new ArrayList<Integer>();
        private List<Integer> rejectList = new ArrayList<Integer>();

        private OperandTestBuilder(boolean unionMode) {
            this.unionMode = unionMode;
        }

        private void addOperandTest(OperandTest operandTest) {
            subList.add(operandTest);
        }

        private void addReject(int operandNumber) {
            rejectList.add(operandNumber);
        }

        private void addAccept(int operandNumber) {
            acceptList.add(operandNumber);
        }

        private OperandTest toOperandTest() {
            int[] acceptOperandNumberArray;
            int acceptSize = acceptList.size();
            if (acceptSize > 0) {
                acceptOperandNumberArray = new int[acceptSize];
                for (int i = 0; i < acceptSize; i++) {
                    acceptOperandNumberArray[i] = acceptList.get(i);
                }
            } else {
                acceptOperandNumberArray = EMPTY_INTARRAY;
            }
            int[] rejectOperandNumberArray;
            int rejectSize = rejectList.size();
            if (rejectSize > 0) {
                rejectOperandNumberArray = new int[rejectSize];
                for (int i = 0; i < rejectSize; i++) {
                    rejectOperandNumberArray[i] = rejectList.get(i);
                }
            } else {
                rejectOperandNumberArray = EMPTY_INTARRAY;
            }
            OperandTest[] subTestArray;
            int subSize = subList.size();
            if (subSize > 0) {
                subTestArray = subList.toArray(new OperandTest[subSize]);
            } else {
                subTestArray = EMPTY_TESTARRAY;
            }
            return new OperandTest(unionMode, subTestArray, acceptOperandNumberArray, rejectOperandNumberArray);
        }

    }


    private static class OperandTest {

        private boolean unionOperator;
        private OperandTest[] subTestArray;
        private int[] acceptOperandNumberArray;
        private int[] rejectOperandNumberArray;

        private OperandTest(boolean unionOperator, OperandTest[] subTestArray, int[] acceptOperandNumberArray, int[] rejectOperandNumberArray) {
            this.unionOperator = unionOperator;
            this.subTestArray = subTestArray;
            this.acceptOperandNumberArray = acceptOperandNumberArray;
            this.rejectOperandNumberArray = rejectOperandNumberArray;
        }

        private boolean test(FichePertinence fichePertinence) throws NoPertinentException {
            if (unionOperator) {
                return testUnion(fichePertinence);
            } else {
                return testIntersection(fichePertinence);
            }
        }

        private boolean testUnion(FichePertinence fichePertinence) throws NoPertinentException {
            boolean withPertinent = false;
            int acceptLength = acceptOperandNumberArray.length;
            for (int i = 0; i < acceptLength; i++) {
                short state = fichePertinence.getOperandState(acceptOperandNumberArray[i]);
                if (state == FichePertinence.OCCURRENCE_STATE) {
                    return true;
                }
                if (state != FichePertinence.NOPERTINENT_STATE) {
                    withPertinent = true;
                }
            }
            int rejectLength = rejectOperandNumberArray.length;
            for (int i = 0; i < rejectLength; i++) {
                short state = fichePertinence.getOperandState(rejectOperandNumberArray[i]);
                if (state == FichePertinence.NO_OCCURRENCE_STATE) {
                    return true;
                }
                if (state != FichePertinence.NOPERTINENT_STATE) {
                    withPertinent = true;
                }
            }
            int subLength = subTestArray.length;
            for (int i = 0; i < subLength; i++) {
                if (subTestArray[i].test(fichePertinence)) {
                    return true;
                }
            }
            if (!withPertinent) {
                throw new NoPertinentException();
            }
            return false;
        }

        private boolean testIntersection(FichePertinence fichePertinence) throws NoPertinentException {
            boolean withPertinent = false;
            int acceptLength = acceptOperandNumberArray.length;
            for (int i = 0; i < acceptLength; i++) {
                short state = fichePertinence.getOperandState(acceptOperandNumberArray[i]);
                if (state == FichePertinence.NO_OCCURRENCE_STATE) {
                    return false;
                }
                if (state != FichePertinence.NOPERTINENT_STATE) {
                    withPertinent = true;
                }
            }
            int rejectLength = rejectOperandNumberArray.length;
            for (int i = 0; i < rejectLength; i++) {
                short state = fichePertinence.getOperandState(rejectOperandNumberArray[i]);
                if (state == FichePertinence.OCCURRENCE_STATE) {
                    return false;
                }
                if (state != FichePertinence.NOPERTINENT_STATE) {
                    withPertinent = true;
                }
            }
            int subLength = subTestArray.length;
            for (int i = 0; i < subLength; i++) {
                try {
                    if (!subTestArray[i].test(fichePertinence)) {
                        return false;
                    }
                    withPertinent = true;
                } catch(NoPertinentException npe) {
                    
                }
            }
            if (!withPertinent) {
                throw new NoPertinentException();
            }
            return true;
        }

    }
    
    private static class NoPertinentException extends Exception {
        
        private NoPertinentException() {
            
        }
    } 

}
