/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.dom;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.logicaloperation.LogicalOperationParser;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.OperandException;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.scrutari.data.FicheData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.ElementCodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.ScrutariDataURIList;
import net.scrutari.searchengine.tools.OperandCheck;
import net.scrutari.searchengine.tools.operands.filters.FilterOperandConverterEngine;
import net.scrutari.searchengine.tools.operands.filters.OperandConverterParameters;
import net.scrutari.searchengine.tools.operands.search.SearchOperationUtils;
import net.scrutari.searchengine.tools.options.OptionsUtils;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchSourceBuilder;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchSourceDOMReader {
    
    private final FicheSearchSourceBuilder ficheSearchSourceBuilder;
    private final OperandConverterParameters operandConverterParameters;
    
    public FicheSearchSourceDOMReader(OperandConverterParameters operandConverterParameters, FicheSearchSourceBuilder ficheSearchSourceBuilder) {
        this.ficheSearchSourceBuilder = ficheSearchSourceBuilder;
        this.operandConverterParameters = operandConverterParameters;
    }
    
    public void readFicheSearchSource(Element element_xml) {
        RootHandler rootHandler = new RootHandler(operandConverterParameters);
        DOMUtils.readChildren(element_xml, rootHandler);
    }
    
    
    private class RootHandler implements ElementHandler {
        
        private final OperandConverterParameters operandConverterParameters;
        
        private RootHandler(OperandConverterParameters operandConverterParameters) {
            this.operandConverterParameters = operandConverterParameters;
        }
        
        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("q-info")) {
                String q = element.getAttribute("q");
                String qSource = element.getAttribute("q-source");
                if ((q.length() > 0) && (qSource.length() > 0)) {
                    try {
                        int qLength = Integer.parseInt(element.getAttribute("q-length"));
                        if (qLength > 0) {
                            ficheSearchSourceBuilder.setQInfo(SearchOperationUtils.toQInfo(q, qSource, qLength));
                        }
                    } catch (NumberFormatException nfe) {
                    }
                }
            } else if (tagName.equals("options")) {
                SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder();
                try {
                    int langUi = LangInteger.parse(element.getAttribute("lang-ui"));
                    searchOptionsBuilder.setLangUi(langUi);
                } catch (LangIntegerException langIntegerException) {
                }
                String defaultSearchType = element.getAttribute("default-search-type");
                if (defaultSearchType.length() > 0) {
                    try {
                        searchOptionsBuilder.setDefaultSearchType(TextConstants.toShort(defaultSearchType));
                    } catch (IllegalArgumentException iae) {
                    }
                }
                SearchOptionsHandler handler = new SearchOptionsHandler(operandConverterParameters, searchOptionsBuilder);
                handler.handle(element);
                ficheSearchSourceBuilder.setSearchOptions(searchOptionsBuilder.toSearchOptions());
            }
        }
        
    }
    
    
    private class SearchOptionsHandler implements ElementHandler {
        
        private final LangsHandler langsHandler = new LangsHandler();
        private final FicheCodeListHandler ficheCodeListHandler;
        private final CodeListHandler corpusCodeListHandler;
        private final CodeListHandler thesaurusCodeListHandler;
        private final OperandConverterParameters operandConverterParameters;
        private final SearchOptionsBuilder searchOptionsBuilder;
        private boolean firstStep = true;
        
        private SearchOptionsHandler(OperandConverterParameters operandConverterParameters, SearchOptionsBuilder searchOptionsBuilder) {
            this.operandConverterParameters = operandConverterParameters;
            DataAccess dataAccess = operandConverterParameters.getDataAccess();
            this.ficheCodeListHandler = new FicheCodeListHandler(dataAccess);
            this.corpusCodeListHandler = new CodeListHandler(dataAccess, ScrutariDataURI.CORPUSURI_TYPE);
            this.thesaurusCodeListHandler = new CodeListHandler(dataAccess, ScrutariDataURI.THESAURUSURI_TYPE);
            this.searchOptionsBuilder = searchOptionsBuilder;
        }
        
        private void handle(Element element) {
            DOMUtils.readChildren(element, this);
            flushFirstStep();
            DOMUtils.readChildren(element, this);
        }
        
        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (firstStep) {
                if (tagName.equals("langs")) {
                    DOMUtils.readChildren(element, langsHandler);
                } else if (tagName.equals("fiche-list")) {
                    DOMUtils.readChildren(element, ficheCodeListHandler);
                } else if (tagName.equals("corpus-list")) {
                    DOMUtils.readChildren(element, corpusCodeListHandler);
                } else if (tagName.equals("thesaurus-list")) {
                    DOMUtils.readChildren(element, thesaurusCodeListHandler);
                } else if (tagName.equals("ponderation")) {
                    int occurrence = 0;
                    int date = 0;
                    int origin = 0;
                    try {
                        occurrence = Integer.parseInt(element.getAttribute("occurrence"));
                    } catch (NumberFormatException nfe) {
                    }
                    try {
                        date = Integer.parseInt(element.getAttribute("date"));
                    } catch (NumberFormatException nfe) {
                    }
                    try {
                        origin = Integer.parseInt(element.getAttribute("origin"));
                    } catch (NumberFormatException nfe) {
                    }
                    searchOptionsBuilder.setPertinencePonderation(PertinencePonderationBuilder.toPertinencePonderation(occurrence, date, origin));
                }
            } else {
                if (tagName.equals("fiche-eligibility")) {
                    OperandCheck operandCheck = toOperandCheck(element);
                    if (operandCheck != null) {
                        searchOptionsBuilder.addFicheOperandCheck(operandCheck);
                    }
                } else if (tagName.equals("motcle-eligibility")) {
                    OperandCheck operandCheck = toOperandCheck(element);
                    if (operandCheck != null) {
                        searchOptionsBuilder.addMotcleOperandCheck(operandCheck);
                    }
                }
            }
            
        }
        
        private void flushFirstStep() {
            searchOptionsBuilder.setLangIntegerArray(langsHandler.toLangIntegerArray());
            ElementCodeList elementCodeList = ficheCodeListHandler.toElementCodeList();
            if (elementCodeList != null) {
                searchOptionsBuilder.reduceFicheList(elementCodeList);
                searchOptionsBuilder.putScrutariDataURIList(ficheCodeListHandler.toScrutariDataURIList());
            }
            CodeList corpusCodeList = corpusCodeListHandler.toCodeList();
            if (corpusCodeList != null) {
                searchOptionsBuilder.reduceCorpusList(corpusCodeList);
                searchOptionsBuilder.putScrutariDataURIList(corpusCodeListHandler.toScrutariDataURIList());
            }
            CodeList thesaurusCodeList = thesaurusCodeListHandler.toCodeList();
            if (thesaurusCodeList != null) {
                searchOptionsBuilder.reduceThesaurusList(thesaurusCodeList);
                searchOptionsBuilder.putScrutariDataURIList(thesaurusCodeListHandler.toScrutariDataURIList());
            }
            firstStep = false;
        }
        
        private OperandCheck toOperandCheck(Element element) {
            String operationString = DOMUtils.readSimpleElement(element);
            try {
                Operand operand = LogicalOperationParser.parseInformatique(operationString);
                if (operand != null) {
                    OperandCheck operandCheck = FilterOperandConverterEngine.convert(operand, searchOptionsBuilder.getCurrentCorpusList(), null, operandConverterParameters);
                    if (!operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
                        return operandCheck;
                    }
                }
            } catch (OperandException pe) {
                return null;
            }
            return null;
        }
        
    }
    
    
    private static class LangsHandler implements ElementHandler {
        
        private final List<Integer> langList = new ArrayList<Integer>();
        
        private LangsHandler() {
        }
        
        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("lang")) {
                try {
                    int langInteger = LangInteger.parse(DOMUtils.readSimpleElement(element));
                    langList.add(langInteger);
                } catch (LangIntegerException nfe) {
                }
            }
        }
        
        private int[] toLangIntegerArray() {
            int size = langList.size();
            if (size == 0) {
                return null;
            }
            int[] array = new int[size];
            for (int i = 0; i < size; i++) {
                array[i] = langList.get(i);
            }
            return array;
        }
        
    }
    
    
    private static class CodeListHandler implements ElementHandler {
        
        private final DataAccess dataAccess;
        private final Set<Integer> codeSet = new LinkedHashSet<Integer>();
        private final List<ScrutariDataURI> list = new ArrayList<ScrutariDataURI>();
        private final short type;
        private boolean done = false;
        
        private CodeListHandler(DataAccess dataAccess, short type) {
            this.dataAccess = dataAccess;
            this.type = type;
        }
        
        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("uri")) {
                try {
                    ScrutariDataURI scrutariDataURI = URIParser.parse(DOMUtils.readSimpleElement(element));
                    if (scrutariDataURI.getType() == type) {
                        done = true;
                        int code = dataAccess.getCode(scrutariDataURI);
                        if (code > 0) {
                            if (!codeSet.contains(code)) {
                                codeSet.add(code);
                                list.add(scrutariDataURI);
                            }
                        }
                    }
                } catch (URIParseException nfe) {
                }
            }
        }
        
        private CodeList toCodeList() {
            if (!done) {
                return null;
            }
            return CodesUtils.toCodeList(codeSet);
        }
        
        private ScrutariDataURIList toScrutariDataURIList() {
            if (!done) {
                return null;
            }
            return OptionsUtils.toScrutariDataURIList(type, list);
        }
        
    }
    
    
    private static class FicheCodeListHandler implements ElementHandler {
        
        private final DataAccess dataAccess;
        private final Map<Integer, Integer> elementMap = new LinkedHashMap<Integer, Integer>();
        private final List<ScrutariDataURI> list = new ArrayList<ScrutariDataURI>();
        private boolean done = false;
        
        private FicheCodeListHandler(DataAccess dataAccess) {
            this.dataAccess = dataAccess;
        }
        
        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("uri")) {
                try {
                    ScrutariDataURI scrutariDataURI = URIParser.parse(DOMUtils.readSimpleElement(element));
                    if (scrutariDataURI.getType() == ScrutariDataURI.FICHEURI_TYPE) {
                        done = true;
                        int code = dataAccess.getCode(scrutariDataURI);
                        if (code > 0) {
                            if (!elementMap.containsKey(code)) {
                                FicheData ficheData = dataAccess.getFicheDataByCode(code);
                                elementMap.put(ficheData.getCodeFiche(), ficheData.getCodeCorpus());
                                list.add(scrutariDataURI);
                            }
                        }
                    }
                } catch (URIParseException nfe) {
                }
            }
        }
        
        private ElementCodeList toElementCodeList() {
            if (!done) {
                return null;
            }
            return CodesUtils.toElementCodeList(elementMap);
        }
        
        private ScrutariDataURIList toScrutariDataURIList() {
            if (!done) {
                return null;
            }
            return OptionsUtils.toScrutariDataURIList(ScrutariDataURI.FICHEURI_TYPE, list);
        }
        
    }
    
}
