/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.filters;

import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.primitives.DateIntegerFilter;
import net.mapeadores.util.primitives.DateIntegerFilterFactory;
import net.scrutari.data.FicheData;
import net.scrutari.searchengine.api.operands.OperandMessageKeys;
import net.scrutari.searchengine.api.operands.OperandsConstants;
import net.scrutari.searchengine.api.operands.filters.FicheDataOperand;
import net.scrutari.searchengine.tools.OperandCheck;


/**
 *
 * @author Vincent Calame
 */
class DateCheck {

    private final static FicheDataOperand NULL_DATEOPERAND = new NullDateOperand(true);
    private final static FicheDataOperand NOTNULL_DATEOPERAND = new NullDateOperand(false);

    private DateCheck() {
    }

    static OperandCheck check(boolean acceptMode, String operandBody, String operandString, OperandConverterParameters operandConverterParameters) {
        FicheDataOperand operand = toOperand(acceptMode, operandBody, operandString, operandConverterParameters);
        if (operand == null) {
            return OperandCheck.UNDEFINED_OPERANDECHECK;
        }
        return new OperandCheck(null, operand);
    }

    static FicheDataOperand toOperand(boolean acceptMode, String operandBody, String operandString, OperandConverterParameters operandConverterParameters) {
        if (operandBody.equals("NULL")) {
            if (acceptMode) {
                return NULL_DATEOPERAND;
            } else {
                return NOTNULL_DATEOPERAND;
            }
        }
        ProcessMessageHandler processMessageHandler = operandConverterParameters.getProcessMessageHandler();
        DateIntegerFilter dateIntegerFilter = null;
        int idx = operandBody.indexOf('/');
        if (idx == - 1) {
            try {
                int dateInteger = DateInteger.parse(operandBody);
                dateIntegerFilter = DateIntegerFilterFactory.newInstance(dateInteger);
            } catch (DateIntegerException die) {
                processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.MALFORMED_DATE, operandString);
            }
        } else {
            String token1 = operandBody.substring(0, idx).trim();
            String token2 = operandBody.substring(idx + 1).trim();
            if (token1.equals("-")) {
                if (token2.equals("-")) {
                    if (!acceptMode) {
                        return NULL_DATEOPERAND;
                    } else {
                        return NOTNULL_DATEOPERAND;
                    }
                } else {
                    try {
                        int dateInteger = DateInteger.parse(token2);
                        dateIntegerFilter = DateIntegerFilterFactory.newMaximumFilter(dateInteger);
                    } catch (DateIntegerException die) {
                        processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.MALFORMED_DATE, operandString);
                    }
                }
            } else if (token2.equals("-")) {
                try {
                    int dateInteger = DateInteger.parse(token1);
                    dateIntegerFilter = DateIntegerFilterFactory.newMinimumFilter(dateInteger);
                } catch (DateIntegerException die) {
                    processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.MALFORMED_DATE, operandString);
                }
            } else {
                try {
                    int date1 = DateInteger.parse(token1);
                    int date2 = DateInteger.parse(token2);
                    dateIntegerFilter = DateIntegerFilterFactory.newInstance(date1, date2);
                } catch (DateIntegerException die) {
                    processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.MALFORMED_DATE, operandString);
                }
            }
        }
        if (dateIntegerFilter != null) {
            return new DateOperand(dateIntegerFilter, acceptMode, operandBody);
        } else {
            return null;
        }
    }


    private static class NullDateOperand implements FicheDataOperand {

        private final boolean acceptMode;

        private NullDateOperand(boolean acceptMode) {
            this.acceptMode = acceptMode;
        }

        @Override
        public boolean acceptFicheData(FicheData ficheData) {
            if (ficheData == null) {
                return acceptMode;
            }
            if (ficheData.getDateInteger() == DateInteger.NULL) {
                return acceptMode;
            } else {
                return !acceptMode;
            }
        }

        @Override
        public String toString() {
            if (acceptMode) {
                return "NULL";
            } else {
                return "!NULL";
            }
        }

    }


    private static class DateOperand implements FicheDataOperand {

        private final boolean acceptMode;
        private final DateIntegerFilter filter;
        private final String operandBody;

        private DateOperand(DateIntegerFilter filter, boolean acceptMode, String operandBody) {
            this.filter = filter;
            this.acceptMode = acceptMode;
            this.operandBody = operandBody;
        }

        @Override
        public boolean acceptFicheData(FicheData ficheData) {
            if (ficheData == null) {
                return !acceptMode;
            }
            int dateInteger = ficheData.getDateInteger();
            boolean test = filter.containsDateInteger(dateInteger);
            if (!acceptMode) {
                test = !test;
            }
            return test;
        }

        @Override
        public String toString() {
            if (acceptMode) {
                return OperandsConstants.DATE_TYPE + ":" + operandBody;
            } else {
                return "!" + OperandsConstants.DATE_TYPE + ":" + operandBody;
            }
        }

    }

}
