/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.filters;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.LogicalOperationUtils;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.SimpleOperand;
import net.mapeadores.util.logicaloperation.SubOperand;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.searchengine.api.operands.OperandMessageKeys;
import net.scrutari.searchengine.api.operands.OperandsConstants;
import net.scrutari.searchengine.tools.OperandCheck;


/**
 *
 * @author Vincent Calame
 */
public class FilterOperandConverterEngine {

    /**
     * Retourne OperandCheck.UNDEFINED_OPERANDECHECK en cas
     * d'impossibilité de définition sans que cela soit une erreur (par exemple, si l'opérande
     * se limite à la déclaration du type).
     * Ne retourne jamais nul
     * 
     */
    public static OperandCheck convert(Operand operand, CodeList corpusCodeList, String defaultType, OperandConverterParameters operandConverterParameters) {
        if (corpusCodeList == null) {
            corpusCodeList = operandConverterParameters.getDataAccess().getCorpusCodeList();
        }
        int count = corpusCodeList.getCodeCount();
        int[] array = new int[count];
        for (int i = 0; i < count; i++) {
            array[i] = corpusCodeList.getCode(i);
        }
        if (operand instanceof SubOperand) {
            return convertSubOperand((SubOperand) operand, array, defaultType, operandConverterParameters);
        } else if (operand instanceof SimpleOperand) {
            return convertSimpleOperand((SimpleOperand) operand, array, defaultType, operandConverterParameters);
        } else {
            throw new IllegalArgumentException("Unkonwn Operand instance = " + operand.getClass().getName());
        }
    }

    private static OperandCheck convertSubOperand(SubOperand subOperand, int[] codecorpusArray, String defaultType, OperandConverterParameters operandConverterParameters) {
        SubOperandCheckBuilder builder = new SubOperandCheckBuilder(subOperand);
        int count = subOperand.getOperandCount();
        boolean noneBreak = false;
        for (int i = 0; i < count; i++) {
            Operand operand = subOperand.getOperand(i);
            OperandCheck operandCheck;
            if (operand instanceof SimpleOperand) {
                operandCheck = convertSimpleOperand((SimpleOperand) operand, codecorpusArray, defaultType, operandConverterParameters);
            } else if (operand instanceof SubOperand) {
                operandCheck = convertSubOperand((SubOperand) operand, codecorpusArray, defaultType, operandConverterParameters);
            } else {
                throw new IllegalArgumentException("Unkonwn Operand instance = " + operand.getClass().getName());
            }
            if (operandCheck.equals(OperandCheck.NONE_OPERANDECHECK)) {
                if (subOperand.getOperator() == LogicalOperationConstants.INTERSECTION_OPERATOR) {
                    noneBreak = true;
                    break;
                }
            } else if (!operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
                builder.addOperandCheck(operandCheck);
            }
        }
        if (noneBreak) {
            return OperandCheck.NONE_OPERANDECHECK;
        }
        if (builder.isEmpty()) {
            return OperandCheck.UNDEFINED_OPERANDECHECK;
        }
        return builder.toOperandCheck();
    }

    /*
     * Retourne null en cas d'erreur et OperandCheck.UNDEFINED_OPERANDECHECK en cas
     * d'impossibilité de définition sans que cela soit une erreur (par exemple, si l'opérande
     * se limite à la déclaration du type
     * 
     */
    private static OperandCheck convertSimpleOperand(SimpleOperand simpleOperand, int[] codecorpusArray, String defaultType, OperandConverterParameters operandConverterParameters) {
        String operandString = simpleOperand.getOperandString();
        int idx = operandString.indexOf(':');
        String type;
        String operandBody;
        if (idx == -1) {
            if (defaultType == null) {
                operandConverterParameters.getProcessMessageHandler().addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.UNKNOWN_OPERAND_TYPE, operandString);
                return OperandCheck.UNDEFINED_OPERANDECHECK;
            } else {
                type = defaultType;
                operandBody = operandString;
            }
        } else {
            type = checkType(operandString.substring(0, idx));
            if (type == null) {
                type = OperandsConstants.ATTR_TYPE;
                operandBody = operandString;
            } else {
                operandBody = operandString.substring(idx + 1).trim();
                if (operandBody.length() == 0) {
                    return OperandCheck.UNDEFINED_OPERANDECHECK;
                }
            }
        }
        boolean acceptMode = simpleOperand.isAcceptMode();
        if (type.equals(OperandsConstants.MOTCLE_TYPE)) {
            return IndexationCheck.check(codecorpusArray, acceptMode, operandBody, operandString, operandConverterParameters);
        } else if (type.equals(OperandsConstants.DATE_TYPE)) {
            return DateCheck.check(acceptMode, operandBody, operandString, operandConverterParameters);
        } else if (type.equals(OperandsConstants.QID_TYPE)) {
            return PreviousResultCheck.check(acceptMode, operandBody, operandString, operandConverterParameters);
        } else if (type.equals(OperandsConstants.ADD_TYPE)) {
            return AddCheck.check(acceptMode, operandBody, operandString, operandConverterParameters);
        } else if (type.equals(OperandsConstants.ATTR_TYPE)) {
            return OperandCheck.NONE_OPERANDECHECK;
        } else {
            throw new IllegalArgumentException("Unknown defaultType = " + defaultType);
        }
    }

    private static String checkType(String s) {
        if (s.equals(OperandsConstants.MOTCLE_TYPE)) {
            return OperandsConstants.MOTCLE_TYPE;
        } else if (s.equals(OperandsConstants.DATE_TYPE)) {
            return OperandsConstants.DATE_TYPE;
        } else if (s.equals(OperandsConstants.QID_TYPE)) {
            return OperandsConstants.QID_TYPE;
        } else if (s.equals(OperandsConstants.ADD_TYPE)) {
            return OperandsConstants.ADD_TYPE;
        } else if (s.equals(OperandsConstants.ATTR_TYPE)) {
            return OperandsConstants.ATTR_TYPE;
        } else {
            return null;
        }
    }


    private static class SubOperandCheckBuilder {

        private List<OperandCheck> operandCheckList = new ArrayList<OperandCheck>();
        private short operator;

        private SubOperandCheckBuilder(SubOperand subOperand) {
            this.operator = subOperand.getOperator();
        }

        private void addOperandCheck(OperandCheck operandCheck) {
            operandCheckList.add(operandCheck);
        }

        private boolean isEmpty() {
            return operandCheckList.isEmpty();
        }

        private OperandCheck toOperandCheck() {
            boolean intersection = (operator == LogicalOperationConstants.INTERSECTION_OPERATOR);
            int size = operandCheckList.size();
            Operand[] array = new Operand[size];
            Set<Integer> resultCodeSet = null;
            for (int i = 0; i < size; i++) {
                OperandCheck operandCheck = operandCheckList.get(i);
                Set<Integer> codeSet = operandCheck.getCodeSet();
                if (i == 0) {
                    if (codeSet != null) {
                        resultCodeSet = new HashSet<Integer>(codeSet);
                    }
                } else {
                    if (intersection) {
                        if (codeSet != null) {
                            if (resultCodeSet != null) {
                                resultCodeSet.retainAll(codeSet);
                            } else {
                                resultCodeSet = new HashSet<Integer>(codeSet);
                            }
                        }
                    } else {
                        if (codeSet == null) {
                            resultCodeSet = null;
                        } else if (resultCodeSet != null) {
                            resultCodeSet.addAll(codeSet);
                        }
                    }
                }
                array[i] = operandCheck.getOperand();
            }
            SubOperand subOperand = LogicalOperationUtils.toSubOperand(array, operator);
            return new OperandCheck(resultCodeSet, subOperand);
        }

    }

}
