/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.search;

import java.text.ParseException;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.logicaloperation.LogicalOperationUtils;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.SimpleOperand;
import net.mapeadores.util.logicaloperation.SubOperand;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.text.search.SearchToken;
import net.mapeadores.util.text.search.SearchUtils;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.searchengine.api.operands.OperandMessageKeys;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;
import net.scrutari.searchengine.api.operands.search.SearchTokenOperand;


/**
 *
 * @author Vincent Calame
 */
public class SearchOperandConverterEngine {

    private SearchOperandConverterEngine() {
    }

    public static SearchOperandCheck convert(Operand operand, ProcessMessageHandler processMessageHandler, AttributeKeyCoder attributeKeyCoder) {
        if (operand instanceof SimpleOperand) {
            SearchTokenOperand searchTokenOperand = toSearchTokenOperand(1, (SimpleOperand) operand, processMessageHandler, attributeKeyCoder);
            if (searchTokenOperand == null) {
                return null;
            }
            return new SearchOperandCheck(1, searchTokenOperand);
        } else {
            return convertSubOperand(1, (SubOperand) operand, processMessageHandler, attributeKeyCoder);
        }
    }

    private static SearchOperandCheck convertSubOperand(int currentOperandNumber, SubOperand subOperand, ProcessMessageHandler processMessageHandler, AttributeKeyCoder attributeKeyCoder) {
        int count = subOperand.getOperandCount();
        Operand[] result = new Operand[count];
        for (int i = 0; i < count; i++) {
            Operand operand = subOperand.getOperand(i);
            if (operand instanceof SimpleOperand) {
                SearchTokenOperand searchTokenOperand = toSearchTokenOperand(currentOperandNumber, (SimpleOperand) operand, processMessageHandler, attributeKeyCoder);
                if (searchTokenOperand == null) {
                    return null;
                }
                result[i] = searchTokenOperand;
                currentOperandNumber++;
            } else {
                SearchOperandCheck check = convertSubOperand(currentOperandNumber, (SubOperand) operand, processMessageHandler, attributeKeyCoder);
                if (check == null) {
                    return null;
                }
                result[i] = check.getOperand();
                currentOperandNumber = check.getOperandMaxNumber() + 1;
            }
        }
        SubOperand resultOperand = LogicalOperationUtils.toSubOperand(result, subOperand.getOperator());
        return new SearchOperandCheck(currentOperandNumber - 1, resultOperand);
    }

    private static SearchTokenOperand toSearchTokenOperand(int operandNumber, SimpleOperand simpleOperand, ProcessMessageHandler processMessageHandler, AttributeKeyCoder attributeKeyCoder) {
        boolean acceptMode = simpleOperand.isAcceptMode();
        String operandString = simpleOperand.getOperandString();
        String tokenString = operandString;
        int idx = operandString.lastIndexOf(':');
        String scope = null;
        if (idx != -1) {
            scope = operandString.substring(0, idx).trim();
            tokenString = operandString.substring(idx + 1).trim();
        }
        AlineaFilter alineaFilter;
        if (scope != null) {
            alineaFilter = AlineaFilters.getSpecialFilter(scope);
            if (alineaFilter == null) {
                try {
                    AttributeKey attributeKey = AttributeKey.parse(scope);
                    int alineaRootCode = attributeKeyCoder.getAlineaRootCode(attributeKey);
                    if (alineaRootCode != -1) {
                        alineaFilter = AlineaFilters.getAlineaRootCodeFilter(scope, alineaRootCode);
                    } else {
                        processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.UNKNOWN_ATTRIBUTE, scope);
                        return null;
                    }
                } catch (ParseException pe) {
                    processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.WRONG_SCOPE, scope);
                    return null;
                }
            }
            if (alineaFilter == null) {
            }
        } else {
            alineaFilter = AlineaFilters.ALL_FILTER;
        }
        SearchToken searchToken = parseSearchToken(tokenString, operandString, processMessageHandler);
        if (searchToken == null) {
            return null;
        }
        return SearchOperationUtils.toSearchTokenOperand(operandNumber, acceptMode, searchToken, alineaFilter);
    }

    private static SearchToken parseSearchToken(String tokenString, String operandString, ProcessMessageHandler processMessageHandler) {
        int length = tokenString.length();
        if (length == 0) {
            processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.EMPTY_SEARCHTOKEN, operandString);
            return null;
        }
        boolean onQuote = false;
        if (tokenString.charAt(0) == '"') {
            if (tokenString.charAt(length - 1) != '"') {
                processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.UNCLOSED_QUOTE, operandString);
                return null;
            }
            tokenString = tokenString.substring(1, length - 1).trim();
            length = tokenString.length();
            onQuote = true;
            if (length == 0) {
                processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.EMPTY_SEARCHTOKEN, operandString);
                return null;
            }
        }
        String[] tokenArray = StringUtils.getTokens(tokenString, ' ', StringUtils.EMPTY_EXCLUDE);
        if (onQuote) {
            int tokenLength = tokenArray.length;
            SimpleSearchToken[] array = new SimpleSearchToken[tokenLength];
            for (int i = 0; i < tokenLength; i++) {
                SimpleSearchToken simpleSearchToken = parseSimpleSearchToken(tokenArray[i], operandString, processMessageHandler);
                if (simpleSearchToken == null) {
                    return null;
                }
                array[i] = simpleSearchToken;
            }
            return SearchUtils.toMultiSearchToken(array);
        } else {
            if (tokenArray.length > 1) {
                processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.UNEXPECTED_SPACE, operandString);
                return null;
            }
            return parseSimpleSearchToken(tokenArray[0], operandString, processMessageHandler);
        }
    }

    private static SimpleSearchToken parseSimpleSearchToken(String token, String operandString, ProcessMessageHandler processMessageHandler) {
        int length = token.length();
        int startIndex = 0;
        int currentType = TextConstants.MATCHES;
        if (token.charAt(0) == '*') {
            startIndex = 1;
            currentType = TextConstants.ENDSWITH;
        }
        if (token.charAt(length - 1) == '*') {
            length = length - 1;
            currentType = currentType | TextConstants.STARTSWITH;
        }
        if (startIndex >= length) {
            processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.MISSING_WORD, operandString);
            return null;
        }
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = startIndex; i < length; i++) {
            char carac = token.charAt(i);
            if (!Character.isLetterOrDigit(carac)) {
                processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.NOT_LETTERORDIGIT_CHARACTER, operandString);
                return null;
            }
            stringBuilder.append(carac);
        }
        return SearchUtils.toSimpleSearchToken(stringBuilder.toString(), currentType);
    }


    public static class SearchOperandCheck {

        private Operand operand;
        private int operandMaxNumber;

        private SearchOperandCheck(int operandMaxNumber, Operand operand) {
            this.operandMaxNumber = operandMaxNumber;
            this.operand = operand;
        }

        public int getOperandMaxNumber() {
            return operandMaxNumber;
        }

        public Operand getOperand() {
            return operand;
        }

    }

}
