/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.search;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.SubOperand;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.text.search.MultiSearchToken;
import net.mapeadores.util.text.search.SearchToken;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.operands.search.QInfo;
import net.scrutari.searchengine.api.operands.search.SearchTokenOperand;


/**
 *
 * @author Vincent Calame
 */
public class SearchOperationUtils {

    private SearchOperationUtils() {
    }

    public static int[] toAcceptOperandArray(SearchOperation searchOperation) {
        List<Integer> list = new ArrayList<Integer>();
        Operand operand = searchOperation.getRootOperand();
        if (operand instanceof SearchTokenOperand) {
            checkAcceptOperand((SearchTokenOperand) operand, list);
        } else {
            checkAcceptOperand((SubOperand) operand, list);
        }
        int size = list.size();
        int[] result = new int[size];
        for (int i = 0; i < size; i++) {
            result[i] = list.get(i);
        }
        return result;
    }

    private static void checkAcceptOperand(SearchTokenOperand operand, List<Integer> list) {
        if (operand.isAcceptMode()) {
            list.add(operand.getOperandNumber());
        }
    }

    private static void checkAcceptOperand(SubOperand subOperand, List<Integer> list) {
        int count = subOperand.getOperandCount();
        for (int i = 0; i < count; i++) {
            Operand operand = subOperand.getOperand(i);
            if (operand instanceof SearchTokenOperand) {
                checkAcceptOperand((SearchTokenOperand) operand, list);
            } else {
                checkAcceptOperand((SubOperand) operand, list);
            }
        }
    }

    public static SearchTokenOperand toSearchTokenOperand(int operandNumber, boolean acceptMode, SearchToken searchToken, AlineaFilter alineaFilter) {
        return new DefaultSearchTokenOperand(operandNumber, acceptMode, searchToken, alineaFilter);
    }

    public static SearchTokenOperand derive(SearchTokenOperand searchTokenOperand, SearchToken searchToken) {
        return new DefaultSearchTokenOperand(searchTokenOperand.getOperandNumber(), searchTokenOperand.isAcceptMode(), searchToken, searchTokenOperand.getAlineaFilter());
    }

    public static QInfo toQInfo(String q, String qSource, int qLength) {
        return new DefaultQInfo(q, qSource, qLength);
    }

    public static boolean hasMandatoryOperand(SearchOperation searchOperation) {
        Operand rootOperand = searchOperation.getRootOperand();
        if (rootOperand instanceof SearchTokenOperand) {
            return ((SearchTokenOperand) rootOperand).isAcceptMode();
        } else {
            return hasMandatoryOperand((SubOperand) rootOperand);
        }
    }

    private static boolean hasMandatoryOperand(SubOperand subOperand) {
        int count = subOperand.getOperandCount();
        short operator = subOperand.getOperator();
        boolean mandatory;
        if (operator == LogicalOperationConstants.INTERSECTION_OPERATOR) {
            mandatory = false;
            for (int i = 0; i < count; i++) {
                Operand operand = subOperand.getOperand(i);
                if (operand instanceof SearchTokenOperand) {
                    if (((SearchTokenOperand) operand).isAcceptMode()) {
                        mandatory = true;
                        break;
                    }
                } else {
                    SubOperand subsubOperand = (SubOperand) operand;
                    boolean test = hasMandatoryOperand(subsubOperand);
                    if (test) {
                        mandatory = true;
                        break;
                    }
                }
            }
        } else {
            mandatory = true;
            for (int i = 0; i < count; i++) {
                Operand operand = subOperand.getOperand(i);
                if (operand instanceof SearchTokenOperand) {
                    if (!((SearchTokenOperand) operand).isAcceptMode()) {
                        mandatory = false;
                        break;
                    }
                } else {
                    SubOperand subsubOperand = (SubOperand) operand;
                    boolean test = hasMandatoryOperand(subsubOperand);
                    if (!test) {
                        mandatory = false;
                        break;
                    }
                }
            }
        }
        return mandatory;
    }

    private static void appendSimpleSearchToken(StringBuilder buf, SimpleSearchToken simpleSearchToken) {
        int searchType = simpleSearchToken.getSearchType();
        if ((searchType & TextConstants.ENDSWITH) != 0) {
            buf.append('*');
        }
        buf.append(simpleSearchToken.getTokenString());
        if ((searchType & TextConstants.STARTSWITH) != 0) {
            buf.append('*');
        }
    }


    private static class DefaultSearchTokenOperand implements SearchTokenOperand {

        private int operandNumber;
        private boolean acceptMode;
        private SearchToken searchToken;
        private AlineaFilter alineaFilter;

        private DefaultSearchTokenOperand(int operandNumber, boolean acceptMode, SearchToken searchToken, AlineaFilter alineaFilter) {
            this.operandNumber = operandNumber;
            this.acceptMode = acceptMode;
            this.searchToken = searchToken;
            this.alineaFilter = alineaFilter;
        }

        @Override
        public SearchToken getSearchToken() {
            return searchToken;
        }

        @Override
        public boolean isAcceptMode() {
            return acceptMode;
        }

        @Override
        public int getOperandNumber() {
            return operandNumber;
        }

        @Override
        public AlineaFilter getAlineaFilter() {
            return alineaFilter;
        }

        @Override
        public String toString() {
            StringBuilder buf = new StringBuilder();
            if (!acceptMode) {
                buf.append('!');
            }
            String filterPrefix = alineaFilter.getFilterPrefix();
            if (filterPrefix.length() > 0) {
                buf.append(filterPrefix);
                buf.append(':');
            }
            if (searchToken instanceof SimpleSearchToken) {
                appendSimpleSearchToken(buf, (SimpleSearchToken) searchToken);
            } else {
                MultiSearchToken multiSearchToken = (MultiSearchToken) searchToken;
                buf.append('"');
                int count = multiSearchToken.getSubtokenCount();
                for (int i = 0; i < count; i++) {
                    if (i != 0) {
                        buf.append(' ');
                    }
                    appendSimpleSearchToken(buf, multiSearchToken.getSubtoken(i));
                }
                buf.append('"');
            }
            return buf.toString();
        }

    }


    private static class DefaultQInfo implements QInfo {

        private final String q;
        private final String qSource;
        private final int qLength;

        private DefaultQInfo(String q, String qSource, int qLength) {
            this.q = q;
            this.qSource = qSource;
            this.qLength = qLength;
        }

        @Override
        public String getQ() {
            return q;
        }

        @Override
        public String getQSource() {
            return qSource;
        }

        @Override
        public int getQLength() {
            return qLength;
        }

    }

}
