/* ScrutariLib_SearchEngine - Copyright (c) 2006-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.exceptions.SwitchException;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.stats.CountStats;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.ScrutariDataURIList;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.tools.FilterBuffer;


/**
 *
 * @author Vincent Calame
 */
public class OptionsUtils {

    private OptionsUtils() {
    }

    public static CodeList getSelectionThesaurusCodeList(CountStats countStats, DataAccess dataAccess, CodeList corpusCodeList) {
        if (corpusCodeList == null) {
            return null;
        }
        Set<Integer> thesaurusSet = new LinkedHashSet<Integer>();
        int corpusCount = corpusCodeList.getCodeCount();
        CodeList thesaurusCodeList = dataAccess.getThesaurusCodeList();
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodeList.getCode(i);
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = thesaurusCodeList.getCode(j);
                if (countStats.hasIndexation(codecorpus, codethesaurus)) {
                    thesaurusSet.add(codethesaurus);
                }
            }
        }
        return CodesUtils.toCodeList(thesaurusSet);
    }

    public static CodeList getFicheCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        FilterBuffer filterBuffer = new FilterBuffer(dataAccess, searchOptions);
        CodeList corpusCodeList = searchOptions.getCorpusCodeList();
        if (corpusCodeList == null) {
            corpusCodeList = dataAccess.getCorpusCodeList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int corpusCount = corpusCodeList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodeList.getCode(i);
            CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
            int ficheCount = corpusData.getFicheCount();
            for (int j = 0; j < ficheCount; j++) {
                int codefiche = corpusData.getCodeFiche(j);
                if (filterBuffer.acceptFiche(codefiche)) {
                    list.add(codefiche);
                }
            }
        }
        return CodesUtils.toCodeList(list);
    }

    public static CodeList getMotcleCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        FilterBuffer filterBuffer = new FilterBuffer(dataAccess, searchOptions);
        CodeList thesaurusCodeList = searchOptions.getThesaurusCodeList();
        if (thesaurusCodeList == null) {
            thesaurusCodeList = dataAccess.getThesaurusCodeList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < thesaurusCount; i++) {
            int codethesaurus = thesaurusCodeList.getCode(i);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                int codemotcle = thesaurusData.getCodeMotcle(j);
                if (filterBuffer.acceptMotcle(codemotcle)) {
                    list.add(codemotcle);
                }
            }
        }
        return CodesUtils.toCodeList(list);
    }

    public static ScrutariDataURIList toScrutariDataURIList(DataAccess dataAccess, short type, CodeList codeList) {
        List<ScrutariDataURI> list = new ArrayList<ScrutariDataURI>();
        int codeCount = codeList.getCodeCount();
        for (int i = 0; i < codeCount; i++) {
            int code = codeList.getCode(i);
            switch (type) {
                case ScrutariDataURI.BASEURI_TYPE:
                    BaseData baseData = dataAccess.getBaseDataByCode(code);
                    if (baseData != null) {
                        list.add(baseData.getBaseURI());
                    }
                    break;
                case ScrutariDataURI.CORPUSURI_TYPE:
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
                    if (corpusData != null) {
                        list.add(corpusData.getCorpusURI());
                    }
                    break;
                case ScrutariDataURI.THESAURUSURI_TYPE:
                    ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
                    if (thesaurusData != null) {
                        list.add(thesaurusData.getThesaurusURI());
                    }
                    break;
                case ScrutariDataURI.FICHEURI_TYPE:
                    FicheData ficheData = dataAccess.getFicheDataByCode(code);
                    if (ficheData != null) {
                        list.add(ficheData.getFicheURI());
                    }
                    break;
                case ScrutariDataURI.MOTCLEURI_TYPE:
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(code);
                    if (motcleData != null) {
                        list.add(motcleData.getMotcleURI());
                    }
                    break;
                default:
                    throw new SwitchException("Unkown type: " + type);
            }
        }
        ScrutariDataURI[] array = list.toArray(new ScrutariDataURI[list.size()]);
        return new ArrayScrutariDataURIList(type, array);
    }

    public static ScrutariDataURIList toScrutariDataURIList(short type, Collection<ScrutariDataURI> collection) {
        ScrutariDataURI[] array = collection.toArray(new ScrutariDataURI[collection.size()]);
        return new ArrayScrutariDataURIList(type, array);
    }


    private static class ArrayScrutariDataURIList implements ScrutariDataURIList {

        private final short type;
        private final ScrutariDataURI[] array;

        private ArrayScrutariDataURIList(short type, ScrutariDataURI[] array) {
            this.type = type;
            this.array = array;
        }

        @Override
        public short getType() {
            return type;
        }

        @Override
        public int getScrutariDataURICount() {
            return array.length;
        }

        @Override
        public ScrutariDataURI getScrutariDataURI(int index) {
            return array[index];
        }

    }

}
