/* ScrutariLib_SearchEngine - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.LogicalOperationUtils;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.text.TextConstants;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.ElementCodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.ScrutariDataURIList;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.tools.OperandCheck;


/**
 *
 *
 * @author  Vincent Calame
 */
public final class SearchOptionsBuilder {

    public final static SearchOptions EMPTY_SEARCHOPTIONS = new InternalSearchOptions(LangInteger.UNDETERMINED, null, TextConstants.STARTSWITH, new HashMap<Short, ScrutariDataURIList>(), null, null, null, null, null, null);
    private final List<Operand> ficheOperandList = new ArrayList<Operand>();
    private final List<Operand> motcleOperandList = new ArrayList<Operand>();
    private final Map<Short, ScrutariDataURIList> scrutariDataURIListMap = new HashMap<Short, ScrutariDataURIList>();
    private int langUi;
    private int[] langIntegerArray = null;
    private int defaultSearchType = TextConstants.STARTSWITH;
    private PertinencePonderation pertinencePonderation;
    private Set<Integer> corpusSet = null;
    private Set<Integer> thesaurusSet = null;
    private Map<Integer, Integer> ficheMap = null;

    public SearchOptionsBuilder() {
    }

    public void setLangUi(int langUi) {
        this.langUi = langUi;
    }

    public int[] getLangIntegerArray() {
        return langIntegerArray;
    }

    /**
     * Nul ou de longueur non nulle.
     *
     */
    public void setLangIntegerArray(int[] langIntegerArray) {
        if ((langIntegerArray != null) && (langIntegerArray.length == 0)) {
            this.langIntegerArray = null;
        } else {
            this.langIntegerArray = langIntegerArray;
        }
    }

    public void setDefaultSearchType(int defaultSearchType) {
        this.defaultSearchType = defaultSearchType;
    }

    public void putScrutariDataURIList(ScrutariDataURIList scrutariDataURIList) {
        if (scrutariDataURIList != null) {
            scrutariDataURIListMap.put(scrutariDataURIList.getType(), scrutariDataURIList);
        }
    }

    public void reduceFicheList(ElementCodeList ficheCodeList) {
        if (ficheCodeList == null) {
            return;
        }
        if ((ficheMap != null) && (ficheMap.isEmpty())) {
            return;
        }
        if ((corpusSet != null) && (corpusSet.isEmpty())) {
            ficheMap = Collections.emptyMap();
            return;
        }
        int ficheCount = ficheCodeList.getCodeCount();
        if (ficheCount == 0) {
            corpusSet = Collections.emptySet();
            ficheMap = Collections.emptyMap();
            return;
        }
        if (corpusSet == null) {
            corpusSet = new LinkedHashSet<Integer>();
            ficheMap = new LinkedHashMap<Integer, Integer>();
            for (int i = 0; i < ficheCount; i++) {
                int codecorpus = ficheCodeList.getSubsetCode(i);
                corpusSet.add(codecorpus);
                ficheMap.put(ficheCodeList.getCode(i), codecorpus);
            }
            return;
        }
        if (ficheMap == null) {
            ficheMap = new LinkedHashMap<Integer, Integer>();
            for (int i = 0; i < ficheCount; i++) {
                int codecorpus = ficheCodeList.getSubsetCode(i);
                if (corpusSet.contains(codecorpus)) {
                    ficheMap.put(ficheCodeList.getCode(i), codecorpus);
                }
            }
            return;
        }
        Set<Integer> newCorpusSet = new HashSet<Integer>();
        Map<Integer, Integer> newFicheMap = new LinkedHashMap<Integer, Integer>();
        for (int i = 0; i < ficheCount; i++) {
            int codecorpus = ficheCodeList.getSubsetCode(i);
            if (corpusSet.contains(codecorpus)) {
                int codefiche = ficheCodeList.getCode(i);
                if (ficheMap.containsKey(codefiche)) {
                    newCorpusSet.add(codecorpus);
                    newFicheMap.put(codefiche, codecorpus);
                }
            }
        }
        corpusSet = newCorpusSet;
        ficheMap = newFicheMap;
    }

    public void reduceCorpusList(CodeList corpusCodeList) {
        if (corpusCodeList == null) {
            return;
        }
        if ((corpusSet != null) && (corpusSet.isEmpty())) {
            return;
        }
        int corpusCount = corpusCodeList.getCodeCount();
        if (corpusCount == 0) {
            corpusSet = Collections.emptySet();
            if (ficheMap != null) {
                ficheMap = Collections.emptyMap();
            }
            return;
        }
        if (corpusSet == null) {
            corpusSet = new LinkedHashSet<Integer>();
            int count = corpusCodeList.getCodeCount();
            for (int i = 0; i < count; i++) {
                corpusSet.add(corpusCodeList.getCode(i));
            }
            return;
        }
        Set<Integer> newCorpusSet = new HashSet<Integer>();
        for (int i = 0; i < corpusCount; i++) {
            int code = corpusCodeList.getCode(i);
            if (corpusSet.contains(code)) {
                newCorpusSet.add(code);
            }
        }
        if (ficheMap != null) {
            for (Iterator<Map.Entry<Integer, Integer>> it = ficheMap.entrySet().iterator(); it.hasNext();) {
                Map.Entry<Integer, Integer> entry = it.next();
                if (!newCorpusSet.contains(entry.getValue())) {
                    it.remove();
                }
            }
        }
        corpusSet = newCorpusSet;
    }

    public void reduceThesaurusList(CodeList thesaurusCodeList) {
        if (thesaurusCodeList == null) {
            return;
        }
        if ((thesaurusSet != null) && (thesaurusSet.isEmpty())) {
            return;
        }
        int count = thesaurusCodeList.getCodeCount();
        if (count == 0) {
            thesaurusSet = Collections.emptySet();
            return;
        }
        if (thesaurusSet == null) {
            thesaurusSet = new LinkedHashSet<Integer>();
            for (int i = 0; i < count; i++) {
                thesaurusSet.add(thesaurusCodeList.getCode(i));
            }
            return;
        }
        Set<Integer> newThesaurusSet = new HashSet<Integer>();
        for (int i = 0; i < count; i++) {
            int code = thesaurusCodeList.getCode(i);
            Integer itg = new Integer(code);
            if (thesaurusSet.contains(itg)) {
                newThesaurusSet.add(itg);
            }
        }
        thesaurusSet = newThesaurusSet;
    }

    public CodeList getCurrentCorpusList() {
        if (corpusSet != null) {
            return CodesUtils.toCodeList(corpusSet);
        } else {
            return null;
        }
    }

    public SearchOptions toSearchOptions() {
        Operand ficheRootOperand = null;
        int ficheOperandSize = ficheOperandList.size();
        if (ficheOperandSize > 0) {
            if (ficheOperandSize == 1) {
                ficheRootOperand = ficheOperandList.get(0);
            } else {
                ficheRootOperand = LogicalOperationUtils.toSubOperand(ficheOperandList.toArray(new Operand[ficheOperandSize]), LogicalOperationConstants.INTERSECTION_OPERATOR);
            }
        }
        Operand motcleRootOperand = null;
        int motcleOperandSize = motcleOperandList.size();
        if (motcleOperandSize > 0) {
            if (motcleOperandSize == 1) {
                motcleRootOperand = motcleOperandList.get(0);
            } else {
                motcleRootOperand = LogicalOperationUtils.toSubOperand(motcleOperandList.toArray(new Operand[motcleOperandSize]), LogicalOperationConstants.INTERSECTION_OPERATOR);
            }
        }
        ElementCodeList codeficheList = null;
        CodeList codecorpusList = null;
        CodeList codethesaurusList = null;
        if (ficheMap != null) {
            codeficheList = CodesUtils.toElementCodeList(ficheMap);
        }
        if (corpusSet != null) {
            codecorpusList = CodesUtils.toCodeList(corpusSet);
        }
        if (thesaurusSet != null) {
            codethesaurusList = CodesUtils.toCodeList(thesaurusSet);
        }
        return new InternalSearchOptions(langUi, langIntegerArray, defaultSearchType, scrutariDataURIListMap, codeficheList, codecorpusList, codethesaurusList, ficheRootOperand, motcleRootOperand, pertinencePonderation);
    }

    public void addFicheOperandCheck(OperandCheck operandCheck) {
        Set<Integer> codeSet = operandCheck.getCodeSet();
        if (codeSet != null) {
            if (corpusSet == null) {
                corpusSet = new HashSet(codeSet);
            } else {
                corpusSet.retainAll(codeSet);
            }
        }
        ficheOperandList.add(operandCheck.getOperand());
    }

    public void addMotcleOperandCheck(OperandCheck operandCheck) {
        Set<Integer> codeSet = operandCheck.getCodeSet();
        if (codeSet != null) {
            if (thesaurusSet == null) {
                thesaurusSet = new HashSet(codeSet);
            } else {
                thesaurusSet.retainAll(codeSet);
            }
        }
        motcleOperandList.add(operandCheck.getOperand());
    }

    public void setPertinencePonderation(PertinencePonderation pertinencePonderation) {
        this.pertinencePonderation = pertinencePonderation;
    }

    public static SearchOptionsBuilder derive(SearchOptions searchOptions) {
        SearchOptionsBuilder builder = new SearchOptionsBuilder();
        builder.setLangUi(searchOptions.getLangUi());
        builder.setLangIntegerArray(searchOptions.getLangIntegerArray());
        builder.setDefaultSearchType(searchOptions.getDefaultSearchType());
        builder.reduceFicheList(searchOptions.getFicheCodeList());
        builder.reduceCorpusList(searchOptions.getCorpusCodeList());
        builder.reduceThesaurusList(searchOptions.getThesaurusCodeList());
        builder.putScrutariDataURIList(searchOptions.getScrutariDataURIList(ScrutariDataURI.BASEURI_TYPE));
        builder.putScrutariDataURIList(searchOptions.getScrutariDataURIList(ScrutariDataURI.CORPUSURI_TYPE));
        builder.putScrutariDataURIList(searchOptions.getScrutariDataURIList(ScrutariDataURI.THESAURUSURI_TYPE));
        builder.putScrutariDataURIList(searchOptions.getScrutariDataURIList(ScrutariDataURI.FICHEURI_TYPE));
        builder.putScrutariDataURIList(searchOptions.getScrutariDataURIList(ScrutariDataURI.MOTCLEURI_TYPE));
        Operand ficheOperand = searchOptions.getFicheEligibilityOperand();
        if (ficheOperand != null) {
            builder.ficheOperandList.add(ficheOperand);
        }
        Operand motcleOperand = searchOptions.getMotcleEligibilityOperand();
        if (motcleOperand != null) {
            builder.motcleOperandList.add(motcleOperand);
        }
        return builder;
    }


    private static class InternalSearchOptions implements SearchOptions {

        private final int langUi;
        private final int[] langIntegerArray;
        private final int defaultSearchType;
        private final Map<Short, ScrutariDataURIList> scrutariDataURIListMap;
        private final ElementCodeList codeficheList;
        private final CodeList codecorpusList;
        private final CodeList codethesaurusList;
        private final Operand ficheEligibilityOperand;
        private final Operand motcleEligibilityOperand;
        private final PertinencePonderation pertinencePonderation;

        private InternalSearchOptions(int langUi, int[] langIntegerArray, int defaultSearchType, Map<Short, ScrutariDataURIList> scrutariDataURIListMap, ElementCodeList codeficheList, CodeList codecorpusList, CodeList codethesaurusList, Operand ficheEligibilityOperand, Operand motcleEligibilityOperand, PertinencePonderation pertinencePonderation) {
            this.langUi = langUi;
            this.langIntegerArray = langIntegerArray;
            this.defaultSearchType = defaultSearchType;
            this.scrutariDataURIListMap = scrutariDataURIListMap;
            this.codeficheList = codeficheList;
            this.codecorpusList = codecorpusList;
            this.codethesaurusList = codethesaurusList;
            this.ficheEligibilityOperand = ficheEligibilityOperand;
            this.motcleEligibilityOperand = motcleEligibilityOperand;
            this.pertinencePonderation = pertinencePonderation;
        }

        @Override
        public int getDefaultSearchType() {
            return defaultSearchType;
        }

        @Override
        public int getLangUi() {
            return langUi;
        }

        @Override
        public int[] getLangIntegerArray() {
            return langIntegerArray;
        }

        @Override
        public ScrutariDataURIList getScrutariDataURIList(short type) {
            return scrutariDataURIListMap.get(type);
        }

        @Override
        public ElementCodeList getFicheCodeList() {
            return codeficheList;
        }

        @Override
        public CodeList getCorpusCodeList() {
            return codecorpusList;
        }

        @Override
        public CodeList getThesaurusCodeList() {
            return codethesaurusList;
        }

        @Override
        public Operand getFicheEligibilityOperand() {
            return ficheEligibilityOperand;
        }

        @Override
        public Operand getMotcleEligibilityOperand() {
            return motcleEligibilityOperand;
        }

        @Override
        public PertinencePonderation getPertinencePonderation() {
            return pertinencePonderation;
        }

    }

}
