/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.pertinence;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.SubstringPosition;
import net.scrutari.db.api.DataAccess;
import net.scrutari.lexie.AlineaInteger;
import net.scrutari.searchengine.api.pertinence.FichePertinence;
import net.scrutari.searchengine.api.pertinence.MotclePertinence;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.tools.result.FicheSearchResultInfoBuilder;
import net.scrutari.searchengine.tools.result.SearchTokenOccurrenceListBuilder;


/**
 *
 * @author Vincent Calame
 */
public class PertinenceUtils {

    private PertinenceUtils() {
    }

    public static FicheSearchResultInfo toFicheSearchResultInfo(PertinencePack pertinencePack, int[] acceptOperandArray) {
        FichePertinence fichePertinence = pertinencePack.getFichePertinence();
        FicheSearchResultInfoBuilder builder = new FicheSearchResultInfoBuilder();
        builder.setCode(fichePertinence.getCodefiche());
        builder.setMotcleCodeArray(fichePertinence.getAcceptCodemotcleArray());
        builder.setInAlineaOccurrenceListArray(getInAlineaOccurrenceListArray(fichePertinence, acceptOperandArray));
        builder.setScoreArray(pertinencePack.getScoreArray());
        return builder.toFicheSearchResultInfo();
    }

    public static InAlineaOccurrenceList[] getInAlineaOccurrenceListArray(FichePertinence fichePertinence, int[] acceptOperandArray) {
        int acceptLength = acceptOperandArray.length;
        LinkedIntHashMap intHashMap = new LinkedIntHashMap();
        for (int i = 0; i < acceptLength; i++) {
            int operandNumber = acceptOperandArray[i];
            int alineaInteger = fichePertinence.getAlineaInteger(operandNumber);
            if (alineaInteger < 0) {
                continue;
            }
            SubstringPosition substringPosition = fichePertinence.getSubstringPosition(operandNumber);
            SearchTokenOccurrenceListBuilder builder = (SearchTokenOccurrenceListBuilder) intHashMap.get(alineaInteger);
            if (builder == null) {
                builder = new SearchTokenOccurrenceListBuilder(acceptLength);
                intHashMap.put(alineaInteger, builder);
            }
            builder.add(operandNumber, substringPosition);
        }
        int size = intHashMap.size();
        InAlineaOccurrenceList[] ficheSearchTokenPositionListArray = new InAlineaOccurrenceList[size];
        for (int i = 0; i < size; i++) {
            int alineaInteger = intHashMap.getKeyAt(i);
            SearchTokenOccurrenceListBuilder builder = (SearchTokenOccurrenceListBuilder) intHashMap.getValueAt(i);
            ficheSearchTokenPositionListArray[i] = builder.toInAlineaOccurrenceList(alineaInteger);
        }
        return ficheSearchTokenPositionListArray;
    }

    public static InLangOccurrenceList[] getInLangOccurrenceListArray(MotclePertinence motclePertinence, int[] acceptOperandArray) {
        int langCount = motclePertinence.getLangOccurrenceCount();
        SearchTokenOccurrenceListBuilder[] builderArray = new SearchTokenOccurrenceListBuilder[langCount];
        int[] langArray = new int[langCount];
        int acceptLength = acceptOperandArray.length;
        for (int i = 0; i < langCount; i++) {
            MotclePertinence.LangOccurrence langOccurrence = motclePertinence.getLangOccurrence(i);
            langArray[i] = langOccurrence.getLangInteger();
            for (int j = 0; j < acceptLength; j++) {
                int operandNumber = acceptOperandArray[j];
                SubstringPosition substringPosition = langOccurrence.getSubstringPosition(operandNumber);
                if (substringPosition == null) {
                    continue;
                }
                SearchTokenOccurrenceListBuilder builder = builderArray[i];
                if (builder == null) {
                    builder = new SearchTokenOccurrenceListBuilder(acceptLength);
                    builderArray[i] = builder;
                }
                builder.add(operandNumber, substringPosition);
            }
        }
        List<InLangOccurrenceList> list = new ArrayList<InLangOccurrenceList>(langCount);
        for (int i = 0; i < langCount; i++) {
            int langInteger = langArray[i];
            SearchTokenOccurrenceListBuilder builder = builderArray[i];
            if (builder != null) {
                list.add(builder.toInLangOccurrenceList(langInteger));
            }
        }
        return list.toArray(new InLangOccurrenceList[list.size()]);
    }

    public static SortedSet<PertinencePack> addOriginScore(SortedSet<PertinencePack> set, float originPonderation, DataAccess dataAccess) {
        SortedSet<PertinencePack> resultSet = new TreeSet<PertinencePack>();
        Map<Integer, Integer> avanceMap = new HashMap<Integer, Integer>();
        for (PertinencePack pertinencePack : set) {
            int codecorpus = pertinencePack.getFichePertinence().getCodecorpus();
            int codebase = dataAccess.getCorpusDataByCode(codecorpus).getCodeBase();
            Integer avance = avanceMap.get(codebase);
            float originScore = 0;
            if (avance == null) {
                originScore = originPonderation;
                avanceMap.put(codebase, 128); //la seconde valeur sera à deux tiers 128 / 192
            } else {
                int newAvance = avance.intValue();
                if (newAvance > 1) {
                    originScore = originPonderation * newAvance / 192;
                    newAvance = newAvance / 2;
                    avanceMap.put(codebase, newAvance);
                }
            }
            if (originScore != 0) {
                resultSet.add(pertinencePack.derive(originScore));
            } else {
                resultSet.add(pertinencePack);
            }
        }
        return resultSet;
    }

    public static float getDateScore(int dateInteger, int currentAnnee) {
        if (dateInteger == DateInteger.NULL) {
            return 0;
        }
        int annee = DateInteger.getAnnee(dateInteger);
        float x = (float) Math.abs(annee - currentAnnee);
        return 1 - (x / (x + 6));
    }

    public static float getOccurrenceScore(FichePertinence fichePertinence, int[] acceptOperandArray) {
        int length = acceptOperandArray.length;
        int bestAlinea = -1;
        int bestBegin = -1;
        int note = 0;
        for (int i = 0; i < length; i++) {
            int operandNumber = acceptOperandArray[i];
            int alineaInteger = fichePertinence.getAlineaInteger(operandNumber);
            int motcleNote = convertMotcleNote(fichePertinence.getMotcleScore(operandNumber));
            if (alineaInteger > 0) {
                if ((bestAlinea == -1) || (alineaInteger < bestAlinea)) {
                    bestAlinea = alineaInteger;
                    bestBegin = fichePertinence.getSubstringPosition(operandNumber).getBeginIndex();
                } else if (alineaInteger == bestAlinea) {
                    if (bestBegin == -1) {
                        bestBegin = fichePertinence.getSubstringPosition(operandNumber).getBeginIndex();
                    } else {
                        bestBegin = Math.min(bestBegin, fichePertinence.getSubstringPosition(operandNumber).getBeginIndex());
                    }
                }
                note = note + getOperandNote(alineaInteger, motcleNote);
            } else if (motcleNote > 0) {
                note = note + 35 + motcleNote;
                if ((bestAlinea == -1) || (bestAlinea > AlineaInteger.COMPLEMENT_START)) {
                    bestAlinea = AlineaInteger.COMPLEMENT_START;
                    bestBegin = 10;
                }
            }
        }
        if (bestAlinea > 0) {
            note = note + getOperandNote(bestAlinea, 0); //On double la note du meilleur champ
            note = note + getBestIndexPrime(bestBegin);
        }
        int notemaximale = 115 * (length + 1) + 4;
        return ((float) note) / notemaximale;
    }

    private static int getBestIndexPrime(int bestIndex) {
        if (bestIndex == 0) {
            return 4;
        } else if (bestIndex < 5) {
            return 3;
        } else if (bestIndex < 20) {
            return 2;
        } else if (bestIndex < 50) {
            return 1;
        } else {
            return 0;
        }
    }

    private static int convertMotcleNote(float floatNote) {
        return (int) (15 * floatNote);
    }

    private static int getOperandNote(int alineaInteger, int motcleNote) {
        if (alineaInteger == AlineaInteger.TITRE) {
            return 100 + motcleNote;
        }
        if (alineaInteger == AlineaInteger.SOUSTITRE) {
            return 80 + motcleNote;
        }
        int category = AlineaInteger.getCategory(alineaInteger);
        switch (category) {
            case AlineaInteger.PRIMARYATTRIBUTE_CATEGORY:
                return 60 + motcleNote;
            case AlineaInteger.COMPLEMENT_CATEGORY:
                if (motcleNote > 0) {
                    return 50 + motcleNote;
                }
                return 40;
            case AlineaInteger.SECONDARYATTRIBUTE_CATEGORY:
                if (motcleNote > 0) {
                    return 40 + motcleNote;
                }
                return 20;
            default:
                return 0;
        }
    }


}
