/* ScrutariLib_SearchEngine - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.seeker.lexie;

import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.localisation.LocalisationLexieFilters;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.SubOperand;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.text.collation.map.ValueFilter;
import net.mapeadores.util.text.search.MultiSearchToken;
import net.mapeadores.util.text.search.SearchToken;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.operands.search.SearchOperationReduction;
import net.scrutari.searchengine.api.operands.search.SearchTokenOperand;
import net.scrutari.searchengine.api.seeker.SeekHandler;
import net.scrutari.searchengine.api.seeker.SeekOptions;
import net.scrutari.searchengine.tools.operands.search.SearchOperationReductionBuilder;


/**
 *
 * @author Vincent Calame
 */
public class LexieSeeker {

    private LexieAccess lexieAccess;
    private LinkedIntHashMap ficheMap = new LinkedIntHashMap();
    private LinkedIntHashMap motcleMap = new LinkedIntHashMap();
    private SeekOptions seekOptions;
    private SeekHandler seekHandler;
    private ValueFilter valueFilter = null;

    public LexieSeeker(SeekOptions seekOptions, SeekHandler seekHandler, LexieAccess lexieAccess) {
        this.lexieAccess = lexieAccess;
        this.seekOptions = seekOptions;
        this.seekHandler = seekHandler;
    }

    public void seek(SearchOperation searchSequence) {
        int[] langIntegerArray = seekOptions.getLangIntegerArray();
        for (int k = 0; k < langIntegerArray.length; k++) {
            int langInteger = langIntegerArray[k];
            SearchOperationReduction operationReduction = SearchOperationReductionBuilder.lexieReduction(searchSequence, LocalisationLexieFilters.getLexieFilter(langInteger), seekOptions.getDefaultSearchType());
            ficheMap.clear();
            motcleMap.clear();
            Operand operand = operationReduction.getRootOperand();
            if (operand == null) {
                continue;
            }
            seekOperand(operand, langInteger);
            int unusedCount = operationReduction.getUnusedNumberCount();
            if (unusedCount > 0) {
                int[] unusedArray = new int[unusedCount];
                for (int i = 0; i < unusedCount; i++) {
                    unusedArray[i] = operationReduction.getUnusedNumber(i);
                }
                int ficheMapSize = ficheMap.size();
                for (int i = 0; i < ficheMapSize; i++) {
                    int codefiche = ficheMap.getKeyAt(i);
                    int codecorpus = ((Integer) ficheMap.getValueAt(i)).intValue();
                    for (int j = 0; j < unusedCount; j++) {
                        seekHandler.addAlineaOccurrence(codefiche, codecorpus, unusedArray[j], -1, null);
                    }
                }
                int motcleMapSize = motcleMap.size();
                for (int i = 0; i < motcleMapSize; i++) {
                    int codemotcle = motcleMap.getKeyAt(i);
                    int codethesaurus = ((Integer) motcleMap.getValueAt(i)).intValue();
                    for (int j = 0; j < unusedCount; j++) {
                        seekHandler.addLangOccurrence(codemotcle, codethesaurus, unusedArray[j], langInteger, null, 0.0f);
                    }
                }
            }
        }
    }

    private void seekOperand(Operand operand, int langInteger) {
        if (operand instanceof SubOperand) {
            seekSubOperand((SubOperand) operand, langInteger);
        } else {
            SearchTokenOperand searchOperand = (SearchTokenOperand) operand;
            AlineaFilter alineaFilter = searchOperand.getAlineaFilter();
            int operandNumber = searchOperand.getOperandNumber();
            SearchToken token = searchOperand.getSearchToken();
            if (token instanceof MultiSearchToken) {
                MultiSearchToken multiToken = (MultiSearchToken) token;
                int count = multiToken.getSubtokenCount();
                if (count > 1) {
                    MultiTokenSeeker.seek(multiToken, operandNumber, langInteger, this, alineaFilter);
                } else if (count == 1) {
                    SimpleTokenSeeker.seek(multiToken.getSubtoken(0), operandNumber, langInteger, this, alineaFilter);
                }
            } else {
                SimpleTokenSeeker.seek((SimpleSearchToken) token, operandNumber, langInteger, this, alineaFilter);
            }
        }
    }

    private void seekSubOperand(SubOperand subOperand, int langInteger) {
        int count = subOperand.getOperandCount();
        for (int i = 0; i < count; i++) {
            seekOperand(subOperand.getOperand(i), langInteger);
        }
    }

    LexieAccess getLexieAccess() {
        return lexieAccess;
    }

    SeekOptions getSeekOptions() {
        return seekOptions;
    }

    SeekHandler getSeekHandler() {
        return seekHandler;
    }

    void addFiche(int codefiche, int codecorpus) {
        ficheMap.put(codefiche, new Integer(codecorpus));
    }

    void addMotcle(int codemotcle, int codethesaurus) {
        motcleMap.put(codemotcle, new Integer(codethesaurus));
    }

    SearchResultUnitList search(String searchText, int langInteger, int type) {
        CollatedKeyMap collatedStringMap = lexieAccess.getCollatedLexieMapByLang(langInteger);
        if (collatedStringMap == null) {
            return null;
        }
        return collatedStringMap.search(searchText, type, valueFilter);
    }

}
