/* SctServer - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver;

import fr.exemole.sctserver.admin.OptionsManager;
import fr.exemole.sctserver.admin.ScrutariInfoManager;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.collect.CollectErrorHandler;
import fr.exemole.sctserver.api.collect.ScrutariInfo;
import fr.exemole.sctserver.api.conf.ConfConstants;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.conf.ConfFileInfo;
import fr.exemole.sctserver.api.conf.ConfIOException;
import fr.exemole.sctserver.api.feed.SctFeedCache;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.tools.AddURITreeProvider;
import fr.exemole.sctserver.tools.feed.SctFeedInfoBuilder;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.ProcessMessage;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.ProcessMessageLogBuilder;
import net.mapeadores.util.logging.TimeLog;
import net.mapeadores.util.logicaloperation.LogicalOperationParser;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.OperandException;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.ScrutariDBName;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.codes.ScrutariDataURICodeList;
import net.scrutari.db.tools.AttributeKeyCoderBuilder;
import net.scrutari.db.tools.DataCoderBuilder;
import net.scrutari.searchengine.FicheSearchEngine;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.operands.search.QInfo;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.tools.operands.filters.OperandConverterParameters;
import net.scrutari.searchengine.tools.operands.search.SearchOperationBuilder;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import net.scrutari.supermotcle.api.SupermotcleProvider;
import net.scrutari.supermotcle.tools.SupermotcleProviderBuilder;
import net.scrutari.xml.ScrutariXmlErrorHandler;
import net.scrutari.xml.XmlPullReader;
import org.xml.sax.SAXException;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;


/**
 *
 * @author Vincent Calame
 */
public final class SctServer implements ConfConstants {

    public final static String MALFORMED_XML_CATEGORY = "MALFORMED_XML";
    public final static String SEVERE_CATEGORY = "SEVERE";
    public final static String WARNING_CATEGORY = "WARNING";
    public final static short INIT_TYPE = 1;
    public final static short FORCE_TYPE = 2;
    public final static short CRON_TYPE = 3;
    public final static short UPDATECONF_TYPE = 4;
    private final String name;
    private final SctServerStorage sctServerStorage;
    private final ScrutariInfoManager scrutariInfoManager;
    private final OptionsManager optionsManager;
    private InternalScrutariSession scrutariSession;
    private InternalConfState confState = new InternalConfState();
    private boolean onReload = false;

    public SctServer(String name, SctServerStorage sctServerStorage) {
        this.name = name;
        this.sctServerStorage = sctServerStorage;
        this.scrutariInfoManager = new ScrutariInfoManager(sctServerStorage);
        this.optionsManager = new OptionsManager(this);
        reload(INIT_TYPE);
    }

    public String getName() {
        return name;
    }

    public boolean reload(short type) {
        if (onReload) {
            return false;
        }
        onReload = true;
        Reloader reloader = new Reloader(type);
        reloader.reload();
        onReload = false;
        return true;
    }

    public synchronized ScrutariSession getScrutariSession() {
        return scrutariSession;
    }

    public ScrutariInfoManager getScrutariInfoManager() {
        return scrutariInfoManager;
    }

    public OptionsManager getOptionsManager() {
        return optionsManager;
    }

    public SctServerStorage getSctServerStorage() {
        return sctServerStorage;
    }

    public ConfState getConfState() {
        return confState;
    }

    private void reinitScrutariDB(UpdateLogHandler updateProcessLog, ScrutariDataURICodeList initialScrutariDataURICodeList, Map<String, AttributeDefList> attributeDefListMap) {
        TimeLog timeLog = new TimeLog();
        ScrutariDB newScrutariDB = newScrutariDB(updateProcessLog, initialScrutariDataURICodeList, attributeDefListMap, timeLog);
        Set<ScrutariDBName> remainingSet = new HashSet<ScrutariDBName>();
        remainingSet.add(newScrutariDB.getScrutariDBName());
        SupermotcleProvider supermotcleProvider = SupermotcleProviderBuilder.build(newScrutariDB);
        timeLog.log("supermotcle_build");
        InternalScrutariSession newScrutariSession = new InternalScrutariSession(this, newScrutariDB, supermotcleProvider);
        InternalScrutariSession oldScrutariSession = scrutariSession;
        if (oldScrutariSession != null) {
            remainingSet.add(oldScrutariSession.getScrutariDB().getScrutariDBName());
        }
        synchronized (this) {
            scrutariSession = newScrutariSession;
        }
        sctServerStorage.cleanCache(remainingSet);
        timeLog.log("update_end");
        sctServerStorage.getLogStorage().log(SctLogConstants.UPDATE_LOG, SctLogConstants.TIME_UPDATE_LOG, timeLog.toString());
    }

    private ScrutariDB newScrutariDB(UpdateLogHandler updateProcessLog, ScrutariDataURICodeList initialScrutariDataURICodeList, Map<String, AttributeDefList> attributeDefListMap, TimeLog timeLog) {
        ScrutariDBName scrutariDbName = ScrutariDBName.newName();
        URITree initialUriTree = sctServerStorage.getURITree(null);
        DataCoder dataCoder = DataCoderBuilder.newInstance(initialScrutariDataURICodeList, initialUriTree);
        AttributeKeyCoder attributeKeyCoder = AttributeKeyCoderBuilder.build(attributeDefListMap);
        ScrutariDB.Builder builder = ScrutariDB.newBuilder(scrutariDbName, dataCoder, attributeKeyCoder, sctServerStorage.getDataCacheWriter(scrutariDbName), sctServerStorage.getLexieCacheWriter(scrutariDbName));
        builWithXmlPull(builder, updateProcessLog, timeLog);
        ScrutariDB newScrutariDB = builder.toScrutariDB();
        timeLog.log("db_build");
        newScrutariDB.updateStats();
        timeLog.log("db_stats");
        sctServerStorage.saveURITree(dataCoder.toURITree(), null);
        if (dataCoder.hasNewURI()) {
            sctServerStorage.saveURITree(dataCoder.getNewURITree(), scrutariDbName);
        }
        return newScrutariDB;
    }

    private void builWithXmlPull(ScrutariDB.Builder builder, UpdateLogHandler updateProcessLog, TimeLog timeLog) {
        XmlPullReader xmlPullReader = new XmlPullReader(builder, updateProcessLog);
        int count = scrutariInfoManager.getScrutariInfoCount();
        for (int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = scrutariInfoManager.getScrutariInfo(i);
            URL url = scrutariInfo.getUrl();
            String urlString = url.toString();
            updateProcessLog.setCurrentURL(url);
            timeLog.addSection(urlString);
            parseXmlPull(xmlPullReader, scrutariInfo, updateProcessLog);
            timeLog.log("data_parsed");
        }
        xmlPullReader.flush();
    }

    private void parseXmlPull(XmlPullReader xmlPullReader, ScrutariInfo scrutariInfo, UpdateLogHandler updateProcessLog) {
        InputStream inputStream = sctServerStorage.getScrutariData(scrutariInfo);
        if (inputStream == null) {
            return;
        }
        XmlPullParser xpp = null;
        try {
            XmlPullParserFactory factory = XmlPullParserFactory.newInstance(System.getProperty(XmlPullParserFactory.PROPERTY_NAME), null);
            xpp = factory.newPullParser();
        } catch (XmlPullParserException xppe) {
            throw new NestedLibraryException(xppe);
        }
        try {
            try {
                xpp.setInput(inputStream, null);
                BaseURI baseURI = xmlPullReader.process(xpp);
                scrutariInfoManager.setBaseURI(scrutariInfo, baseURI);

            } catch (XmlPullParserException xppe) {
                scrutariInfoManager.setState(scrutariInfo, ScrutariInfo.DATA_NOTXML_STATE);
                updateProcessLog.xmlPullParserException(xppe);
            } catch (EOFException ioe) {
                scrutariInfoManager.setState(scrutariInfo, ScrutariInfo.DATA_NOTXML_STATE);
                updateProcessLog.xmlPullParserException(ioe);
            } finally {
                inputStream.close();
            }

        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }

    }


    private class Reloader {

        private short type;
        private ConfDirectory confDirectory;

        Reloader(short type) {
            this.type = type;
        }

        public void reload() {
            initConfDirectory();
            scrutariInfoManager.reloadInfoUrlArray(confDirectory.getInfoUrlKeyArray());
            if (type != UPDATECONF_TYPE) {
                UpdateLogHandler updateLogHandler = new UpdateLogHandler();
                boolean forceReload = (type == FORCE_TYPE);
                boolean done = scrutariInfoManager.collect(updateLogHandler, forceReload);
                if ((type != CRON_TYPE) || (done)) {
                    reinitScrutariDB(updateLogHandler, confDirectory.getInitialScrutariDataURICodeList(), confDirectory.getAttributeDefListMap());
                    sctServerStorage.getLogStorage().log(SctLogConstants.UPDATE_LOG, SctLogConstants.DATA_UPDATE_LOG, updateLogHandler.toContentString());
                }
                scrutariInfoManager.store();
            }
            optionsManager.reloadCorpusClasseArray(confDirectory.getCorpusClasseArray());
            PertinencePonderation globalPertinencePonderation = confDirectory.getGlobalPertinencePonderation();
            if (globalPertinencePonderation != null) {
                optionsManager.setGlobalPertinencePonderation(globalPertinencePonderation);
            } else {
                optionsManager.setGlobalPertinencePonderation(PertinencePonderationBuilder.DEFAULT_PONDERATION);
            }
        }

        private void initConfDirectory() {
            confDirectory = sctServerStorage.getConfDirectory();
            try {
                confDirectory.reload();
                checkConfLog(SOURCES_NAME);
                checkConfLog(ATTRIBUTES_NAME);
                checkConfLog(CORPUS_CLASSES_NAME);
                checkConfLog(URI_CODES_NAME);
                checkConfLog(FEED_NAME);
                confState.setOkState();
            } catch (ConfIOException cioe) {
                confState.setGlobalConfigException(cioe);
            }
        }

        private void checkConfLog(String name) {
            ConfFileInfo confFileInfo = confDirectory.getConfFileInfo(name);
            if (!confFileInfo.isHere()) {
                confState.setFileState(name, FILE_MISSING);
            } else {
                ProcessMessageList processMessageList = confFileInfo.getProcessMessageList();
                int count = processMessageList.getProcessMessageCount();
                if (count > 0) {
                    int errorType = 0;
                    for (int i = 0; i < count; i++) {
                        ProcessMessage processMessage = processMessageList.getProcessMessage(i);
                        String category = processMessage.getCategory();
                        if (category.equals(MALFORMED_XML_CATEGORY)) {
                            errorType = 2;
                            break;
                        } else if (category.equals(SEVERE_CATEGORY)) {
                            errorType = 1;
                        }
                    }
                    if (errorType == 2) {
                        confState.setFileState(name, FILE_XML_MALFORMED);
                    } else if (errorType == 1) {
                        confState.setFileState(name, FILE_XML_ERROR);
                    } else {
                        confState.setFileState(name, FILE_XML_WARNING);
                    }
                } else {
                    confState.setFileState(name, FILE_OK);
                }
                StringBuilder buf = new StringBuilder();
                try {
                    LogUtils.append(buf, confFileInfo.getProcessURI(), processMessageList);
                } catch (IOException ioe) {
                }
                sctServerStorage.getLogStorage().log(SctLogConstants.CONF_LOG, name, buf.toString());
            }

        }

    }


    private static class UpdateLogHandler implements ScrutariXmlErrorHandler, CollectErrorHandler {

        private ProcessMessageLogBuilder builder = new ProcessMessageLogBuilder();

        private UpdateLogHandler() {
        }

        public void setCurrentURL(URL url) {
            builder.setCurrentURI(url.toString());
        }

        public void saxException(SAXException saxe) {
            builder.addMessage(MALFORMED_XML_CATEGORY, SctServerLog.SEVERE_DATA_BASEFILE_XML, saxe.getMessage());
        }

        public void xmlPullParserException(Exception xppe) {
            builder.addMessage(MALFORMED_XML_CATEGORY, SctServerLog.SEVERE_DATA_BASEFILE_XML, xppe.getMessage());
        }

        @Override
        public void addSevereLog(String key, ScrutariDataURI mapeadoresURI) {
            builder.addMessage(SEVERE_CATEGORY, key, mapeadoresURI.toString());
        }

        @Override
        public void addWarningLog(String key, ScrutariDataURI mapeadoresURI) {
            builder.addMessage(WARNING_CATEGORY, key, mapeadoresURI.toString());
        }

        @Override
        public void addSevereLog(String key) {
            addSevereLog(key, null);
        }

        @Override
        public void addWarningLog(String key) {
            addWarningLog(key, null);
        }

        @Override
        public void warningDownload(URL url, String message) {
            setCurrentURL(url);
            builder.addMessage(WARNING_CATEGORY, "warning.download", message);
        }

        private String toContentString() {
            return LogUtils.toString(builder.toProcessMessageLog());
        }

    }


    private class InternalConfState implements ConfState {

        private short state = GLOBAL_UNDEFINED;
        private String confIOMessage = null;
        private Map<String, Short> stateMap = new HashMap<String, Short>();

        private InternalConfState() {
        }

        private void setGlobalConfigException(ConfIOException ioe) {
            state = GLOBAL_CONFIO_ERROR;
            confIOMessage = ioe.getFullMessage();
        }

        @Override
        public String getConfIOMessage() {
            return confIOMessage;
        }

        @Override
        public short getGlobalState() {
            return state;
        }

        void setOkState() {
            this.state = GLOBAL_OK;
        }

        void setFileState(String name, short fileState) {
            stateMap.put(name, new Short(fileState));
        }

        @Override
        public short getFileState(String name) {
            Short sh = stateMap.get(name);
            if (sh == null) {
                return FILE_STATE_UNKNOWN;
            }
            return sh.shortValue();
        }

    }


    private static class InternalScrutariSession implements ScrutariSession {

        private final SctServer sctServer;
        private final ScrutariDB scrutariDB;
        private final SupermotcleProvider supermotcleProvider;
        private final SctFeedInfo sctFeedInfo;
        private final SctFeedCache sctFeedCache;
        private int subid = 1;

        private InternalScrutariSession(SctServer sctServer, ScrutariDB scrutariDB, SupermotcleProvider supermotcleProvider) {
            this.sctServer = sctServer;
            this.scrutariDB = scrutariDB;
            this.supermotcleProvider = supermotcleProvider;
            ScrutariDBName scrutariDBName = scrutariDB.getScrutariDBName();
            this.sctFeedInfo = SctFeedInfoBuilder.build(sctServer, scrutariDBName);
            this.sctFeedCache = sctServer.getSctServerStorage().getSctFeedCache(scrutariDBName);
        }

        @Override
        public String getSctServerName() {
            return sctServer.getName();
        }

        @Override
        public SctServer getSctServer() {
            return sctServer;
        }

        @Override
        public ScrutariDB getScrutariDB() {
            return scrutariDB;
        }

        @Override
        public SupermotcleProvider getSupermotcleProvider() {
            return supermotcleProvider;
        }

        @Override
        public SctFeedInfo getSctFeedInfo() {
            return sctFeedInfo;
        }

        @Override
        public SctFeedCache getSctFeedCache() {
            return sctFeedCache;
        }

        @Override
        public GlobalSearchOptions getGlobalSearchOptions() {
            return sctServer.optionsManager.getGlobalSearchOptions();
        }

        @Override
        public FicheSearchResult newSearch(SearchOperation searchOperation, SearchOptions searchOptions) {
            ScrutariDBName scrutariDBName = scrutariDB.getScrutariDBName();
            QId qId;
            synchronized (this) {
                qId = new QId(scrutariDBName, subid);
                subid++;
            }
            DataAccess dataAccess = scrutariDB.openDataAccess();
            FicheSearchResult ficheSearchResult = FicheSearchEngine.search(scrutariDB, searchOperation, searchOptions, getGlobalSearchOptions(), qId);
            sctServer.sctServerStorage.getFicheSearchCache(scrutariDBName).saveFicheSearchResult(ficheSearchResult);
            sctServer.sctServerStorage.getQIdStorage().saveFicheSearchSource(dataAccess, ficheSearchResult.getFicheSearchSource());
            dataAccess.close();
            return ficheSearchResult;
        }

        @Override
        public FicheSearchResult getSearch(QId qId) {
            FicheSearchResult ficheSearchResult = sctServer.sctServerStorage.getFicheSearchCache(scrutariDB.getScrutariDBName()).getFicheSearchResult(qId);
            if (ficheSearchResult != null) {
                return ficheSearchResult;
            }
            DataAccess dataAccess = scrutariDB.openDataAccess();
            OperandConverterParameters operandConverterParameters = new OperandConverterParameters();
            operandConverterParameters.setDataAccess(dataAccess);
            operandConverterParameters.setFicheSearchResultHolder(this);
            operandConverterParameters.setProcessMessageHandler(LogUtils.NULL_MULTIPROCESSMESSAGEHANDLER);
            operandConverterParameters.setUriTreeProvider(new AddURITreeProvider(sctServer.sctServerStorage));
            FicheSearchSource ficheSearchSource = sctServer.sctServerStorage.getQIdStorage().getFicheSearchSource(operandConverterParameters, qId);
            if (ficheSearchSource == null) {
                dataAccess.close();
                return null;
            }
            SearchOptions searchOptions = ficheSearchSource.getSearchOptions();
            if (searchOptions == null) {
                searchOptions = SearchOptionsBuilder.EMPTY_SEARCHOPTIONS;
            }
            QInfo qInfo = ficheSearchSource.getQInfo();
            SearchOperation searchOperation = null;
            Operand operand;
            try {
                operand = LogicalOperationParser.parseInformatique(qInfo.getQ());
                if (operand != null) {
                    searchOperation = SearchOperationBuilder.fromSimpleOperandOperation(operand, qInfo.getQSource(), LogUtils.NULL_MULTIPROCESSMESSAGEHANDLER, scrutariDB.getAttributeKeyCoder());
                }
            } catch (OperandException pe) {
            }
            if (searchOperation == null) {
                dataAccess.close();
                return null;
            }
            ficheSearchResult = FicheSearchEngine.search(scrutariDB, searchOperation, searchOptions, getGlobalSearchOptions(), qId);
            sctServer.sctServerStorage.getFicheSearchCache(scrutariDB.getScrutariDBName()).saveFicheSearchResult(ficheSearchResult);
            dataAccess.close();
            return ficheSearchResult;
        }

    }

}
