/* SctServer - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.tools.AddURITreeProvider;
import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.langinteger.LangIntegerUtils;
import net.mapeadores.util.localisation.MessageListBuilder;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.LogicalOperationParser;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.OperandException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.TextConstants;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.codes.CodeParser;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.operands.OperandsConstants;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.tools.OperandCheck;
import net.scrutari.searchengine.tools.operands.filters.FilterOperandConverterEngine;
import net.scrutari.searchengine.tools.operands.filters.OperandConverterParameters;
import net.scrutari.searchengine.tools.operands.search.SearchOperationBuilder;
import net.scrutari.searchengine.tools.options.OptionsUtils;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import net.scrutari.supermotcle.api.Supermotcle;
import net.scrutari.supermotcle.api.SupermotcleMap;


/**
 *
 * @author Vincent Calame
 */
public final class RequestMapUtils implements Parameters {

    private RequestMapUtils() {
    }

    public static int[] getLangIntegerArray(RequestMap requestMap) {
        String param_langs = requestMap.getParameter(LANG_LIST);
        if (param_langs == null) {
            param_langs = requestMap.getParameter("langs");
        }
        if (param_langs == null) {
            return null;
        }
        int[] langIntegerArray = LangIntegerUtils.toCleanLangIntegerArray(param_langs);
        if (langIntegerArray.length == 0) {
            return null;
        }
        return langIntegerArray;
    }

    public static int getLangUi(RequestMap requestMap, WarningHandler warningHandler) {
        String langUi = requestMap.getParameter(LANG_UI);
        if (langUi == null) {
            return LangInteger.UNDETERMINED;
        }
        try {
            int langInteger = LangInteger.parse(langUi);
            return langInteger;
        } catch (LangIntegerException lie) {
            warningHandler.setCurrentParameter(LANG_UI);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, langUi);
            return LangInteger.UNDETERMINED;
        }
    }

    public static int getLimit(RequestMap requestMap, WarningHandler warningHandler) {
        String limit = requestMap.getParameter(LIMIT);
        if (limit == null) {
            return -1;
        }
        try {
            int limitInteger = Integer.parseInt(limit);
            if (limitInteger > 0) {
                return limitInteger;
            } else {
                return -1;
            }
        } catch (NumberFormatException nfe) {
            warningHandler.setCurrentParameter(LIMIT);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, limit);
            return -1;
        }
    }

    public static int getStart(RequestMap requestMap, WarningHandler warningHandler) {
        String start = requestMap.getParameter(START);
        if (start == null) {
            return - 1;
        }
        try {
            int globalStartInteger = Integer.parseInt(start);
            if (globalStartInteger > 0) {
                return globalStartInteger;
            } else {
                return -1;
            }
        } catch (NumberFormatException nfe) {
            warningHandler.setCurrentParameter(START);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, start);
            return -1;
        }
    }

    public static String getQIdString(RequestMap requestMap) {
        String qIdString = requestMap.getParameter(Q_ID);
        if (qIdString == null) {
            qIdString = requestMap.getParameter("idrecherche");
        }
        return qIdString;
    }

    public static SearchOperation getSearchOperation(RequestMap requestMap, AttributeKeyCoder attributeKeyCoder) throws ErrorMessageException {
        String q = requestMap.getParameter(Q);
        if (q == null) {
            q = requestMap.getParameter("recherche");
        }
        if (q == null) {
            throw new ErrorMessageException(ErrorKeys.MISSING_PARAMETER, Q);
        }
        q = q.trim();
        if (q.length() == 0) {
            throw new ErrorMessageException(ErrorKeys.EMPTY_PARAMETER, Q);
        }
        String modeString = requestMap.getParameter(Q_MODE);
        if (modeString == null) {
            modeString = INTERSECTION_Q_MODE;
        }
        if (modeString.equals(OPERATION_Q_MODE)) {
            Operand operand;
            try {
                operand = LogicalOperationParser.parseInformatique(q);
                if (operand == null) {
                    throw new ErrorMessageException(ErrorKeys.EMPTY_PARAMETER, LogUtils.toValue(Q, q));
                }
            } catch (OperandException pe) {
                throw new ErrorMessageException(ErrorKeys.BAD_OPERATION_SYNTAX, LogUtils.toValue(Q, q + "(" + pe.getErrorKey() + " / " + pe.getCharIndex() + ")"));
            }
            SearchProcessMessageHandler handler = new SearchProcessMessageHandler(Q, q);
            SearchOperation searchOperation = SearchOperationBuilder.fromSimpleOperandOperation(operand, q, handler, attributeKeyCoder);
            if (searchOperation == null) {
                throw new ErrorMessageException(handler.toErrorMessage());
            }
            return searchOperation;
        } else {
            short operator;
            if (modeString.equals(UNION_Q_MODE)) {
                operator = LogicalOperationConstants.UNION_OPERATOR;
            } else {
                operator = LogicalOperationConstants.INTERSECTION_OPERATOR;
            }
            SearchSequence searchSequence = SearchSequence.createSearchSequence(q);
            if (searchSequence == null) {
                throw new ErrorMessageException(ErrorKeys.EMPTY_PARAMETER, LogUtils.toValue(Q, q));
            }
            return SearchOperationBuilder.fromSearchSequence(searchSequence, operator);
        }
    }

    public static SearchOptionsBuilder getSearchOptionsBuilder(RequestMap requestMap, WarningHandler warningHandler) {
        SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder();
        int langUi = getLangUi(requestMap, warningHandler);
        searchOptionsBuilder.setLangUi(langUi);
        int[] langIntegerArray = getLangIntegerArray(requestMap);
        searchOptionsBuilder.setLangIntegerArray(langIntegerArray);
        searchOptionsBuilder.setPertinencePonderation(getPertinencePonderation(requestMap, warningHandler));
        return searchOptionsBuilder;
    }

    private static PertinencePonderation getPertinencePonderation(RequestMap requestMap, WarningHandler warningHandler) {
        String ponderationString = requestMap.getParameter(PONDERATION);
        if (ponderationString == null) {
            return null;
        }
        warningHandler.setCurrentParameter(PONDERATION);
        String[] ponderationTokenArray = StringUtils.getTechnicalTokens(ponderationString, true);
        if (ponderationTokenArray.length != 3) {
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, ponderationString);
            return null;
        }
        try {
            int occurrence = Integer.parseInt(ponderationTokenArray[0]);
            int date = Integer.parseInt(ponderationTokenArray[1]);
            int origin = Integer.parseInt(ponderationTokenArray[2]);
            return PertinencePonderationBuilder.toPertinencePonderation(occurrence, date, origin);
        } catch (NumberFormatException nfe) {
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, ponderationString);
            return null;
        }
    }

    public static Supermotcle getSupermotcle(ScrutariSession scrutariSession, RequestMap requestMap) {
        String supermotcleString = requestMap.getParameter("supermotcle");
        if (supermotcleString == null) {
            return null;
        }
        int idx = supermotcleString.indexOf("/");
        if (idx == -1) {
            return null;
        }
        int langInteger;
        int code;
        try {
            langInteger = LangInteger.parse(supermotcleString, 0, idx, false);
        } catch (LangIntegerException lie) {
            return null;
        }
        try {
            code = Integer.parseInt(supermotcleString.substring(idx + 1));
        } catch (NumberFormatException nfe) {
            return null;
        }
        SupermotcleMap supermotcleMap = scrutariSession.getSupermotcleProvider().getSupermotcleMap(langInteger);
        if (supermotcleMap == null) {
            return null;
        }
        Supermotcle supermotcle = supermotcleMap.getSupermotcleByCode(code);
        if (supermotcle == null) {
            return null;
        }
        return supermotcle;
    }

    public static OperandCheck getGeneralOperandCheck(RequestMap requestMap, CodeList corpusCodeList, WarningHandler warningHandler, OperandConverterParameters operandConverterParameters) {
        String filterString = requestMap.getParameter(FILTER);
        if (filterString == null) {
            return null;
        }
        warningHandler.setCurrentParameter(FILTER);
        Operand operand = parseOperand(filterString, warningHandler);
        if (operand == null) {
            return null;
        }
        OperandCheck operandCheck = FilterOperandConverterEngine.convert(operand, corpusCodeList, null, operandConverterParameters);
        if (operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
            return null;
        } else {
            return operandCheck;
        }
    }

    public static OperandCheck getPreviousResultOperandCheck(RequestMap requestMap, CodeList corpusCodeList, WarningHandler warningHandler, OperandConverterParameters operandConverterParameters) {
        String qidString = requestMap.getParameter(QID_FILTER);
        if (qidString == null) {
            return null;
        }
        warningHandler.setCurrentParameter(QID_FILTER);
        Operand operand = parseOperand(qidString, warningHandler);
        if (operand == null) {
            return null;
        }
        OperandCheck operandCheck = FilterOperandConverterEngine.convert(operand, corpusCodeList, OperandsConstants.QID_TYPE, operandConverterParameters);
        if (operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
            return null;
        } else {
            return operandCheck;
        }
    }

    public static OperandCheck getDateOperandCheck(RequestMap requestMap, CodeList corpusCodeList, WarningHandler warningHandler, OperandConverterParameters operandConverterParameters) {
        String dateFilterString = requestMap.getParameter(DATE_FILTER);
        if (dateFilterString == null) {
            return null;
        }
        warningHandler.setCurrentParameter(DATE_FILTER);
        Operand operand = parseOperand(dateFilterString, warningHandler);
        if (operand == null) {
            return null;
        }
        OperandCheck operandCheck = FilterOperandConverterEngine.convert(operand, corpusCodeList, OperandsConstants.DATE_TYPE, operandConverterParameters);
        if (operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
            return null;
        } else {
            return operandCheck;
        }
    }

    public static OperandCheck getIndexationOperandeCheck(RequestMap requestMap, CodeList corpusCodeList, WarningHandler warningHandler, OperandConverterParameters operandConverterParameters) {
        String indexationFilterString = requestMap.getParameter(INDEXATION_FILTER);
        if (indexationFilterString == null) {
            indexationFilterString = requestMap.getParameter("indexation");
        }
        if (indexationFilterString == null) {
            return null;
        }
        indexationFilterString = indexationFilterString.trim();
        if (indexationFilterString.length() == 0) {
            return null;
        }
        warningHandler.setCurrentParameter(INDEXATION_FILTER);
        Operand operand = parseOperand(indexationFilterString, warningHandler);
        if (operand == null) {
            return null;
        }
        OperandCheck operandCheck = FilterOperandConverterEngine.convert(operand, corpusCodeList, OperandsConstants.MOTCLE_TYPE, operandConverterParameters);
        if (operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
            return null;
        } else {
            return operandCheck;
        }
    }

    public static OperandCheck getAddOperandCheck(RequestMap requestMap, CodeList corpusCodeList, WarningHandler warningHandler, OperandConverterParameters operandConverterParameters) {
        String addFilterString = requestMap.getParameter(ADD_FILTER);
        if (addFilterString == null) {
            return null;
        }
        warningHandler.setCurrentParameter(ADD_FILTER);
        Operand operand = parseOperand(addFilterString, warningHandler);
        if (operand == null) {
            return null;
        }
        OperandCheck operandCheck = FilterOperandConverterEngine.convert(operand, corpusCodeList, OperandsConstants.ADD_TYPE, operandConverterParameters);
        if (operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
            return null;
        } else {
            return operandCheck;
        }
    }

    public static String getOrigin(RequestMap requestMap) {
        String origin = requestMap.getParameter(ORIGIN);
        if (origin == null) {
            origin = requestMap.getParameter("site");
        }
        if (origin == null) {
            return "";
        } else {
            return origin;
        }
    }

    public static SearchOptions getSearchOptions(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler, boolean reduceThesaurus) {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SearchOptionsBuilder searchOptionsBuilder = RequestMapUtils.getSearchOptionsBuilder(requestMap, warningHandler);
        CodeList thesaurusCodeList = getThesaurusCodeList(dataAccess, requestMap, warningHandler);
        if (thesaurusCodeList != null) {
            searchOptionsBuilder.reduceThesaurusList(thesaurusCodeList);
            searchOptionsBuilder.putScrutariDataURIList(OptionsUtils.toScrutariDataURIList(dataAccess, ScrutariDataURI.THESAURUSURI_TYPE, thesaurusCodeList));
        }
        CodeList corpusCodeList = getCorpusCodeList(dataAccess, requestMap, warningHandler);
        if (corpusCodeList != null) {
            searchOptionsBuilder.reduceCorpusList(corpusCodeList);
            searchOptionsBuilder.putScrutariDataURIList(OptionsUtils.toScrutariDataURIList(dataAccess, ScrutariDataURI.CORPUSURI_TYPE, corpusCodeList));
        }
        OperandConverterParameters operandConverterParameters = new OperandConverterParameters();
        operandConverterParameters.setDataAccess(dataAccess);
        operandConverterParameters.setFicheSearchResultHolder(scrutariSession);
        operandConverterParameters.setProcessMessageHandler(warningHandler.getCodeProcessMessageHandler());
        operandConverterParameters.setUriTreeProvider(new AddURITreeProvider(scrutariSession.getSctServer().getSctServerStorage()));
        OperandCheck indexationOperandCheck = RequestMapUtils.getIndexationOperandeCheck(requestMap, corpusCodeList, warningHandler, operandConverterParameters);
        if (indexationOperandCheck != null) {
            searchOptionsBuilder.addFicheOperandCheck(indexationOperandCheck);
        }
        OperandCheck dateOperandCheck = RequestMapUtils.getDateOperandCheck(requestMap, corpusCodeList, warningHandler, operandConverterParameters);
        if (dateOperandCheck != null) {
            searchOptionsBuilder.addFicheOperandCheck(dateOperandCheck);
        }
        OperandCheck previousResultOperandCheck = RequestMapUtils.getPreviousResultOperandCheck(requestMap, corpusCodeList, warningHandler, operandConverterParameters);
        if (previousResultOperandCheck != null) {
            searchOptionsBuilder.addFicheOperandCheck(previousResultOperandCheck);
        }
        OperandCheck addOperandCheck = RequestMapUtils.getAddOperandCheck(requestMap, corpusCodeList, warningHandler, operandConverterParameters);
        if (addOperandCheck != null) {
            searchOptionsBuilder.addFicheOperandCheck(addOperandCheck);
        }
        OperandCheck generalOperandCheck = RequestMapUtils.getGeneralOperandCheck(requestMap, corpusCodeList, warningHandler, operandConverterParameters);
        if (generalOperandCheck != null) {
            searchOptionsBuilder.addFicheOperandCheck(generalOperandCheck);
        }
        Supermotcle supermotcle = RequestMapUtils.getSupermotcle(scrutariSession, requestMap);
        if (supermotcle != null) {
            //@todo : 
        }
        if (reduceThesaurus) {
            searchOptionsBuilder.reduceThesaurusList(OptionsUtils.getSelectionThesaurusCodeList(scrutariDB.getCountStats(), dataAccess, searchOptionsBuilder.getCurrentCorpusList()));
        }
        searchOptionsBuilder.setDefaultSearchType(TextConstants.STARTSWITH);
        dataAccess.close();
        SearchOptions searchOptions = searchOptionsBuilder.toSearchOptions();
        return searchOptions;
    }

    public static CodeList getCorpusCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(CORPUS_LIST);
        if (param == null) {
            param = requestMap.getParameter("codescorpus");
        }
        return toCodeList(dataAccess, param, warningHandler, "corpus:", ScrutariDataURI.CORPUSURI_TYPE);
    }

    public static CodeList getThesaurusCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(THESAURUS_LIST);
        return toCodeList(dataAccess, param, warningHandler, "thesaurus:", ScrutariDataURI.THESAURUSURI_TYPE);
    }

    public static CodeList getBaseCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(BASE_LIST);
        if (param == null) {
            param = requestMap.getParameter("codesbases");
        }
        return toCodeList(dataAccess, param, warningHandler, "base:", ScrutariDataURI.BASEURI_TYPE);
    }

    /**
     * Retourne null si la chaine est nulle ou blanche, si la chaine ne comprend
     * aucun caractère correct, retourne une liste vide
     */
    public static CodeList toCodeList(DataAccess dataAccess, String values, WarningHandler warningHandler, String uriBase, short uriType) {
        if (values == null) {
            return null;
        }
        values = values.trim();
        if (values.length() == 0) {
            return null;
        }
        String[] tokens = StringUtils.getTechnicalTokens(values, false);
        int length = tokens.length;
        if (length == 0) {
            warningHandler.addWarning(WarningKeys.ONLYSEPARATORS_PARAMETER_VALUE, values);
            return CodesUtils.EMPTY_CODELIST;
        }
        Set<Integer> codeSet = new LinkedHashSet<Integer>();
        for (int i = 0; i < length; i++) {
            int code = toCode(dataAccess, tokens[i], warningHandler, uriBase, uriType);
            if (code != -1) {
                codeSet.add(code);
            }
        }
        return CodesUtils.toCodeList(codeSet);
    }

    private static int toCode(DataAccess dataAccess, String s, WarningHandler warningHandler, String uriBase, short uriType) {
        return CodeParser.toCode(dataAccess, s, warningHandler.getCodeProcessMessageHandler(), uriBase, uriType);
    }

    public static IntitulesFilter getIntitulesFilter(RequestMap requestMap) {
        String intitules = requestMap.getParameter(INTITULES);
        if (intitules == null) {
            return null;
        }
        IntitulesFilter intitulesFilter = new IntitulesFilter();
        intitulesFilter.buildFromString(intitules);
        return intitulesFilter;
    }

    public static FicheFieldsFilter getFicheFieldsFilter(RequestMap requestMap) {
        FicheFieldsFilter ficheFieldsFilter = new FicheFieldsFilter();
        String ficheFields = requestMap.getParameter(FICHE_FIELDS);
        if (ficheFields == null) {
            ficheFields = requestMap.getParameter("prop");
        }
        if (ficheFields != null) {
            ficheFieldsFilter.buildFromString(ficheFields);
        }
        return ficheFieldsFilter;
    }

    public static MotcleFieldsFilter getMotcleFieldsFilter(RequestMap requestMap) {
        MotcleFieldsFilter motcleFieldsFilter = new MotcleFieldsFilter();
        String ficheFields = requestMap.getParameter(MOTCLE_FIELDS);
        if (ficheFields != null) {
            motcleFieldsFilter.buildFromString(ficheFields);
        }
        return motcleFieldsFilter;
    }

    public static CountFilter getCountFilter(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        CountFilter countFilter = new CountFilter();
        countFilter.setLangIntegerArray(getLangIntegerArray(requestMap));
        countFilter.setCorpusCodeList(getCorpusCodeList(dataAccess, requestMap, warningHandler));
        countFilter.setBaseCodeList(getBaseCodeList(dataAccess, requestMap, warningHandler));
        return countFilter;
    }

    private static Operand parseOperand(String operationString, WarningHandler warningHandler) {
        try {
            Operand operand = LogicalOperationParser.parseInformatique(operationString);
            return operand;
        } catch (OperandException pe) {
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, operationString + "(" + pe.getErrorKey() + " / " + pe.getCharIndex() + ")");
            return null;
        }
    }


    private static class SearchProcessMessageHandler implements ProcessMessageHandler {

        private String sourceString;
        private String parameter;
        private MessageListBuilder errorListBuilder = new MessageListBuilder();

        private SearchProcessMessageHandler(String parameter, String sourceString) {
            this.parameter = parameter;
            this.sourceString = sourceString;
        }

        @Override
        public void addMessage(String category, String key, Object value) {
            errorListBuilder.add(key, value);
        }

        private CommandMessage toErrorMessage() {
            return CommandMessage.error(ErrorKeys.BAD_OPERATION_SYNTAX, LogUtils.toValue(parameter, sourceString), errorListBuilder.toMessageList());

        }

    }

}
