/* SctServer - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.request.CountFilter;
import fr.exemole.sctserver.request.ErrorKeys;
import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.IntitulesFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import fr.exemole.sctserver.request.Parameters;
import static fr.exemole.sctserver.request.Parameters.Q_ID;
import fr.exemole.sctserver.request.RequestMapUtils;
import fr.exemole.sctserver.request.WarningHandler;
import fr.exemole.sctserver.request.WarningKeys;
import fr.exemole.sctserver.tools.log.SctLogUtils;
import java.text.ParseException;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.MotcleSearchEngine;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.SupermotcleSearchEngine;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.MotcleSearchResult;
import net.scrutari.searchengine.api.result.SupermotcleSearchResult;
import net.scrutari.searchengine.tools.options.OptionsUtils;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;


/**
 *
 * @author Vincent Calame
 */
public class JsonProducerFactory implements Parameters {

    public static JsonProducer getJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap) {
        WarningHandler warningHandler = new WarningHandler();
        AbstractJsonProducer abstractJsonProducer;
        try {
            abstractJsonProducer = getJsonProducer(scrutariSession, requestMap, warningHandler);
        } catch (ErrorMessageException eme) {
            abstractJsonProducer = new ErrorJsonProducer(eme.getErrorMessage());
        }
        if (!warningHandler.isEmpty()) {
            String warnings = requestMap.getParameter(WARNINGS);
            if ((warnings != null) && (warnings.equals("1"))) {
                abstractJsonProducer.setWarningHandler(warningHandler);
            }
        }
        return abstractJsonProducer;
    }

    private static AbstractJsonProducer getJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) throws ErrorMessageException {
        String type = requestMap.getParameter(TYPE);
        if (type == null) {
            throw new ErrorMessageException(ErrorKeys.MISSING_PARAMETER, TYPE);
        }
        if (type.isEmpty()) {
            throw new ErrorMessageException(ErrorKeys.EMPTY_PARAMETER, TYPE);
        }
        if (type.equals("motcle")) {
            return getMotcleDataJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("q-motcle")) {
            return getMotcleSearchResultJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("q-supermotcle")) {
            return getSupermotcleSearchResultJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("base")) {
            return getBaseDataJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("fiche")) {
            return getFicheDataJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("q-fiche")) {
            return getFicheSearchResultJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("corpus")) {
            return getCorpusDataJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("thesaurus")) {
            return getThesaurusDataJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("c-fiche")) {
            return getFicheCountJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("classe")) {
            return getClasseDefJsonProducer(scrutariSession, requestMap, warningHandler);
        } else if (type.equals("lang")) {
            return getLangJsonProducer(scrutariSession, requestMap, warningHandler);
        } else {
            throw new ErrorMessageException(ErrorKeys.UNKNOWN_PARAMETER_VALUE, toArray(TYPE, type));
        }
    }

    private static AbstractJsonProducer getSupermotcleSearchResultJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) throws ErrorMessageException {
        SearchOperation searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariSession.getScrutariDB().getAttributeKeyCoder());
        String languiString = requestMap.getParameter(LANG_UI);
        if (languiString == null) {
            throw new ErrorMessageException(ErrorKeys.MISSING_PARAMETER, LANG_UI);
        }
        if (languiString.length() == 0) {
            throw new ErrorMessageException(ErrorKeys.EMPTY_PARAMETER, LANG_UI);
        }
        int langui;
        try {
            langui = LangInteger.parse(languiString);
        } catch (LangIntegerException lie) {
            throw new ErrorMessageException(ErrorKeys.MALFORMED_PARAMETER_VALUE, toArray(LANG_UI, languiString));
        }
        SearchOptionsBuilder searchOptionsBuilder = RequestMapUtils.getSearchOptionsBuilder(requestMap, warningHandler);
        int[] tmp = new int[1];
        tmp[0] = langui;
        searchOptionsBuilder.setLangIntegerArray(tmp);
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        SupermotcleSearchResult searchResult = SupermotcleSearchEngine.search(scrutariSession.getSupermotcleProvider(), searchOperation, searchOptionsBuilder.toSearchOptions());
        return new SupermotcleSearchResultJsonProducer(searchResult, langui, limit);
    }

    private static AbstractJsonProducer getMotcleSearchResultJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) throws ErrorMessageException {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        SearchOperation searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariDB.getAttributeKeyCoder());
        SearchOptionsBuilder searchOptionsBuilder = RequestMapUtils.getSearchOptionsBuilder(requestMap, warningHandler);
        if (searchOptionsBuilder.getLangIntegerArray() == null) {
            //@todo message d'erreur
            return null;
        }
        int langInteger = checkUniquelang(searchOptionsBuilder);
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        MotcleSearchResult searchResult = MotcleSearchEngine.search(scrutariDB, searchOperation, searchOptionsBuilder.toSearchOptions());
        return new MotcleSearchResultJsonProducer(searchResult, scrutariDB, langInteger, limit);

    }

    private static AbstractJsonProducer getFicheSearchResultJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) throws ErrorMessageException {
        String qIdString = RequestMapUtils.getQIdString(requestMap);
        FicheSearchResult searchResult;
        if (qIdString != null) {
            if (qIdString.isEmpty()) {
                throw new ErrorMessageException(ErrorKeys.EMPTY_PARAMETER, Q_ID);
            }
            try {
                QId qId = QId.parse(qIdString);
                searchResult = scrutariSession.getSearch(qId);
            } catch (ParseException pe) {
                throw new ErrorMessageException(ErrorKeys.MALFORMED_PARAMETER_VALUE, toArray(Q_ID, qIdString));
            }
            if (searchResult == null) {
                throw new ErrorMessageException(ErrorKeys.UNKNOWN_PARAMETER_VALUE, toArray(Q_ID, qIdString));
            }
        } else {
            SearchOperation searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariSession.getScrutariDB().getAttributeKeyCoder());
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, warningHandler, true);
            searchResult = scrutariSession.newSearch(searchOperation, searchOptions);
            SctLogUtils.log(scrutariSession, searchResult, RequestMapUtils.getOrigin(requestMap));
        }
        FicheFieldsFilter ficheFieldsFilter = RequestMapUtils.getFicheFieldsFilter(requestMap);
        MotcleFieldsFilter motcleFieldsFilter = RequestMapUtils.getMotcleFieldsFilter(requestMap);
        IntitulesFilter intitulesFilter = RequestMapUtils.getIntitulesFilter(requestMap);
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        int start = RequestMapUtils.getStart(requestMap, warningHandler);
        return new FicheSearchResultJsonProducer(searchResult, scrutariSession.getScrutariDB(), scrutariSession.getGlobalSearchOptions(), ficheFieldsFilter, motcleFieldsFilter, intitulesFilter, limit, start);
    }

    private static AbstractJsonProducer getClasseDefJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new ClasseDefJsonProducer(scrutariSession.getGlobalSearchOptions(), langUi);
    }

    private static AbstractJsonProducer getBaseDataJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new BaseDataJsonProducer(scrutariSession.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getCorpusDataJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new CorpusDataJsonProducer(scrutariSession.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getThesaurusDataJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new ThesaurusDataJsonProducer(scrutariSession.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getLangJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        int[] langIntegerArray = RequestMapUtils.getLangIntegerArray(requestMap);
        return new LangJsonProducer(langUi, langIntegerArray);
    }

    private static AbstractJsonProducer getFicheDataJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        CodeList ficheCodeList;
        String ficheListParam = requestMap.getParameter(FICHE_LIST);
        if (ficheListParam != null) {
            ficheCodeList = RequestMapUtils.toCodeList(dataAccess, ficheListParam, warningHandler, "fiche:", ScrutariDataURI.FICHEURI_TYPE);
            if (ficheCodeList == null) {
                ficheCodeList = CodesUtils.EMPTY_CODELIST;
            }
        } else {
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, warningHandler, true);
            ficheCodeList = OptionsUtils.getFicheCodeList(dataAccess, searchOptions);
        }
        dataAccess.close();
        String randomString = requestMap.getParameter(RANDOM);
        if (randomString != null) {
            warningHandler.setCurrentParameter(RANDOM);
            try {
                int randomNumber = Integer.parseInt(randomString);
                if ((randomNumber == -1) || (randomNumber > 0)) {
                    ficheCodeList = CodesUtils.shuffle(ficheCodeList, randomNumber);
                } else {
                    warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, randomString);
                }
            } catch (NumberFormatException nfe) {
                warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, randomString);
            }
        }
        FicheFieldsFilter ficheFieldsFilter = RequestMapUtils.getFicheFieldsFilter(requestMap);
        return new FicheDataJsonProducer(scrutariSession.getScrutariDB(), ficheCodeList, ficheFieldsFilter, langUi);
    }

    private static AbstractJsonProducer getMotcleDataJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        CodeList motcleCodeList;
        String motcleListParam = requestMap.getParameter(MOTCLE_LIST);
        if (motcleListParam != null) {
            motcleCodeList = RequestMapUtils.toCodeList(dataAccess, motcleListParam, warningHandler, "motcle:", ScrutariDataURI.MOTCLEURI_TYPE);
            if (motcleCodeList == null) {
                motcleCodeList = CodesUtils.EMPTY_CODELIST;
            }
        } else {
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, warningHandler, false);
            motcleCodeList = OptionsUtils.getMotcleCodeList(dataAccess, searchOptions);
        }
        MotcleFieldsFilter motcleFieldsFilter = RequestMapUtils.getMotcleFieldsFilter(requestMap);
        return new MotcleDataJsonProducer(scrutariDB, motcleCodeList, motcleFieldsFilter);
    }

    private static int checkUniquelang(SearchOptionsBuilder searchOptionsBuilder) {
        int[] langIntegerArray = searchOptionsBuilder.getLangIntegerArray();
        if (langIntegerArray.length > 1) {
            int[] tmp = new int[1];
            tmp[0] = langIntegerArray[0];
            langIntegerArray = tmp;
            searchOptionsBuilder.setLangIntegerArray(tmp);
        }
        return langIntegerArray[0];
    }

    private static AbstractJsonProducer getFicheCountJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        CountFilter countFilter = RequestMapUtils.getCountFilter(dataAccess, requestMap, warningHandler);
        dataAccess.close();
        return new FicheCountJsonProducer(scrutariSession.getScrutariDB(), countFilter);
    }

    private static String[] toArray(String parameter, String value) {
        String[] array = new String[2];
        array[0] = parameter;
        array[1] = value;
        return array;
    }

}
