/* SctServer - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import fr.exemole.sctserver.api.ScrutariSession;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.data.FicheData;
import net.scrutari.datauri.FicheURI;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.DataUtils;
import net.scrutari.lexie.AlineaInteger;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;


/**
 * Classe produisant l'arbre XML des fiches.
 *
 * @author Vincent Calame
 */
public class FichesRequest {

    private final FicheSearchResult searchResult;
    private final ScrutariDB scrutariDB;
    private final int langUi;

    public FichesRequest(QId qId, ScrutariSession scrutariSession) throws XmlRequestException {
        if (qId == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_SEARCHID_PARAMETER);
        }
        this.searchResult = scrutariSession.getSearch(qId);
        if (searchResult == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.WRONG_SEARCHID_PARAMETER, qId.toString());
        }
        this.scrutariDB = scrutariSession.getScrutariDB();
        this.langUi = searchResult.getFicheSearchSource().getSearchOptions().getLangUi();
    }

    /**
     * Écrit l'arbre XML des fiches comme réponse à une requête de servlet.
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE fiches SYSTEM \"dtd/fiches.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<fiches id=\"fiches\" search-id=\"");
        QId qId = searchResult.getFicheSearchSource().getQId();
        if (qId != null) {
            pw.print(qId.toString());
        }
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        pw.print("\"");
        pw.print(">");
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            int infoCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            for (int j = 0; j < infoCount; j++) {
                printFicheResult(pw, ficheSearchResultGroup.getFicheSearchResultInfo(j), tabulations + 1, dataAccess);
            }
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</fiches>");
        dataAccess.close();
    }

    /*
     * Écrit les informations liées à une fiche d'un résultat. Méthode appelée
     * par printResponse.
     */
    private void printFicheResult(PrintWriter pw, FicheSearchResultInfo ficheResult, int tabulations, DataAccess dataAccess) {
        int code = ficheResult.getCode();
        String code_string = String.valueOf(code);
        FicheData ficheData = dataAccess.getFicheDataByCode(code);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<fiche id=\"");
        pw.print(code_string);
        pw.print("\" fiche-code=\"");
        pw.print(code_string);
        pw.print("\" corpus-code=\"");
        pw.print(String.valueOf(ficheData.getCodeCorpus()));
        pw.print("\" lang=\"");
        pw.print(XmlRequestUtils.getLangString(ficheData.getLangInteger()));
        int dateInteger = ficheData.getDateInteger();
        if (dateInteger != DateInteger.NULL) {
            pw.print("\" date=\"");
            pw.print(DateInteger.toString(dateInteger));
        }
        String href = DataUtils.getHref(ficheData, dataAccess, langUi);
        if (href != null) {
            pw.print("\" href=\"");
            XMLUtils.printEscape(pw, href);
        }
        pw.print("\"");
        pw.print(">");
        String titre = ficheData.getTitre();
        if (titre != null) {
            InAlineaOccurrenceList occurrenceList = ficheResult.getInAlineaOccurrenceListByAlinea(AlineaInteger.TITRE);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<titre id=\"");
            pw.print(code_string);
            pw.print("_titre\" role=\"");
            if (occurrenceList == null) {
                pw.print("0");
            } else {
                pw.print("1");
            }
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw, titre, occurrenceList);
            pw.print("</titre>");
        }
        String soustitre = ficheData.getSoustitre();
        if (soustitre != null) {
            InAlineaOccurrenceList occurrenceList = ficheResult.getInAlineaOccurrenceListByAlinea(AlineaInteger.SOUSTITRE);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<soustitre id=\"");
            pw.print(code_string);
            pw.print("_soustitre\" role=\"");
            if (occurrenceList == null) {
                pw.print("0");
            } else {
                pw.print("1");
            }
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw, soustitre, occurrenceList);
            pw.print("</soustitre>");
        }
        pw.print("<numero id=\"");
        pw.print(code_string);
        pw.print("_numero\">");
        FicheURI ficheURI = ficheData.getFicheURI();
        pw.print(ficheURI.getFicheId());
        pw.print("</numero>");
        for (int p = 1; p <= ficheData.getComplementMaxNumber(); p++) {
            String complement = ficheData.getComplementByNumber(p);
            if (complement != null) {
                InAlineaOccurrenceList occurrenceList = ficheResult.getInAlineaOccurrenceListByAlinea(p);
                XMLUtils.printTabs(pw, tabulations + 1);
                pw.print("<complement id=\"");
                pw.print(code_string);
                pw.print("_complement_");
                pw.print(String.valueOf(p));
                pw.print("\" complement-number=\"");
                pw.print(String.valueOf(p));
                pw.print("\" role=\"");
                if (occurrenceList == null) {
                    pw.print("0");
                } else {
                    pw.print("1");
                }
                pw.print("\">");
                XmlRequestUtils.printPositionList(pw, complement, occurrenceList);
                pw.print("</complement>");
            }
        }
        for (int i = 0; i < ficheResult.getMotcleCount(); i++) {
            int codeMotcle = ficheResult.getMotcleCode(i);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<mtcl motcle-code=\"");
            pw.print(String.valueOf(codeMotcle));
            pw.print("\"/>");
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</fiche>");
    }

}
