/* SctServer - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.tools.log.SctLogUtils;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.text.search.MultiSearchToken;
import net.mapeadores.util.text.search.SearchToken;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.db.ScrutariDB;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.tools.operands.search.SearchOperationBuilder;
import net.scrutari.searchengine.tools.options.OptionsUtils;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SearchRequest {

    private final static int DEFAULT_LANG = LangInteger.build("fr");
    private SearchSequence searchSequence;
    private FicheSearchResult searchResult;

    /**
     * Construit une nouvelle instance à partir du paramètre xml
     */
    public SearchRequest(String xml, ScrutariSession scrutariSession, String origin) throws XmlRequestException {
        if (xml == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_XML_PARAMETER);
        }
        LinkedIntHashMap baseCodeMap = new LinkedIntHashMap();
        LinkedIntHashMap corpusCodeMap = new LinkedIntHashMap();
        LinkedIntHashMap langIntMap = new LinkedIntHashMap();
        int langUi = DEFAULT_LANG;
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        String source = null;
        Document doc = XmlRequestUtils.getDocument(xml);
        Element racine = doc.getDocumentElement();
        NodeList nodeList = racine.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            if (node instanceof Element) {
                Element el = (Element) node;
                String tagname = el.getTagName();
                if (tagname.equals("source")) {
                    source = XMLUtils.getData(el);
                } else if (tagname.equals("base")) {
                    try {
                        int code = Integer.parseInt(el.getAttribute("base-code"));
                        baseCodeMap.put(code, Boolean.TRUE);
                    } catch (NumberFormatException nfe) {
                    }
                } else if (tagname.equals("corpus")) {
                    try {
                        int code = Integer.parseInt(el.getAttribute("corpus-code"));
                        CorpusData corpusInfo = dataAccess.getCorpusDataByCode(code);
                        if (corpusInfo != null) {
                            corpusCodeMap.put(code, Boolean.TRUE);
                        }
                    } catch (NumberFormatException nfe) {
                    }
                } else if (tagname.equals("lang")) {
                    String strg_lang = el.getAttribute("lang");
                    try {
                        int langInteger = LangInteger.parse(strg_lang);
                        langIntMap.put(langInteger, Boolean.TRUE);
                    } catch (LangIntegerException ile) {
                    }
                } else if (tagname.equals("lang-ui")) {
                    String strg_langui = el.getAttribute("lang");
                    try {
                        langUi = LangInteger.parse(strg_langui);
                    } catch (LangIntegerException ile) {
                    }
                }
            }
        }
        for (int i = 0; i < baseCodeMap.size(); i++) {
            BaseData baseInfo = dataAccess.getBaseDataByCode(baseCodeMap.getKeyAt(i));
            if (baseInfo != null) {
                int count = baseInfo.getCorpusCount();
                for (int j = 0; j < count; j++) {
                    corpusCodeMap.put(baseInfo.getCodeCorpus(j), Boolean.TRUE);
                }
            }
        }
        if (source == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_SOURCE_ELEMENT);
        }
        searchSequence = SearchSequence.createSearchSequence(source);
        SearchOperation searchOperation;
        if (searchSequence.isEmpty()) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.POOR_SOURCE_ELEMENT, searchSequence.getSource());
        } else {
            searchOperation = SearchOperationBuilder.fromSearchSequence(searchSequence, LogicalOperationConstants.INTERSECTION_OPERATOR);
        }
        CodeList corpusCodeList = null;
        if (corpusCodeMap.size() > 0) {
            corpusCodeList = CodesUtils.toCodeList(corpusCodeMap.getKeyArray());
        }
        int[] langIntegerArray = (langIntMap.size() > 0) ? langIntMap.getKeyArray() : null;
        SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder();
        searchOptionsBuilder.setLangUi(langUi);
        searchOptionsBuilder.setLangIntegerArray(langIntegerArray);
        if (corpusCodeList != null) {
            searchOptionsBuilder.reduceCorpusList(corpusCodeList);
            searchOptionsBuilder.reduceThesaurusList(OptionsUtils.getSelectionThesaurusCodeList(scrutariDB.getCountStats(), dataAccess, corpusCodeList));
        }
        searchOptionsBuilder.setDefaultSearchType(TextConstants.STARTSWITH);
        searchResult = scrutariSession.newSearch(searchOperation, searchOptionsBuilder.toSearchOptions());
        SctLogUtils.log(scrutariSession, searchResult, origin);
        dataAccess.close();
    }

    /**
     * Écrit la réponse à la requête : il s'agit de l'arbre xml « search ».
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        int allMotcleCount = searchResult.getMotcleCount();
        int completeMotcleCount = allMotcleCount;
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE search SYSTEM \"dtd/search.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<search id=\"search\" search-id=\"");
        QId qId = searchResult.getFicheSearchSource().getQId();
        if (qId != null) {
            pw.print(qId.toString());
        }
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(searchResult.getFicheMaximum()));
        pw.print("\" all-motcle-count=\"");
        pw.print(String.valueOf(allMotcleCount));
        pw.print("\" complete-motcle-count=\"");
        pw.print(String.valueOf(completeMotcleCount));
        pw.print("\" partial-motcle-count=\"");
        pw.print(String.valueOf(allMotcleCount - completeMotcleCount));
        pw.print("\"");
        pw.print(">");
        printSequence(pw, searchSequence, tabulations + 1);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</search>");

    }

    /**
     * Écrit le format xml décrivant la séquence.
     */
    public void printSequence(PrintWriter pw, SearchSequence searchSequence, int tabulations) {
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<sequence id=\"sequence\" token-count=\"");
        pw.print(String.valueOf(searchSequence.getTokenCount()));
        pw.print("\">");
        XMLUtils.printTabs(pw, tabulations + 1);
        pw.print("<source id=\"source\" clean=\"");
        XMLUtils.printEscape(pw, searchSequence.toString());
        pw.print("\">");
        XMLUtils.printEscape(pw, searchSequence.getSource());
        pw.print("</source>");
        for (int i = 0; i < searchSequence.getTokenCount(); i++) {
            SearchToken token = searchSequence.getToken(i);
            XMLUtils.printTabs(pw, tabulations + 1);
            if (token instanceof MultiSearchToken) {
                MultiSearchToken multiToken = (MultiSearchToken) token;
                pw.print("<token token-type=\"multi\" id=\"token_");
                pw.print(String.valueOf(i));
                pw.print("\" token-index=\"");
                pw.print(String.valueOf(i));
                pw.print("\">");
                int subtokenCount = multiToken.getSubtokenCount();
                for (int j = 0; j < subtokenCount; j++) {
                    SimpleSearchToken simpleToken = multiToken.getSubtoken(j);
                    XMLUtils.printTabs(pw, tabulations + 2);
                    pw.print("<subtoken search-type=\"");
                    pw.print(TextConstants.toString(simpleToken.getSearchType()));
                    pw.print("\">");
                    XMLUtils.printEscape(pw, simpleToken.getTokenString());
                    pw.print("</subtoken>");
                }
                XMLUtils.printTabs(pw, tabulations + 1);
                pw.print("</token>");
            } else {
                SimpleSearchToken simpleToken = (SimpleSearchToken) token;
                pw.print("<token token-type=\"simple\" id=\"token_");
                pw.print(String.valueOf(i));
                pw.print("\" token-index=\"");
                pw.print(String.valueOf(i));
                pw.print("\" search-type=\"");
                pw.print(TextConstants.toString(simpleToken.getSearchType()));
                pw.print("\">");
                XMLUtils.printEscape(pw, simpleToken.getTokenString());
                pw.print("</token>");
            }
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</sequence>");
    }

}
