/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.CollectErrorHandler;
import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariInfo;
import fr.exemole.sctserver.api.collect.ScrutariInfoContent;
import fr.exemole.sctserver.api.collect.StateErrorException;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.api.storage.SctStorageException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class CollectEngine {

    static {
        CollectUtils.initSslFactory();
    }

    private CollectEngine() {
    }

    public static CollectResult collectData(ScrutariInfo scrutariInfo, SctServerStorage sctServerStorage, CollectErrorHandler collectErrorHandler, boolean forceDataDownload) {
        try {
            return internalCollectData(scrutariInfo, sctServerStorage, collectErrorHandler, forceDataDownload);
        } catch (StateErrorException stateErrorException) {
            return CollectUtils.toCollectResult(stateErrorException.getErrorState(), 0);
        }
    }

    private static CollectResult internalCollectData(ScrutariInfo scrutariInfo, SctServerStorage sctServerStorage, CollectErrorHandler collectErrorHandler, boolean forceDataDownload) throws StateErrorException {
        ScrutariInfoContent scrutariInfoContent = getInfoContent(scrutariInfo.getUrl(), collectErrorHandler);
        int lastUpdate = scrutariInfoContent.getLastUpdateDateInteger();
        if (!forceDataDownload) {
            if (lastUpdate <= scrutariInfo.getLastUpdate()) {
                return CollectUtils.toCollectResult(ScrutariInfo.OK_STATE, scrutariInfo.getLastUpdate());
            }
        }
        OutputStream outputStream = sctServerStorage.getScrutariDataOutputStream(scrutariInfo);
        try {
            int dataCount = scrutariInfoContent.getDataUrlCount();
            for (int i = 0; i < dataCount; i++) {
                URL dataUrl = scrutariInfoContent.getDataUrl(i);
                URLConnection con = null;
                try {
                    con = dataUrl.openConnection();
                    if (!CollectUtils.testConnection(con, collectErrorHandler)) {
                        throw new StateErrorException(ScrutariInfo.DATA_BROKENURL_STATE);
                    }
                } catch (IOException ioe) {
                    collectErrorHandler.warningDownload(dataUrl, ioe.getMessage());
                    outputStream.close();
                    throw new StateErrorException(ScrutariInfo.DATA_NOCONNECTION_STATE);
                }
                try {
                    InputStream in = con.getInputStream();
                    byte[] buffer = new byte[1024];
                    int size;
                    while ((size = in.read(buffer, 0, 1024)) > 0) {
                        outputStream.write(buffer, 0, size);
                    }
                    in.close();
                } catch (IOException ioe) {
                    collectErrorHandler.warningDownload(dataUrl, ioe.getMessage());
                    outputStream.close();
                    throw new StateErrorException(ScrutariInfo.DATA_DOWNLOADERROR_STATE);
                }
            }
            outputStream.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return CollectUtils.toCollectResult(ScrutariInfo.OK_STATE, lastUpdate);
    }

    private static ScrutariInfoContent getInfoContent(URL url, CollectErrorHandler collectErrorHandler) throws StateErrorException {
        URLConnection con = null;
        try {
            con = (HttpURLConnection) url.openConnection();
            if (!CollectUtils.testConnection(con, collectErrorHandler)) {
                throw new StateErrorException(ScrutariInfo.INFO_BROKENURL_STATE);
            }
        } catch (IOException ioe) {
            collectErrorHandler.warningDownload(url, ioe.getMessage());
            throw new StateErrorException(ScrutariInfo.INFO_NOCONNECTION_STATE);
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new NestedLibraryException(e);
        }
        Document doc = null;
        try {
            doc = db.parse(con.getInputStream());
        } catch (IOException ioe) {
            collectErrorHandler.warningDownload(url, ioe.getMessage());
            throw new StateErrorException(ScrutariInfo.INFO_DOWNLOADERROR_STATE);
        } catch (SAXException saxe) {
            collectErrorHandler.warningDownload(url, saxe.getMessage());
            throw new StateErrorException(ScrutariInfo.INFO_NOTXML_STATE);
        }
        String infoUrlString = url.toString();
        int idx = infoUrlString.lastIndexOf('/');
        String urlRoot = infoUrlString.substring(0, idx + 1);
        return readScrutariInfoDOM(doc.getDocumentElement(), urlRoot);
    }

    private static ScrutariInfoContent readScrutariInfoDOM(Element element_xml, String urlRoot) throws StateErrorException {
        String lastDateString = null;
        List<String> dataUrlStringList = new ArrayList<String>();
        NodeList nodeList = element_xml.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            if (node.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) node;
                String tagname = el.getTagName();
                if (tagname.equals("date")) {
                    lastDateString = XMLUtils.getData(el);
                } else if ((tagname.equals("path")) || (tagname.equals("scrutaridata-url"))) {
                    String pathString = XMLUtils.getData(el);
                    if (pathString.length() > 0) {
                        dataUrlStringList.add(pathString);
                    }
                }
            }
        }
        if ((dataUrlStringList.isEmpty()) || (lastDateString == null)) {
            throw new StateErrorException(ScrutariInfo.INFO_WRONGXML_STATE);
        }
        int lastUpdate;
        try {
            lastUpdate = DateInteger.parse(lastDateString);
        } catch (DateIntegerException ide) {
            throw new StateErrorException(ScrutariInfo.INFO_WRONGXML_STATE);
        }
        List<URL> urlList = new ArrayList<URL>();
        for (String urlString : dataUrlStringList) {
            try {
                URL dataUrl = buildDataUrl(urlString, urlRoot);
                urlList.add(dataUrl);
            } catch (MalformedURLException mue) {
                throw new StateErrorException(ScrutariInfo.INFO_WRONGXML_STATE);
            }
        }
        return new InternalScrutariInfoContent(lastUpdate, urlList);
    }

    private static URL buildDataUrl(String pathString, String urlRoot) throws MalformedURLException {
        try {
            URL url = new URL(pathString);
            return url;
        } catch (MalformedURLException mue) {
            URL url = new URL(urlRoot + pathString);
            return url;
        }
    }


    private static class InternalScrutariInfoContent implements ScrutariInfoContent {

        private int lastUpdate;
        private List<URL> urlList;

        private InternalScrutariInfoContent(int lastUpdate, List<URL> urlList) {
            this.lastUpdate = lastUpdate;
            this.urlList = urlList;
        }

        public int getLastUpdateDateInteger() {
            return lastUpdate;
        }

        public int getDataUrlCount() {
            return urlList.size();
        }

        public URL getDataUrl(int i) {
            return urlList.get(i);
        }

    }

}
