/* SctServer - Copyright (c) 2006-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.SctServer;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.searchengine.api.options.CorpusClasse;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusClassesDOMReader {

    private Builder builder;
    private ProcessMessageHandler errorHandler;

    public CorpusClassesDOMReader(Builder builder, ProcessMessageHandler errorHandler) {
        this.builder = builder;
        this.errorHandler = errorHandler;
    }

    public void readCorpusClasses(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if ((tagName.equals("classes-defs")) || (tagName.equals("classe-def-list"))) {
                    addDefs(element);
                }
            }
        }
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("corpus-list")) {
                    addCorpusList(element);
                } else if (tagName.equals("ponderation")) {
                    setPonderation(element);
                } else if ((!tagName.equals("classes-defs")) && (!tagName.equals("classe-def-list"))) {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }

    private void setPonderation(Element element_xml) {
        int occurrence = getInt(element_xml, "occurrence");
        int date = getInt(element_xml, "date");
        int origin = getInt(element_xml, "origin");
        builder.setGlobalPertinencePonderation(PertinencePonderationBuilder.toPertinencePonderation(occurrence, date, origin));
    }

    private int getInt(Element element_xml, String attribute) {
        String vale = element_xml.getAttribute(attribute);
        if (vale.length() == 0) {
            return 0;
        }
        try {
            int itg = Integer.parseInt(vale);
            return itg;
        } catch (NumberFormatException nfe) {
            return 0;
        }
    }

    private void addDefs(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("classe-def")) {
                    addDef(element);
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }

    private void addDef(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            ErrorUtils.emptyAttributeError(errorHandler, element_xml.getTagName(), "name");
            return;
        }
        if (name.equals("_default")) {
            name = null;
        }
        CorpusClasse corpusClasse = builder.addClasse(name);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib")) {
                    try {
                        Libelle libelle = LibelleUtils.readLib(element);
                        if (libelle != null) {
                            builder.putLibelle(corpusClasse, libelle);
                        }
                    } catch (LangIntegerException lie) {
                    }
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }

    }

    private void addCorpusList(Element element_xml) {
        String name = element_xml.getAttribute("classe-name");
        if (name.length() == 0) {
            ErrorUtils.emptyAttributeError(errorHandler, element_xml.getTagName(), "classe-name");
            return;
        }
        CorpusClasse corpusClasse = builder.getCorpusClasse(name);
        if (corpusClasse == null) {
            errorHandler.addMessage(SctServer.SEVERE_CATEGORY, "unknownCorpusClasse", name);
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    String uriString = XMLUtils.getData(element);
                    if (uriString.length() == 0) {
                        ErrorUtils.emptyElementError(errorHandler, "uri");
                        continue;
                    }
                    try {
                        ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
                        if (scrutariDataURI instanceof CorpusURI) {
                            builder.addURI(corpusClasse, (CorpusURI) scrutariDataURI);
                        }
                    } catch (URIParseException uripe) {
                        ErrorUtils.uriSyntaxError(errorHandler, uriString);
                    }
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }


    public static interface Builder {

        public CorpusClasse addClasse(String name);

        public CorpusClasse getCorpusClasse(String name);

        public void putLibelle(CorpusClasse corpusClasse, Libelle libelle);

        public boolean addURI(CorpusClasse corpusClasse, CorpusURI corpusURI);

        public void setGlobalPertinencePonderation(PertinencePonderation pertinencePonderation);

    }

}
