/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.feed;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import fr.exemole.sctserver.request.ErrorKeys;
import fr.exemole.sctserver.request.Parameters;
import static fr.exemole.sctserver.request.Parameters.Q_ID;
import fr.exemole.sctserver.request.RequestMapUtils;
import fr.exemole.sctserver.request.WarningHandler;
import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.db.ScrutariDBName;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.FicheSearchEngine;
import net.scrutari.searchengine.QId;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class FicheFeedListEngine {

    private final static long TWO_DAYS = 2 * 24 * 60 * 60 * 1000;

    private FicheFeedListEngine() {
    }

    public static FicheFeedList build(ScrutariSession scrutariSession, SctFeedInfo sctFeedInfo, DataAccess dataAccess, SctFeedOptions feedOptions) throws ErrorMessageException {
        int langUi = feedOptions.getLangUi();
        ScrutariDBName scrutariDBName = sctFeedInfo.getLastScrutariDBName(0);
        RequestMap requestMap = feedOptions.getRequestMap();
        FicheSearchResult ficheSearchResult = null;
        SearchOperation searchOperation = null;
        SearchOptions searchOptions = null;
        if (requestMap != null) {
            String qIdString = requestMap.getParameter(Parameters.Q_ID);
            if (qIdString != null) {
                try {
                    QId qId = QId.parse(qIdString);
                    ficheSearchResult = scrutariSession.getSearch(qId);
                    if (ficheSearchResult == null) {
                        throw new ErrorMessageException(ErrorKeys.UNKNOWN_PARAMETER_VALUE, LogUtils.toValue(Q_ID, qIdString));
                    }
                    boolean forceAll = false;
                    String allString = requestMap.getParameter(Parameters.ALL);
                    if ((allString != null) && (allString.equals("1"))) {
                        forceAll = true;
                    }
                    Date queryDate = qId.getScrutariDBName().getDate();
                    if ((forceAll) || (isRecent(queryDate))) {
                        return getRecentQList(scrutariSession, dataAccess, ficheSearchResult, feedOptions, scrutariDBName);
                    }
                } catch (ParseException pe) {
                    throw new ErrorMessageException(ErrorKeys.MALFORMED_PARAMETER_VALUE, LogUtils.toValue(Q_ID, qIdString));
                }
            } else if (requestMap.getParameter(Parameters.Q) != null) {
                searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariSession.getScrutariDB().getAttributeKeyCoder());
                searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, new WarningHandler(), true);
            }
        }
        URITree uriTree = scrutariSession.getSctServer().getSctServerStorage().getURITree(scrutariDBName);
        if (uriTree == null) {
            return null;
        }
        FicheFeedList ficheResult = init(uriTree, dataAccess, langUi, scrutariDBName, scrutariSession.getGlobalSearchOptions());
        if (searchOperation != null) {
            SearchOptionsBuilder searchOptionsBuilder = SearchOptionsBuilder.derive(searchOptions);
            searchOptionsBuilder.setLangUi(langUi);
            searchOptionsBuilder.reduceFicheList(ficheResult);
            ficheSearchResult = FicheSearchEngine.search(scrutariSession.getScrutariDB(), searchOperation, searchOptions, scrutariSession.getGlobalSearchOptions(), null);
        }
        if (ficheSearchResult != null) {
            ficheResult.update(ficheSearchResult);
        }
        return ficheResult;
    }

    private static FicheFeedList init(URITree uriTree, DataAccess dataAccess, int langUi, ScrutariDBName scrutariDBName, GlobalSearchOptions globalSearchOptions) {
        FicheFeedList ficheResult = new FicheFeedList(langUi, scrutariDBName);
        int baseCount = uriTree.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseNode baseNode = uriTree.getBaseNode(i);
            BaseURI baseURI = baseNode.getBaseURI();
            BaseData baseData = (BaseData) dataAccess.getScrutariData(baseURI);
            if (baseData == null) {
                continue;
            }
            BaseWrapper baseWrapper = BaseWrapper.wrapBase(baseData, langUi);
            int corpusCount = baseNode.getCorpusCount();
            for (int j = 0; j < corpusCount; j++) {
                CorpusNode corpusNode = baseNode.getCorpusNode(j);
                CorpusURI corpusURI = corpusNode.getCorpusURI();
                CorpusData corpusData = (CorpusData) dataAccess.getScrutariData(corpusURI);
                if (corpusData == null) {
                    continue;
                }
                CorpusWrapper corpusWrapper = CorpusWrapper.wrapCorpus(corpusData, langUi, globalSearchOptions);
                int ficheCount = corpusNode.getFicheCount();
                for (int k = 0; k < ficheCount; k++) {
                    FicheNode ficheNode = corpusNode.getFicheNode(k);
                    FicheURI ficheURI = ficheNode.getFicheURI();
                    FicheData ficheData = (FicheData) dataAccess.getScrutariData(ficheURI);
                    if (ficheData == null) {
                        continue;
                    }
                    ficheResult.addFicheData(ficheData, corpusWrapper, baseWrapper);
                }
            }
        }
        return ficheResult;
    }

    private static FicheFeedList getRecentQList(ScrutariSession scrutariSession, DataAccess dataAccess, FicheSearchResult ficheSearchResult, SctFeedOptions feedOptions, ScrutariDBName lastDate) {
        int langUi = feedOptions.getLangUi();
        WrapperHolder wrapperHolder = new WrapperHolder(dataAccess, langUi, scrutariSession.getGlobalSearchOptions());
        FicheFeedList ficheFeedList = new FicheFeedList(langUi, lastDate);
        int groupCount = ficheSearchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup group = ficheSearchResult.getFicheSearchResultGroup(i);
            int ficheCount = group.getFicheSearchResultInfoCount();
            for (int j = 0; j < ficheCount; j++) {
                FicheSearchResultInfo info = group.getFicheSearchResultInfo(j);
                int codefiche = info.getCode();
                FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                ficheFeedList.addFicheData(ficheData, wrapperHolder.getCorpusWrapper(ficheData.getCodeCorpus()), wrapperHolder.getBaseWrapper(ficheData.getCodeBase()));
            }
        }
        return ficheFeedList;
    }

    private static boolean isRecent(Date date) {
        long time = date.getTime();
        long current = System.currentTimeMillis();
        return (current - time < TWO_DAYS);
    }


    private static class WrapperHolder {

        private final Map<Integer, BaseWrapper> baseWrapperMap = new HashMap<Integer, BaseWrapper>();
        private final Map<Integer, CorpusWrapper> corpusWrapperMap = new HashMap<Integer, CorpusWrapper>();
        private final DataAccess dataAccess;
        private final int langUi;
        private final GlobalSearchOptions globalSearchOptions;

        private WrapperHolder(DataAccess dataAccess, int langUi, GlobalSearchOptions globalSearchOptions) {
            this.dataAccess = dataAccess;
            this.langUi = langUi;
            this.globalSearchOptions = globalSearchOptions;
        }

        private BaseWrapper getBaseWrapper(int codebase) {
            BaseWrapper baseWrapper = baseWrapperMap.get(codebase);
            if (baseWrapper == null) {
                BaseData baseData = dataAccess.getBaseDataByCode(codebase);
                baseWrapper = BaseWrapper.wrapBase(baseData, langUi);
                baseWrapperMap.put(codebase, baseWrapper);
            }
            return baseWrapper;
        }

        private CorpusWrapper getCorpusWrapper(int codecorpus) {
            CorpusWrapper corpusWrapper = corpusWrapperMap.get(codecorpus);
            if (corpusWrapper == null) {
                CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
                corpusWrapper = CorpusWrapper.wrapCorpus(corpusData, langUi, globalSearchOptions);
                corpusWrapperMap.put(codecorpus, corpusWrapper);
            }
            return corpusWrapper;
        }

    }

}
