/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.xml.feed;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import fr.exemole.sctserver.tools.feed.BaseWrapper;
import fr.exemole.sctserver.tools.feed.CorpusWrapper;
import fr.exemole.sctserver.tools.feed.FicheFeedList;
import fr.exemole.sctserver.tools.feed.FicheFeedListEngine;
import fr.exemole.sctserver.tools.feed.FicheWrapper;
import fr.exemole.sctserver.tools.feed.ThesaurusWrapper;
import java.io.IOException;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.ns.NameSpace;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.IndexationCodeList;


/**
 *
 * @author Vincent Calame
 */
class FichesEntriesXMLPart extends AbstractEntriesXMLPart {

    private final AttributeDefList primaryAttributeDefList;
    private final AttributeDefList secondaryAttributeDefList;

    FichesEntriesXMLPart(AppendableXMLWriter xmlWriter, ScrutariSession scrutariSession) {
        super(xmlWriter, scrutariSession);
        AttributeKeyCoder attributeKeyCoder = scrutariSession.getScrutariDB().getAttributeKeyCoder();
        this.primaryAttributeDefList = attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.PRIMARY_GROUP);
        this.secondaryAttributeDefList = attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.SECONDARY_GROUP);
    }

    @Override
    public void appendEntries(String feedId, SctFeedInfo sctFeedInfo, DataAccess dataAccess, SctFeedOptions feedOptions) throws IOException {
        int langUi = feedOptions.getLangUi();
        int thesaurusCount = dataAccess.getThesaurusCount();
        ThesaurusWrapper[] thesaurusWrapperArray = new ThesaurusWrapper[thesaurusCount];
        for (int i = 0; i < thesaurusCount; i++) {
            thesaurusWrapperArray[i] = ThesaurusWrapper.wrapThesaurus(dataAccess.getThesaurusDataByIndex(i), langUi);
        }
        try {
            FicheFeedList ficheFeedList = FicheFeedListEngine.build(scrutariSession, sctFeedInfo, dataAccess, feedOptions);
            if (ficheFeedList != null) {
                appendFicheFeedList(ficheFeedList, dataAccess, thesaurusWrapperArray);
            }
        } catch (ErrorMessageException errorMessageException) {
            CommandMessage errorMessage = errorMessageException.getErrorMessage();
            String label = "";
            if (errorMessage.getMessageValue() != null) {
                label = errorMessage.getMessageValue().toString();
            }
            appendDebugCategory("PARAMETER_ERROR/" + errorMessage.getMessageKey(), label);
        }
    }

    private void appendDebugCategory(String term, String label) throws IOException {
        startOpenTag("category");
        addAttribute("term", term);
        addAttribute("label", label);
        closeEmptyTag();
    }

    private void appendFicheFeedList(FicheFeedList ficheFeedList, DataAccess dataAccess, ThesaurusWrapper[] thesaurusWrapperArray) throws IOException {
        int langUi = ficheFeedList.getLangUi();
        int ficheCount = ficheFeedList.getFicheCount();
        String updatedString = ficheFeedList.getScrutariDBName().toGmtIsoFormat();
        for (int i = 0; i < ficheCount; i++) {
            FicheWrapper ficheWrapper = ficheFeedList.getFicheWrapper(i);
            FicheData ficheData = ficheWrapper.getFicheData();
            CorpusWrapper corpusWrapper = ficheWrapper.getCorpusWrapper();
            BaseWrapper baseWrapper = ficheWrapper.getBaseWrapper();
            openTag("entry");
            String title = ficheData.getTitre();
            if (title == null) {
                title = "?";
            }
            addSimpleElement("title", title);
            addSimpleElement("updated", updatedString);
            addSimpleElement("id", ficheData.getFicheURI().toString());
            openTag("author");
            addSimpleElement("name", baseWrapper.getShortLibelleString());
            closeTag("author");
            startOpenTag("link");
            addAttribute("href", ficheData.getHref(baseWrapper.getBaseCheck().getCheckedLangUi()));
            addAttribute("rel", "alternate");
            closeEmptyTag();
            String categoryTerm = corpusWrapper.getCategoryTerm();
            if (categoryTerm != null) {
                startOpenTag("category");
                addAttribute("term", categoryTerm);
                addAttribute("label", corpusWrapper.getCategoryLabel());
                closeEmptyTag();
            }
            startOpenTag("content");
            addAttribute("type", "xhtml");
            endOpenTag();
            startOpenTag("div");
            XMLUtils.appendNameSpaceAttribute(this, NameSpace.XHTML_NAMESPACE);
            endOpenTag();
            appendContent(ficheWrapper, dataAccess, langUi, thesaurusWrapperArray);
            closeTag("div");
            closeTag("content");
            closeTag("entry");
        }
    }

    private void appendContent(FicheWrapper ficheWrapper, DataAccess dataAccess, int langUi, ThesaurusWrapper[] thesaurusWrapperArray) throws IOException {
        FicheData ficheData = ficheWrapper.getFicheData();
        CorpusWrapper corpusWrapper = ficheWrapper.getCorpusWrapper();
        String soustitre = ficheData.getSoustitre();
        if (soustitre != null) {
            openTag("p", true, false);
            openTag("em", false, false);
            addText(soustitre);
            closeTag("em", false, false);
            closeTag("p", false, true);
        }
        AttributeHolder attributeHolder = ficheData.getAttributHolder();
        int primaryCount = primaryAttributeDefList.getAttributeDefCount();
        for (int i = 0; i < primaryCount; i++) {
            AttributeDef attributeDef = primaryAttributeDefList.getAttributeDef(i);
            Attribute attribute = attributeHolder.getAttributeByKey(attributeDef.getAttributeKey());
            if (attribute != null) {
                appendAttribute(attribute, attributeDef, langUi);
            }
        }
        int maxNumber = ficheData.getComplementMaxNumber();
        if (maxNumber > 0) {
            for (int num = 1; num <= maxNumber; num++) {
                String comp = ficheData.getComplementByNumber(num);
                if (comp == null) {
                    continue;
                }
                openTag("p", true, false);
                openTag("strong", false, false);
                addText(corpusWrapper.getComplementLibelle(num));
                addText(" : ");
                closeTag("strong", false, false);
                addText(comp);
                closeTag("p", false, true);
            }
        }
        int secondaryCount = secondaryAttributeDefList.getAttributeDefCount();
        for (int i = 0; i < secondaryCount; i++) {
            AttributeDef attributeDef = secondaryAttributeDefList.getAttributeDef(i);
            Attribute attribute = attributeHolder.getAttributeByKey(attributeDef.getAttributeKey());
            if (attribute != null) {
                appendAttribute(attribute, attributeDef, langUi);
            }
        }
        int codefiche = ficheData.getCodeFiche();
        int thesaurusCount = thesaurusWrapperArray.length;
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusWrapper thesaurusWrapper = thesaurusWrapperArray[i];
            IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codefiche, thesaurusWrapper.getCodeThesaurus());
            int indexationCount = indexationCodeList.getCodeCount();
            if (indexationCount > 0) {
                openTag("p");
                openTag("strong", false, false);
                addText(thesaurusWrapper.getLibelleString());
                addText(" : ");
                closeTag("strong", false, false);
                boolean next = false;
                for (int j = 0; j < indexationCount; j++) {
                    int codeMotcle = indexationCodeList.getCode(j);
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(codeMotcle);
                    if (motcleData != null) {
                        if (next) {
                            addText(", ");
                        } else {
                            next = true;
                        }
                        Libelle libelle = motcleData.getLibelleByLang(langUi);
                        if (libelle == null) {
                            if (motcleData.getLibelleCount() > 0) {
                                libelle = motcleData.getLibelleByIndex(0);
                            }
                        }
                        if (libelle != null) {
                            addText(libelle.getLibelleString());
                        }
                    }
                }
                closeTag("p", false, true);
            }
        }
        openTag("p");
        openTag("strong", false, false);
        openTag("em", false, false);
        addText(ficheWrapper.getBaseWrapper().getLongLibelleString());
        addText(" – ");
        addText(corpusWrapper.getCorpusLibelleString());
        closeTag("em", false, false);
        closeTag("strong", false, false);
        closeTag("p", false, true);
    }

    private void appendAttribute(Attribute attribute, AttributeDef attributeDef, int langUi) throws IOException {
        LibelleHolder libelleHolder = attributeDef.getLibelleHolder();
        Libelle libelle = libelleHolder.getLibelleByLang(langUi);
        if (libelle == null) {
            if (libelleHolder.getLibelleCount() > 0) {
                libelle = libelleHolder.getLibelleByIndex(0);
            }
        }
        String libelleString;
        if (libelle != null) {
            libelleString = libelle.getLibelleString();
        } else {
            libelleString = attributeDef.getAttributeKey().toString();
        }

        String formatType = attributeDef.getFormatType();
        if (formatType.equals(ScrutariAttributes.BLOCK_TYPE)) {
            openTag("p");
            openTag("strong", false, false);
            addText(libelleString);
            addText(" : ");
            closeTag("strong", false, false);
            closeTag("p", false, true);
            int valueCount = attribute.getValueCount();
            for (int i = 0; i < valueCount; i++) {
                addSimpleElement("p", attribute.getValue(i));
            }
        } else if (formatType.equals(ScrutariAttributes.LIST_TYPE)) {
            openTag("p");
            openTag("strong", false, false);
            addText(libelleString);
            addText(" : ");
            closeTag("strong", false, false);
            closeTag("p", false, true);
            openTag("ul");
            int valueCount = attribute.getValueCount();
            for (int i = 0; i < valueCount; i++) {
                addSimpleElement("li", attribute.getValue(i));
            }
            closeTag("ul");
        } else {
            openTag("p");
            openTag("strong", false, false);
            addText(libelleString);
            addText(" : ");
            closeTag("strong", false, false);
            int valueCount = attribute.getValueCount();
            for (int i = 0; i < valueCount; i++) {
                if (i > 0) {
                    addText(", ");
                }
                addText(attribute.getValue(i));
            }
            closeTag("p", false, true);
        }
    }

}
