/* SctServer_Directory - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory;

import fr.exemole.sctserver.api.collect.ScrutariInfo;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.feed.SctFeedCache;
import fr.exemole.sctserver.api.storage.FicheSearchCache;
import fr.exemole.sctserver.api.storage.LogStorage;
import fr.exemole.sctserver.api.storage.QIdStorage;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.api.storage.SctStorageException;
import fr.exemole.sctserver.directory.cache.DirectoryDataCacheWriter;
import fr.exemole.sctserver.directory.cache.DirectoryFicheSearchCache;
import fr.exemole.sctserver.directory.cache.DirectoryLexieCacheWriter;
import fr.exemole.sctserver.directory.cache.DirectorySctFeedCache;
import fr.exemole.sctserver.directory.persistent.DirectoryLogStorage;
import fr.exemole.sctserver.directory.persistent.DirectoryQIdStorage;
import fr.exemole.sctserver.tools.conf.factory.ConfDirectoryFactory;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.text.ParseException;
import java.util.Set;
import java.util.TreeSet;
import net.mapeadores.util.io.IOUtils;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.datauri.tree.io.TsvReader;
import net.scrutari.datauri.tree.io.TsvWriter;
import net.scrutari.db.ScrutariDBName;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.db.api.caches.LexieCacheWriter;


/**
 *
 * @author Vincent Calame
 */
public class DirectorySctServerStorage implements SctServerStorage {

    private final File varDir;
    private final File cacheDirectory;
    private final File dataDirectory;
    private final File uriDirectory;
    private final ConfDirectory confDirectory;
    private final DirectoryQIdStorage qIdStorage;
    private final DirectoryLogStorage logStorage;

    public DirectorySctServerStorage(File confDir, File varDir) {
        this.varDir = varDir;
        this.confDirectory = ConfDirectoryFactory.fromDirectory(confDir);
        this.cacheDirectory = getDirectory("cache");
        try {
            IOUtils.cleanDirectory(cacheDirectory);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.dataDirectory = getDirectory("data");
        this.uriDirectory = getDirectory("uri");
        this.qIdStorage = new DirectoryQIdStorage(getDirectory("qid"));
        this.logStorage = new DirectoryLogStorage(getDirectory("logs"));
        convert();
    }

    @Override
    public DataCacheWriter getDataCacheWriter(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/scrutaridb");
        dbCacheDirectory.mkdirs();
        return new DirectoryDataCacheWriter(dbCacheDirectory);
    }

    @Override
    public LexieCacheWriter getLexieCacheWriter(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/scrutaridb");
        dbCacheDirectory.mkdirs();
        return new DirectoryLexieCacheWriter(dbCacheDirectory);
    }

    @Override
    public SctFeedCache getSctFeedCache(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/feed");
        dbCacheDirectory.mkdirs();
        return new DirectorySctFeedCache(dbCacheDirectory);
    }

    @Override
    public FicheSearchCache getFicheSearchCache(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/search");
        dbCacheDirectory.mkdirs();
        return new DirectoryFicheSearchCache(dbCacheDirectory);
    }

    @Override
    public QIdStorage getQIdStorage() {
        return qIdStorage;
    }

    @Override
    public LogStorage getLogStorage() {
        return logStorage;
    }

    @Override
    public void cleanCache(Set<ScrutariDBName> remainingSet) {
        File[] dirs = cacheDirectory.listFiles();
        int length = dirs.length;
        for (int i = 0; i < length; i++) {
            File dir = dirs[i];
            if (!dir.isDirectory()) {
                continue;
            }
            String name = dir.getName();
            boolean remain = false;
            try {
                ScrutariDBName dbName = ScrutariDBName.parse(name);
                remain = remainingSet.contains(dbName);
            } catch (ParseException pe) {
            }
            if (!remain) {
                try {
                    IOUtils.cleanDirectory(dir);
                    dir.delete();
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }
        }
    }

    @Override
    public ConfDirectory getConfDirectory() {
        return confDirectory;
    }

    private File getDirectory(String name) {
        File directory = new File(varDir, name);
        testDirectory(directory);
        return directory;
    }

    private void testDirectory(File directory) {
        if (!directory.exists()) {
            directory.mkdirs();
        } else if (!directory.isDirectory()) {
            throw new SctStorageException("Not a directory: " + directory.getPath());
        }
    }

    @Override
    public InputStream getScrutariData(ScrutariInfo scrutariInfo) {
        File file = getDataFile(scrutariInfo);
        if (!file.exists()) {
            return null;
        }
        if (file.isDirectory()) {
            return null;
        }
        try {
            return new FileInputStream(file);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public OutputStream getScrutariDataOutputStream(ScrutariInfo scrutariInfo) {
        File file = getDataFile(scrutariInfo);
        try {
            return new FileOutputStream(file);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }

    }

    private File getDataFile(ScrutariInfo sourceInfo) {
        return new File(dataDirectory, sourceInfo.getKey() + ".xml");
    }

    @Override
    public String getContent(String contentName) {
        File contentFile = getContentFile(contentName);
        if (contentFile == null) {
            return "";
        }
        if (!contentFile.exists()) {
            return "";
        }
        try {
            FileInputStream inputStream = new FileInputStream(contentFile);
            String content = IOUtils.toString(inputStream, "UTF-8");
            inputStream.close();
            return content;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public void saveContent(String contentName, String content) {
        File contentFile = getContentFile(contentName);
        if (contentFile == null) {
            throw new IllegalArgumentException("Unknown content : " + contentName);
        }
        try {
            OutputStreamWriter writer = new OutputStreamWriter(new FileOutputStream(contentFile), "UTF-8");
            IOUtils.write(content, writer);
            writer.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public void saveURITree(URITree uriTree, ScrutariDBName scrutariDBName) {
        File file = getUriTreeFile(scrutariDBName);
        try {
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
            TsvWriter.write(writer, uriTree);
            writer.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public URITree getURITree(ScrutariDBName scrutariDBName) {
        File file = getUriTreeFile(scrutariDBName);
        if (!file.exists()) {
            return null;
        }
        try {
            InputStreamReader reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
            URITree uriTree = TsvReader.readURITree(reader);
            reader.close();
            return uriTree;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public ScrutariDBName[] getAddScrutariDBNameArray() {
        Set<ScrutariDBName> set = new TreeSet<ScrutariDBName>();
        File[] dirs = uriDirectory.listFiles();
        int length = dirs.length;
        for (int i = 0; i < length; i++) {
            File dir = dirs[i];
            if (dir.isDirectory()) {
                String year = dir.getName();
                File[] files = dir.listFiles();
                int fileLength = files.length;
                for (int j = 0; j < fileLength; j++) {
                    File f = files[j];
                    String name = f.getName();
                    if (name.endsWith(".txt")) {
                        name = name.substring(0, name.length() - 4);
                        try {
                            ScrutariDBName scrutariDBName = ScrutariDBName.parse(name);
                            if (year.equals(scrutariDBName.getYear())) {
                                set.add(scrutariDBName);
                            }
                        } catch (ParseException pe) {
                        }
                    }
                }

            }
        }
        int size = set.size();
        int p = size - 1;
        ScrutariDBName[] result = new ScrutariDBName[size];
        for (ScrutariDBName scrutariDBName : set) {
            result[p] = scrutariDBName;
            p--;
        }
        return result;
    }

    private File getUriTreeFile(ScrutariDBName scrutariDBName) {
        if (scrutariDBName == null) {
            return new File(uriDirectory, "tree.txt");
        }
        File dir = new File(uriDirectory, scrutariDBName.getYear());
        dir.mkdirs();
        return new File(dir, scrutariDBName.getName() + ".txt");
    }

    private File getContentFile(String contentName) {
        if (contentName.equals(SctServerStorage.INFOS_NAME)) {
            return new File(dataDirectory, "infos.txt");
        }
        return null;
    }

    private void convert() {
        File dir = new File(uriDirectory, "add");
        if (!dir.exists()) {
            return;
        }
        File[] files = dir.listFiles();
        int length = files.length;
        for (int i = 0; i < length; i++) {
            File f = files[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            if (name.endsWith(".txt")) {
                try {
                    ScrutariDBName scrutariDBName = ScrutariDBName.parse(name.substring(0, name.length() - 4));
                    File destDir = new File(uriDirectory, scrutariDBName.getYear());
                    destDir.mkdirs();
                    f.renameTo(new File(destDir, name));
                } catch (ParseException pe) {
                }
            }
        }
        try {
            IOUtils.forceDelete(dir);
        } catch (IOException ioe) {
        }
    }

}
