/* SctServer_Directory - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory.persistent;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.log.QLog;
import fr.exemole.sctserver.api.storage.LogStorage;
import fr.exemole.sctserver.tools.log.SctLogUtils;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryLogStorage implements LogStorage {

    private final File logsDirectory;

    public DirectoryLogStorage(File logsDirectory) {
        this.logsDirectory = logsDirectory;
    }

    @Override
    public synchronized void log(ScrutariSession scrutariSession, QLog qLog) {
        int currentDateInteger = DateInteger.current();
        File logFile = getLogFile(currentDateInteger, true);
        try {
            FileOutputStream foe = new FileOutputStream(logFile, true);
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(foe, "UTF-8"));
            writeString(writer, qLog.getQ());
            writer.write('\t');
            writeString(writer, qLog.getOrigin());
            writer.write('\t');
            int ficheCount = qLog.getFicheCount();
            writer.write(String.valueOf(ficheCount));
            int ficheMaximum = qLog.getFicheMaximum();
            if (ficheMaximum < 0) {
                ficheMaximum = scrutariSession.getScrutariDB().getCountStats().getFicheCount();
            }
            writer.write('\t');
            writer.write(String.valueOf(ficheMaximum));
            writer.write('\n');
            writer.close();
        } catch (IOException ioe) {
        }

    }

    @Override
    public int[] getDateIntegerArray(short dateIntegerType, int dateIntegerFilter) {
        if (dateIntegerType == DateInteger.TYPE_ANNEE) {
            return getAnneeArray();
        }
        if (dateIntegerType == DateInteger.TYPE_MOIS) {
            if ((dateIntegerFilter > 0) && (DateInteger.getDateIntegerType(dateIntegerFilter) == DateInteger.TYPE_ANNEE)) {
                return getMoisArray(DateInteger.getAnnee(dateIntegerFilter));
            } else {
                return getMoisArray();
            }
        }
        if (dateIntegerType == DateInteger.TYPE_JOUR) {
            if ((dateIntegerFilter > 0) && (DateInteger.getDateIntegerType(dateIntegerFilter) == DateInteger.TYPE_MOIS)) {
                return getJourArray(dateIntegerFilter);
            } else {
                return getJourArray();
            }
        }
        return new int[0];
    }

    @Override
    public String getQLog(int dateInteger) {
        File logFile = getLogFile(dateInteger, false);
        if (!logFile.exists()) {
            return null;
        }
        try {
            FileInputStream fie = new FileInputStream(logFile);
            String content = IOUtils.toString(fie, "UTF-8");
            fie.close();
            return content;
        } catch (IOException ioe) {
            return null;
        }
    }

    @Override
    public void log(short logType, String name, String content) {
        File subDir = getSubDir(logType);
        subDir.mkdirs();
        File logFile = new File(subDir, name + ".txt");
        try {
            OutputStreamWriter writer = new OutputStreamWriter(new FileOutputStream(logFile), "UTF-8");
            writer.write(content);
            writer.close();
        } catch (IOException ioe) {
        }
    }

    @Override
    public String getLog(short logType, String name) {
        File subDir = getSubDir(logType);
        File logFile = new File(subDir, name + ".txt");
        try {
            FileInputStream fie = new FileInputStream(logFile);
            String content = IOUtils.toString(fie, "UTF-8");
            fie.close();
            return content;
        } catch (IOException ioe) {
            return null;
        }
    }

    private void writeString(Writer writer, String text) throws IOException {
        int length = text.length();
        for (int i = 0; i < length; i++) {
            char carac = text.charAt(i);
            switch (carac) {
                case '\t':
                case '\n':
                case '\r':
                    writer.write(' ');
                    break;
                default:
                    writer.write(carac);
            }
        }
    }

    private File getLogFile(int dateInteger, boolean mkdirs) {
        StringBuilder buf = new StringBuilder();
        buf.append("q");
        buf.append(File.separatorChar);
        int annee = DateInteger.getAnnee(dateInteger);
        buf.append(annee);
        buf.append(File.separatorChar);
        int mois = DateInteger.getMois(dateInteger);
        if (mois < 10) {
            buf.append('0');
        }
        buf.append(mois);
        File dir = new File(logsDirectory, buf.toString());
        if (mkdirs) {
            dir.mkdirs();
        }
        int jour = DateInteger.getJour(dateInteger);
        String name;
        if (jour < 10) {
            name = "0" + jour + ".txt";
        } else {
            name = jour + ".txt";
        }
        return new File(dir, name);
    }

    private int[] getAnneeArray() {
        SortedSet<Integer> sortedSet = new TreeSet<Integer>();
        File dir = new File(logsDirectory, "q");
        File[] list = dir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File d = list[i];
            if (!d.isDirectory()) {
                continue;
            }
            try {
                int dateInteger = DateInteger.parse(d.getName());
                if (DateInteger.getDateIntegerType(dateInteger) == DateInteger.TYPE_ANNEE) {
                    sortedSet.add(dateInteger);
                }
            } catch (DateIntegerException die) {
            }
        }
        return toArray(sortedSet);
    }

    private int[] getMoisArray(int annee) {
        SortedSet<Integer> sortedSet = new TreeSet<Integer>();
        File dir = new File(logsDirectory, "q" + File.separator + annee);
        if (dir.exists()) {
            addMois(sortedSet, dir);
        }
        return toArray(sortedSet);
    }

    private int[] getMoisArray() {
        SortedSet<Integer> sortedSet = new TreeSet<Integer>();
        File dir = new File(logsDirectory, "q");
        File[] list = dir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File d = list[i];
            if (!d.isDirectory()) {
                continue;
            }
            addMois(sortedSet, d);
        }
        return toArray(sortedSet);
    }

    private void addMois(SortedSet<Integer> sortedSet, File dir) {
        String prefix = dir.getName() + "-";
        File[] list = dir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File d = list[i];
            if (!d.isDirectory()) {
                continue;
            }
            try {
                int dateInteger = DateInteger.parse(prefix + d.getName());
                if (DateInteger.getDateIntegerType(dateInteger) == DateInteger.TYPE_MOIS) {
                    sortedSet.add(dateInteger);
                }
            } catch (DateIntegerException die) {
            }
        }
    }

    private int[] getJourArray(int dateInteger) {
        SortedSet<Integer> sortedSet = new TreeSet<Integer>();
        int annee = DateInteger.getAnnee(dateInteger);
        StringBuilder buf = new StringBuilder();
        buf.append("q");
        buf.append(File.separatorChar);
        buf.append(annee);
        buf.append(File.separatorChar);
        int mois = DateInteger.getMois(dateInteger);
        if (mois < 10) {
            buf.append("0");
        }
        buf.append(mois);
        File dir = new File(logsDirectory, buf.toString());
        if (dir.exists()) {
            addJours(sortedSet, annee, dir);
        }
        return toArray(sortedSet);
    }

    private int[] getJourArray() {
        SortedSet<Integer> sortedSet = new TreeSet<Integer>();
        File dir = new File(logsDirectory, "q");
        File[] list = dir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File d = list[i];
            if (!d.isDirectory()) {
                continue;
            }
            int annee;
            try {
                annee = Integer.parseInt(d.getName());
            } catch (NumberFormatException nfe) {
                continue;
            }
            File[] list2 = d.listFiles();
            int length2 = list2.length;
            for (int j = 0; j < length2; j++) {
                File d2 = list2[j];
                if (!d2.isDirectory()) {
                    continue;
                }
                addJours(sortedSet, annee, d2);
            }

        }
        return toArray(sortedSet);
    }

    private void addJours(SortedSet<Integer> sortedSet, int annee, File dir) {
        String prefix = annee + "-" + dir.getName() + "-";
        File[] list = dir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File f = list[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            int idx = name.lastIndexOf('.');
            if (idx == -1) {
                continue;
            }
            name = name.substring(0, idx);
            try {
                int dateInteger = DateInteger.parse(prefix + name);
                if (DateInteger.getDateIntegerType(dateInteger) == DateInteger.TYPE_JOUR) {
                    sortedSet.add(dateInteger);
                }
            } catch (DateIntegerException nfe) {
            }
        }
    }

    private static int[] toArray(Set<Integer> set) {
        int size = set.size();
        int[] result = new int[size];
        int p = 0;
        for (Integer itg : set) {
            result[p] = itg.intValue();
            p++;
        }
        return result;
    }

    private File getSubDir(short logType) {
        try {
            String typeString = SctLogUtils.logTypeToString(logType);
            return new File(logsDirectory, typeString);
        } catch (IllegalArgumentException iae) {
            return logsDirectory;
        }
    }

}
