/* UtilLib - Copyright (c) 2006-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.io.PrintWriter;
import java.io.Writer;
import java.util.Map;
import net.mapeadores.util.io.AppendableWriter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisationPack;


/**
 *
 * @author Vincent Calame
 */
public class HtmlPrinter {

    protected PrintWriter pw;
    private ResourceLocalisationPack resourceLocalisationPack = new ResourceLocalisationPack();
    private ResourceLocalisation resourceLocalisation = resourceLocalisationPack.getResourceLocalisationInterface();
    protected boolean breakLine = true;

    public HtmlPrinter() {
    }

    public void addResourceLocalisation(ResourceLocalisation resourceLocalisation) {
        resourceLocalisationPack.addResourceLocalisation(resourceLocalisation);
    }

    public ResourceLocalisation getResourceLocalisation() {
        return resourceLocalisation;
    }

    public void initPrinter(PrintWriter pw) {
        this.pw = pw;
    }

    public void initPrinter(Appendable appendable) {
        if (appendable instanceof PrintWriter) {
            this.pw = (PrintWriter) appendable;
        } else if (appendable instanceof Writer) {
            this.pw = new PrintWriter((Writer) appendable);
        } else {
            this.pw = new PrintWriter(new AppendableWriter(appendable));
        }
    }

    public void flush() {
        pw.flush();
    }

    public PrintWriter getPrintWriter() {
        return pw;
    }

    public void appendHtmlDocType() {
        pw.println("<!DOCTYPE html>");
    }

    public void append(String s) {
        pw.print(s);
    }

    public void space() {
        pw.print(" ");
    }

    public void doublespace() {
        pw.print("&nbsp; ");
    }

    public void escape(String s) {
        if (s == null) {
            pw.print("null");
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            escape(carac);
        }
    }

    public void escape(char carac) {
        if (carac < 32) {
            pw.print(' ');
        } else {
            switch (carac) {
                case '&':
                    pw.print("&amp;");
                    break;
                case '"':
                    pw.print("&quot;");
                    break;
                case '<':
                    pw.print("&lt;");
                    break;
                case '>':
                    pw.print("&gt;");
                    break;
                default:
                    pw.print(carac);
            }
        }
    }

    public void escapeScriptString(String s) {
        if (s == null) {
            return;
        } else if (s.length() == 0) {
            return;
        }
        int taille = s.length();
        for (int i = 0; i < taille; i++) {
            char carac = s.charAt(i);
            if (carac < 32) {
                pw.print(' ');
            } else {
                switch (carac) {
                    case '"':
                        pw.print("\\\"");
                        break;
                    case '\'':
                        pw.print("\\'");
                        break;
                    case '\\':
                        pw.print("\\\\");
                        break;
                    default:
                        pw.print(carac);
                }
            }
        }
    }

    public void escape(String s, boolean preformatted) {
        if (preformatted) {
            if (s == null) {
                pw.print("null");
                return;
            } else if (s.length() == 0) {
                return;
            }
            int taille = s.length();
            for (int i = 0; i < taille; i++) {
                char carac = s.charAt(i);
                switch (carac) {
                    case '&':
                        pw.print("&amp;");
                        break;
                    case '"':
                        pw.print("&quot;");
                        break;
                    case '<':
                        pw.print("&lt;");
                        break;
                    case '>':
                        pw.print("&gt;");
                        break;
                    default:
                        pw.print(carac);
                }
            }
        } else {
            escape(s);
        }
    }

    public void localize(String key) {
        escape(getLocalization(key, null));
    }

    public String getLocalization(String key) {
        return getLocalization(key, null);
    }

    public String getLocalization(Message message) {
        return getLocalization(message.getMessageKey(), message.getMessageValue());
    }

    public String getLocalization(String resourceKey, Object value) {
        return resourceLocalisation.toString(resourceKey, value);
    }

    public void localize(String key, Object value) {
        escape(getLocalization(key, value));
    }

    public void localize(Message message) {
        escape(getLocalization(message));
    }

    public void startA(HTMLAttributes htmlAttributes) {
        openTag("a");
        if (htmlAttributes != null) {
            printEscapedAttribute("href", htmlAttributes.href());
            printAttribute("target", htmlAttributes.target());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endA() {
        endTag("a");
    }

    public void startBody() {
        startBody(null);
    }

    public void startBody(HTMLAttributes htmlAttributes) {
        openTag("body");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endBody() {
        endTag("body");
        breakLine();
    }

    public void appendBr() {
        pw.print("<br>");
    }

    public void appendBr(int i) {
        for (int j = 0; j < i; j++) {
            appendBr();
        }
    }

    public void startCell() {
        pw.print("<td>");
    }

    public void startCellWithTopAlign() {
        openTag("td");
        printAttribute("valign", "top");
        closeTag();
    }

    public void startCell(HTMLAttributes htmlAttributes) {
        openTag("td");
        if (htmlAttributes != null) {
            int colSpan = htmlAttributes.colspan();
            if (colSpan > 1) {
                printAttribute("colspan", String.valueOf(colSpan));
            }
            int rowSpan = htmlAttributes.rowspan();
            if (rowSpan > 1) {
                printAttribute("rowspan", String.valueOf(rowSpan));
            }
            printAttribute("width", htmlAttributes.width());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endCell() {
        endTag("td");
        breakLine();
    }

    public void appendEmptyCell() {
        startCell();
        endCell();
    }

    public void startHeaderCell(HTMLAttributes htmlAttributes) {
        openTag("th");
        if (htmlAttributes != null) {
            int colSpan = htmlAttributes.colspan();
            if (colSpan > 1) {
                printAttribute("colspan", String.valueOf(colSpan));
            }
            int rowSpan = htmlAttributes.rowspan();
            if (rowSpan > 1) {
                printAttribute("rowspan", String.valueOf(rowSpan));
            }
            printAttribute("width", htmlAttributes.width());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endHeaderCell() {
        endTag("th");
        breakLine();
    }

    public void appendCol(HTMLAttributes htmlAttributes) {
        openTag("col");
        if (htmlAttributes != null) {
            printAttribute("width", htmlAttributes.width());
            /*
             * printAttribute("height",htmlAttributes.getS.getHeight());
             */
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void startCode() {
        pw.print("<code>");
    }

    public void endCode() {
        endTag("code");
    }

    public void startEm() {
        pw.print("<em>");
    }

    public void endEm() {
        endTag("em");
    }

    public void startDiv(HTMLAttributes htmlAttributes) {
        openTag("div");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endDiv() {
        endTag("div");
        breakLine();
    }

    public void startFieldset(HTMLAttributes htmlAttributes) {
        openTag("fieldset");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endFieldset() {
        endTag("fieldset");
    }

    public void startForm(HTMLAttributes htmlAttributes) {
        openTag("form");
        if (htmlAttributes != null) {
            printAttribute("action", htmlAttributes.action());
            printAttribute("method", htmlAttributes.method());
            printAttribute("target", htmlAttributes.target());
            printAttribute("enctype", htmlAttributes.enctype());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endForm() {
        endTag("form");
    }

    public void startH1(HTMLAttributes htmlAttributes) {
        openTag("h1");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endH1() {
        endTag("h1");
        breakLine();
    }

    public void startH2(HTMLAttributes htmlAttributes) {
        openTag("h2");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endH2() {
        endTag("h2");
        breakLine();
    }

    public void startH3(HTMLAttributes htmlAttributes) {
        openTag("h3");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endH3() {
        endTag("h3");
        breakLine();
    }

    public void startH4(HTMLAttributes htmlAttributes) {
        openTag("h4");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endH4() {
        endTag("h4");
        breakLine();
    }

    public void startH5(HTMLAttributes htmlAttributes) {
        openTag("h5");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endH5() {
        endTag("h5");
        breakLine();
    }

    public void startH6(HTMLAttributes htmlAttributes) {
        openTag("h6");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endH6() {
        endTag("h6");
        breakLine();
    }

    public void startHead() {
        pw.print("<head>");
        breakLine();
    }

    public void endHead() {
        endTag("head");
        breakLine();
    }

    public void appendHr(HTMLAttributes htmlAttributes) {
        openTag("hr");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void startHtml(int langInteger) {
        openTag("html");
        if (langInteger != 0) {
            printAttribute("lang", LangInteger.toString(langInteger));
            if (LangInteger.isRTLScript(langInteger)) {
                printAttribute("dir", "rtl");
            }

        }
        closeTag();
        breakLine();
    }

    public void endHtml() {
        endTag("html");
    }

    public void appendImg(HTMLAttributes htmlAttributes) {
        openTag("img");
        if (htmlAttributes != null) {
            printAttribute("src", htmlAttributes.src());
            printEscapedAttribute("alt", htmlAttributes.alt());
            printAttribute("width", htmlAttributes.width());
            printAttribute("height", htmlAttributes.height());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void startIframe(HTMLAttributes htmlAttributes) {
        openTag("iframe");
        if (htmlAttributes != null) {
            printAttribute("src", htmlAttributes.src());
            printAttribute("width", htmlAttributes.width());
            printAttribute("height", htmlAttributes.height());
            printAttribute("scrolling", htmlAttributes.scrolling());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endIframe() {
        endTag("iframe");
        breakLine();
    }

    public void startLegend(HTMLAttributes htmlAttributes) {
        openTag("legend");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endLegend() {
        endTag("legend");
        breakLine();
    }

    public void startLi() {
        startLi(null);
    }

    public void startLi(HTMLAttributes htmlAttributes) {
        openTag("li");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endLi() {
        endTag("li");
        breakLine();
    }

    public void appendLink(HTMLAttributes htmlAttributes) {
        openTag("link");
        if (htmlAttributes != null) {
            printEscapedAttribute("href", htmlAttributes.href());
            printAttribute("rel", htmlAttributes.rel());
            printAttribute("type", htmlAttributes.type());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void appendMeta(HTMLAttributes htmlAttributes) {
        openTag("meta");
        if (htmlAttributes != null) {
            printAttribute("http-equiv", htmlAttributes.httpEquiv());
            printAttribute("content", htmlAttributes.content());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void startP() {
        pw.print("<p>");
    }

    public void startP(HTMLAttributes htmlAttributes) {
        openTag("p");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void startP(String className) {
        openTag("p");
        printAttribute("class", className);
        closeTag();
    }

    public void endP() {
        endTag("p");
        breakLine();
    }

    public void startPre(HTMLAttributes htmlAttributes) {
        openTag("pre");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endPre() {
        endTag("pre");
    }

    public void startOption(HTMLAttributes htmlAttributes) {
        openTag("option");
        if (htmlAttributes != null) {
            printEscapedAttribute("value", htmlAttributes.value());
            printBooleanAttribute("selected", htmlAttributes.selected());
            printBooleanAttribute("disabled", htmlAttributes.disabled());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endOption() {
        endTag("option");
        breakLine();
    }

    public void startOptGroup(HTMLAttributes htmlAttributes) {
        openTag("optgroup");
        if (htmlAttributes != null) {
            printEscapedAttribute("label", htmlAttributes.label());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endOptGroup() {
        endTag("optgroup");
        breakLine();
    }

    public void startRow() {
        pw.print("<tr>");
        breakLine();
    }

    public void startRow(HTMLAttributes htmlAttributes) {
        openTag("tr");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endRow() {
        endTag("tr");
        breakLine();
    }

    public void startScript(HTMLAttributes htmlAttributes) {
        openTag("script");
        if (htmlAttributes != null) {
            printAttribute("type", htmlAttributes.type());
            printAttribute("src", htmlAttributes.src());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endScript() {
        endTag("script");
        breakLine();
    }

    public void startNoscript() {
        pw.print("<noscript>");
    }

    public void endNoscript() {
        endTag("noscript");
    }

    public void startSelect(HTMLAttributes htmlAttributes) {
        openTag("select");
        printHTMLAttributes(htmlAttributes);
        printBooleanAttribute("multiple", htmlAttributes.multiple());
        printAttribute("size", htmlAttributes.size());
        closeTag();
        breakLine();
    }

    public void endSelect() {
        endTag("select");
    }

    public void startButton(HTMLAttributes htmlAttributes) {
        openTag("button");
        printHTMLAttributes(htmlAttributes);
        printBooleanAttribute("disabled", htmlAttributes.disabled());
        printAttribute("type", htmlAttributes.type());
        closeTag();
    }

    public void endButton() {
        endTag("button");
    }

    public void startSmall() {
        pw.print("<small>");
    }

    public void endSmall() {
        endTag("small");
    }

    public void startSpan(HTMLAttributes htmlAttributes) {
        openTag("span");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endSpan() {
        endTag("span");
    }

    public void startStrong() {
        pw.print("<strong>");
    }

    public void endStrong() {
        endTag("strong");
    }

    public void startTable() {
        startTable(null);
    }

    public void startTable(HTMLAttributes htmlAttributes) {
        openTag("table");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTable() {
        endTag("table");
        breakLine();
    }

    public void startTHead(HTMLAttributes htmlAttributes) {
        openTag("thead");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTHead() {
        endTag("thead");
        breakLine();
    }

    public void startTFoot(HTMLAttributes htmlAttributes) {
        openTag("tfoot");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTFoot() {
        endTag("tfoot");
        breakLine();
    }

    public void startTBody(HTMLAttributes htmlAttributes) {
        openTag("tbody");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endTBody() {
        pw.print("</tbody>");
        breakLine();
    }

    public void startTitle() {
        pw.print("<title>");
    }

    public void endTitle() {
        endTag("title");
        breakLine();
    }

    public void startTextArea(HTMLAttributes htmlAttributes) {
        openTag("textarea");
        int cols = 0;
        int rows = 0;
        if (htmlAttributes != null) {
            cols = htmlAttributes.cols();
            rows = htmlAttributes.rows();
        }
        if (cols < 1) {
            cols = 50;
        }
        if (rows < 1) {
            rows = 3;
        }
        printAttribute("cols", String.valueOf(cols));
        printAttribute("rows", String.valueOf(rows));
        printBooleanAttribute("required", htmlAttributes.required());
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endTextArea() {
        endTag("textarea");
    }

    public void startUl() {
        pw.print("<ul>");
        breakLine();
    }

    public void startUl(HTMLAttributes htmlAttributes) {
        openTag("ul");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void endUl() {
        endTag("ul");
        breakLine();
    }

    public void appendInput(HTMLAttributes htmlAttributes) {
        appendInput(htmlAttributes, false);
    }

    public void appendInput(HTMLAttributes htmlAttributes, boolean preformatted) {
        openTag("input");
        if (htmlAttributes != null) {
            printAttribute("type", htmlAttributes.type());
            printEscapedAttribute("value", htmlAttributes.value(), preformatted);
            printAttribute("size", htmlAttributes.size());
            printBooleanAttribute("checked", htmlAttributes.checked());
            printBooleanAttribute("disabled", htmlAttributes.disabled());
            printBooleanAttribute("readonly", htmlAttributes.readonly());
            printBooleanAttribute("required", htmlAttributes.required());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void appendSmallestInput(String name, String value) {
        appendTextInput(name, value, 3);
    }

    public void appendSmallInput(String name, String value) {
        appendTextInput(name, value, 15);
    }

    public void appendMediumInput(String name, String value) {
        appendTextInput(name, value, 30);
    }

    public void appendLargeInput(String name, String value) {
        appendTextInput(name, value, 60);
    }

    public void appendHiddenInput(String name, String value) {
        openTag("input");
        printAttribute("type", HtmlConstants.HIDDEN_TYPE);
        printAttribute("name", name);
        printEscapedAttribute("value", value);
        closeTag();
    }

    private void appendTextInput(String name, String value, int size) {
        openTag("input");
        printAttribute("type", HtmlConstants.TEXT_TYPE);
        printAttribute("name", name);
        printEscapedAttribute("value", value);
        if (size > 0) {
            printAttribute("size", String.valueOf(size));
        }
        closeTag();
    }

    public void startLabel(HTMLAttributes htmlAttributes) {
        openTag("label");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void endLabel() {
        endTag("label");
    }

    public void breakLine() {
        if (breakLine) {
            pw.println("");
        }
    }

    private boolean printHTMLAttributes(HTMLAttributes htmlAttributes) {
        if (htmlAttributes == null) {
            return false;
        }
        printAttribute("class", htmlAttributes.classes());
        printAttribute("id", htmlAttributes.id());
        printAttribute("style", htmlAttributes.style());
        printAttribute("name", htmlAttributes.name());
        Map<String, String> attrMap = htmlAttributes.getAttrMap();
        if (attrMap != null) {
            for (Map.Entry<String, String> entry : attrMap.entrySet()) {
                printAttribute(entry.getKey(), entry.getValue());
            }
        }
        printEscapedAttribute("title", htmlAttributes.title());
        return true;
    }

    public void printAttribute(String name, String value) {
        if (value == null) {
            return;
        }
        pw.print(" ");
        pw.print(name);
        pw.print("=\"");
        pw.print(value);
        pw.print("\"");
    }

    private void printBooleanAttribute(String name, boolean value) {
        if (value) {
            printAttribute(name, name);
        }
    }

    public void openTag(String tag) {
        pw.print("<");
        pw.print(tag);
    }

    public void closeTag() {
        pw.print(">");
    }

    public void endTag(String tag) {
        pw.print("</");
        pw.print(tag);
        pw.print(">");
    }

    private void printEscapedAttribute(String name, String value) {
        printEscapedAttribute(name, value, false);
    }

    private void printEscapedAttribute(String name, String value, boolean preformatted) {
        if (value == null) {
            return;
        }
        pw.print(" ");
        pw.print(name);
        pw.print("=\"");
        escape(value, preformatted);
        pw.print("\"");
    }

    public void startRowsFrameset(String rows) {
        openTag("frameset");
        printAttribute("rows", rows);
        closeTag();
        breakLine();
    }

    public void startColsFrameset(String cols) {
        openTag("frameset");
        printAttribute("cols", cols);
        closeTag();
        breakLine();
    }

    public void appendFrame(String name, String src) {
        openTag("frame");
        printAttribute("name", name);
        printAttribute("src", src);
        closeTag();
        breakLine();
    }

    public void endFrameset() {
        endTag("frameset");
        breakLine();
    }

}
