/* UtilLib - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logicaloperation;

import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class LogicalOperationUtils {

    private LogicalOperationUtils() {
    }

    public static SubOperand toSubOperand(Operand[] array, short operator) {
        if (array.length < 2) {
            throw new IllegalArgumentException("array.length < 2");
        }
        return new ArraySubOperand(array, operator);
    }

    public static SimpleOperand toSimpleOperand(boolean acceptMode, String operandString) {
        return new DefaultSimpleOperand(acceptMode, operandString);
    }

    public static String toInformatiqueString(Operand operand) {
        StringBuilder buf = new StringBuilder();
        appendInformatiqueOperand(buf, operand, true);
        return buf.toString();
    }

    private static void appendInformatiqueOperand(StringBuilder buf, Operand operand, boolean root) {
        if (operand instanceof SimpleOperand) {
            SimpleOperand simpleOperand = (SimpleOperand) operand;
            if (!simpleOperand.isAcceptMode()) {
                buf.append('!');
            }
            String operandString = ((SimpleOperand) operand).getOperandString();
            int length = operandString.length();
            for (int i = 0; i < length; i++) {
                char carac = operandString.charAt(i);
                switch (carac) {
                    case ')':
                    case '(':
                    case '\\':
                    case '&':
                    case '|':
                        buf.append('\\');
                    default:
                        buf.append(carac);
                }
            }
        } else if (operand instanceof SubOperand) {
            SubOperand subOperand = (SubOperand) operand;
            String separator = getSeparator(subOperand.getOperator());
            if (!root) {
                buf.append('(');
            }
            int count = subOperand.getOperandCount();
            for (int i = 0; i < count; i++) {
                if (i > 0) {
                    buf.append(separator);
                }
                appendInformatiqueOperand(buf, subOperand.getOperand(i), false);
            }
            if (!root) {
                buf.append(')');
            }
        } else {
            buf.append(operand.toString());
        }
    }

    public static String toEnsembleString(Operand operand) {
        StringBuilder buf = new StringBuilder();
        appendEnsembleOperand(buf, operand, true);
        return buf.toString();
    }

    public static short inverseOperator(short operator) {
        if (operator == LogicalOperationConstants.UNION_OPERATOR) {
            return LogicalOperationConstants.INTERSECTION_OPERATOR;
        } else {
            return LogicalOperationConstants.UNION_OPERATOR;
        }
    }

    private static void appendEnsembleOperand(StringBuilder buf, Operand operand, boolean root) {
        if (operand instanceof SimpleOperand) {
            SimpleOperand simpleOperand = (SimpleOperand) operand;
            if (!simpleOperand.isAcceptMode()) {
                buf.append('\u00AC');
            }
            buf.append(((SimpleOperand) operand).getOperandString());
        } else if (operand instanceof SubOperand) {
            SubOperand subOperand = (SubOperand) operand;
            buf.append(getOperator(subOperand.getOperator()));
            buf.append('(');
            int count = subOperand.getOperandCount();
            for (int i = 0; i < count; i++) {
                if (i > 0) {
                    buf.append(", ");
                }
                appendEnsembleOperand(buf, subOperand.getOperand(i), false);
            }
            buf.append(')');
        } else {
            String s = operand.toString();
            int length = s.length();
            if (length > 0) {
                if (s.charAt(0) == '!') {
                    buf.append('\u00AC');
                    buf.append(s, 1, length);
                } else {
                    buf.append(s);
                }
            }
        }
    }

    private static String getSeparator(short operator) {
        switch (operator) {
            case LogicalOperationConstants.INTERSECTION_OPERATOR:
                return " && ";
            case LogicalOperationConstants.UNION_OPERATOR:
                return " || ";
            default:
                throw new SwitchException("Unknown operator = " + operator);
        }
    }

    private static String getOperator(short operator) {
        switch (operator) {
            case LogicalOperationConstants.INTERSECTION_OPERATOR:
                return "\u22C2";
            case LogicalOperationConstants.UNION_OPERATOR:
                return "\u22C3";
            default:
                throw new SwitchException("Unknown operator = " + operator);
        }
    }


    private static class ArraySubOperand implements SubOperand {

        private Operand[] array;
        private short operator;

        private ArraySubOperand(Operand[] array, short operator) {
            this.array = array;
            this.operator = operator;
        }

        public short getOperator() {
            return operator;
        }

        public int getOperandCount() {
            return array.length;
        }

        public Operand getOperand(int i) {
            return array[i];
        }

    }


    private static class DefaultSimpleOperand implements SimpleOperand {

        private boolean acceptMode;
        private String operandString;

        private DefaultSimpleOperand(boolean acceptMode, String operandString) {
            this.acceptMode = acceptMode;
            this.operandString = operandString;
        }

        public boolean isAcceptMode() {
            return acceptMode;
        }

        public String getOperandString() {
            return operandString;
        }

    }

}
