/* UtilLib - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class MoneyUtils {

    public final static MoneyConversion SAME_MONEYCONVERSION = new SameMoneyConversion();
    private final static Decimal UN = new Decimal(1L, (byte) 0, 0);
    private final static Decimal PETIT = new Decimal(0L, (byte) 4, 1);

    private MoneyUtils() {
    }

    /**
     * 1 otherCurrency = taux pivotCurrency
     */
    public static MoneyConversion toMoneyConversion(int pivotCurrency, int otherCurrency, Decimal decimal) {
        return new DefaultMoneyConversion(pivotCurrency, otherCurrency, decimal);
    }

    public static Decimal getConversionRate(int otherCurrency, long otherMoneyLong, int pivotCurrency, long pivotMoneyLong) {
        int otherMultiplicator = CurrencyInteger.getSubunitMultiplicator(otherCurrency);
        int pivotMultiplicator = CurrencyInteger.getSubunitMultiplicator(pivotCurrency);
        double otherDouble = ((double) otherMoneyLong) / otherMultiplicator;
        double pivotDouble = ((double) pivotMoneyLong) / pivotMultiplicator;
        double taux = pivotDouble / otherDouble;
        long tauxLong = (long) (taux * 100000);
        if (tauxLong < 0) {
            tauxLong = -tauxLong;
        }
        if (tauxLong == 0) {
            return PETIT;
        }
        String tauxString = Long.toString(tauxLong);
        int length = tauxString.length();
        if (length > 5) {
            try {
                return StringUtils.parseDecimal(tauxString.substring(0, length - 5), tauxString.substring(length - 5));
            } catch (NumberFormatException nfe) {
                throw new ShouldNotOccurException(nfe);
            }
        } else {
            StringBuilder buf = new StringBuilder();
            for (int i = 0; i < (5 - length); i++) {
                buf.append('0');
            }
            buf.append(tauxString);
            try {
                return StringUtils.parseDecimal("0", buf.toString());
            } catch (NumberFormatException nfe) {
                throw new ShouldNotOccurException(nfe);
            }
        }
    }


    private static class SameMoneyConversion implements MoneyConversion {

        private SameMoneyConversion() {
        }

        public long convertFromPivot(long pivotMoneyLong) {
            return pivotMoneyLong;
        }

        public long convertToPivot(long otherMoneyLong) {
            return otherMoneyLong;
        }

        public Decimal getConversionRate() {
            return UN;
        }

    }


    private static class DefaultMoneyConversion implements MoneyConversion {

        private int pivotMultiplicator;
        private int otherMultiplicator;
        private Decimal conversionRate;
        private double taux;

        /*
         * 1 pivot = taux * other
         */
        private DefaultMoneyConversion(int pivotCurrency, int otherCurrency, Decimal conversionRate) {
            this.pivotMultiplicator = CurrencyInteger.getSubunitMultiplicator(pivotCurrency);
            this.otherMultiplicator = CurrencyInteger.getSubunitMultiplicator(otherCurrency);
            this.conversionRate = conversionRate;
            this.taux = conversionRate.toDouble();
        }

        public Decimal getConversionRate() {
            return conversionRate;
        }

        public long convertFromPivot(long pivotMoneyLong) {
            boolean opposite = false;
            if (pivotMoneyLong < 0) {
                //Note : le passage en positif permet d'éviter des différences d'arrondi suivant si le nombre est positif ou négatif
                opposite = true;
                pivotMoneyLong = -pivotMoneyLong;
            }
            double pivotDouble = ((double) pivotMoneyLong) / pivotMultiplicator;
            double otherDouble = pivotDouble / taux;
            otherDouble = otherDouble * otherMultiplicator;
            long lg = (long) Math.round(otherDouble);
            if (opposite) {
                lg = -lg;
            }
            return lg;
        }

        public long convertToPivot(long otherMoneyLong) {
            boolean opposite = false;
            if (otherMoneyLong < 0) {
                opposite = true;
                otherMoneyLong = -otherMoneyLong;
            }
            double otherDouble = ((double) otherMoneyLong) / otherMultiplicator;
            double pivotDouble = otherDouble * taux;
            pivotDouble = pivotDouble * pivotMultiplicator;
            long lg = (long) Math.round(pivotDouble);
            if (opposite) {
                lg = -lg;
            }
            return lg;
        }

    }

}
