/* UtilLib - Copyright (c) 2005-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.text.lexie.LexieParser;
import net.mapeadores.util.text.lexie.LexieParserListener;
import net.mapeadores.util.text.search.MultiSearchToken;
import net.mapeadores.util.text.search.SearchToken;
import net.mapeadores.util.text.search.SearchTokenList;
import net.mapeadores.util.text.search.SearchUtils;
import net.mapeadores.util.text.search.SimpleSearchToken;


/**
 *
 * @author  Vincent Calame
 */
public class SearchSequence implements SearchTokenList {

    private SearchToken[] tokens;
    private String source;
    private int[] nonpertinentIndexArray;

    private SearchSequence() {
        this.source = "";
        this.tokens = new SearchToken[0];
    }

    private SearchSequence(String source) {
        this.source = source;
    }

    private SearchSequence(String source, SearchToken[] tokens, int[] nonpertinentIndexArray) {
        this.source = source;
        this.tokens = tokens;
        this.nonpertinentIndexArray = nonpertinentIndexArray;
    }

    public static SearchSequence newInstance(String source, SearchToken[] tokens, int[] nonpertinentIndexArray) {
        return new SearchSequence(source, tokens, nonpertinentIndexArray);
    }

    public static SearchSequence createSearchSequence(String chaine) {
        chaine = chaine.trim();
        if (chaine.length() == 0) {
            return new SearchSequence();
        }
        SearchSequenceParser parser = new SearchSequenceParser(chaine);
        parser.parse();
        SearchSequence sequence = new SearchSequence(chaine);
        sequence.tokens = parser.getTokenArray();
        return sequence;
    }

    public boolean isEmpty() {
        return (tokens.length == 0);
    }

    public int getTokenCount() {
        return tokens.length;
    }

    public boolean hasNonPertinentTokens() {
        return (nonpertinentIndexArray != null);
    }

    public int[] getNonPertinentIndexArray() {
        return nonpertinentIndexArray;
    }

    public SearchToken getToken(int i) {
        return tokens[i];
    }

    @Override
    public String toString() {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < tokens.length; i++) {
            if (i > 0) {
                buf.append(' ');
            }
            SearchToken token = tokens[i];
            if (token instanceof MultiSearchToken) {
                buf.append('\"');
                MultiSearchToken multiToken = (MultiSearchToken) token;
                int tokenCount = multiToken.getSubtokenCount();
                for (int j = 0; j < tokenCount; j++) {
                    if (j > 0) {
                        buf.append(' ');
                    }
                    buf.append(SearchUtils.toCompleteString(multiToken.getSubtoken(j)));
                }
                buf.append('\"');
            } else {
                buf.append(SearchUtils.toCompleteString((SimpleSearchToken) token));
            }
        }
        return buf.toString();
    }

    public String getSource() {
        return source;
    }


    static private class SearchSequenceParser implements LexieParserListener {

        String source;
        List<SearchToken> tokenList = new ArrayList<SearchToken>();
        int currentType = TextConstants.MATCHES;
        String currentWord = null;
        boolean onQuote = false;
        List<SimpleSearchToken> quoteList = new ArrayList<SimpleSearchToken>();

        SearchSequenceParser(String source) {
            this.source = source;
        }

        void parse() {
            LexieParser.parse(source, this);
            if (onQuote) {
                flushQuote();
            } else {
                flushCurrentWord();
            }
        }

        public void flushLexie(String word, int wordStart) {
            flushCurrentWord();
            currentWord = word;
        }

        private void flushCurrentWord() {
            if (currentWord != null) {
                if (onQuote) {
                    quoteList.add(SearchUtils.toSimpleSearchToken(currentWord, currentType));
                } else {
                    tokenList.add(SearchUtils.toSimpleSearchToken(currentWord, currentType));
                }
                currentWord = null;
                currentType = TextConstants.MATCHES;
            }
        }

        private void flushQuote() {
            flushCurrentWord();
            int size = quoteList.size();
            if (size > 0) {
                SimpleSearchToken[] array = new SimpleSearchToken[size];
                for (int j = 0; j < size; j++) {
                    array[j] = quoteList.get(j);
                }
                int p = tokenList.size();
                tokenList.add(SearchUtils.toMultiSearchToken(array));
            }
            onQuote = false;
            quoteList.clear();
        }

        public void checkSpecialChar(char carac) {
            if (carac == '*') {
                if (currentWord == null) {
                    currentType = TextConstants.ENDSWITH;
                } else {
                    currentType = currentType | TextConstants.STARTSWITH;
                    flushCurrentWord();
                }
            } else if (carac == '\"') {
                if (onQuote) {
                    flushQuote();
                } else {
                    flushCurrentWord();
                    onQuote = true;
                }
            } else {
                flushCurrentWord();
            }
        }

        public SearchToken[] getTokenArray() {
            int size = tokenList.size();
            return tokenList.toArray(new SearchToken[size]);
        }

    }

}
