/* UtilLib - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.alphabet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.Collator;
import java.text.RuleBasedCollator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.collation.CollationUnit;


/**
 *
 * @author Vincent Calame
 */
public class Alphabet implements AlphabeticEntry {

    private final static Map<Integer, InitialInfo> initialInfoMap = new HashMap<Integer, InitialInfo>();
    private final static Map<Integer, Article[]> articleArrayMap = new HashMap<Integer, Article[]>();
    private final static Set<Integer> nulLangIntegerSet = new HashSet<Integer>();
    private String sourceString;
    private String articleString;
    private String entryString;
    private String initialChar;
    private boolean withSeparationSpace;

    public Alphabet() {
    }

    private Alphabet(String sourceString) {
        setSourceString(sourceString);
        setEntryString(sourceString);

    }

    public static AlphabeticEntry newInstance(String sourceString, int langInteger) {
        Article[] array = getArray(langInteger);
        if (array == null) {
            return new Alphabet(sourceString);
        }
        Alphabet alphabet = new Alphabet();
        alphabet.setSourceString(sourceString);
        String articleString = null;
        boolean withEspace = false;
        String entryString = sourceString;
        for (int i = 0; i < array.length; i++) {
            Article currentArticle = array[i];
            if (sourceString.startsWith(currentArticle.getArticleString())) {
                int length = currentArticle.getArticleLength();
                if (sourceString.length() > length) {
                    articleString = currentArticle.getTrimedArticleString();
                    withEspace = currentArticle.isWithEspace();
                    entryString = sourceString.substring(length);
                }
                break;
            }
        }
        alphabet.setEntryString(entryString);
        alphabet.setArticleString(articleString);
        alphabet.setWithSeparationSpace(withEspace);
        alphabet.setInitialChar(getInitialChar(entryString, langInteger));
        return alphabet;
    }

    private static Article[] getArray(int langInteger) {
        if (nulLangIntegerSet.contains(langInteger)) {
            return null;
        }
        Article[] obj = articleArrayMap.get(langInteger);
        if (obj == null) {
            obj = initArray(langInteger);
            if (obj == null) {
                int langPart = LangInteger.getLangPart(langInteger);
                if (langPart != langInteger) {
                    obj = getArray(langPart);
                }
            }
            if (obj == null) {
                synchronized (nulLangIntegerSet) {
                    nulLangIntegerSet.add(langInteger);
                }
            } else {
                synchronized (articleArrayMap) {
                    articleArrayMap.put(langInteger, obj);
                }
            }
        }
        return obj;
    }

    /*
     * Les articles doivent commencer par les plus longs (voir la boucle de newInstance)
     */
    private static Article[] initArray(int langInteger) {
        InputStream inputStream = Alphabet.class.getResourceAsStream("resources/article_" + LangInteger.toString(langInteger) + ".txt");
        if (inputStream == null) {
            return null;
        }
        try {
            List<String> list = new ArrayList<String>();
            BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
            String ligne;
            while ((ligne = reader.readLine()) != null) {
                ligne = ligne.trim();
                if ((ligne.length() > 0) && (ligne.charAt(0) != '#')) {
                    ligne = ligne.replace('_', ' ');
                    list.add(ligne);
                }
            }
            reader.close();
            int size = list.size();
            Article[] result = new Article[size];
            for (int i = 0; i < size; i++) {
                result[i] = new Article(list.get(i));
            }
            return result;
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }
    }

    public boolean isWithArticle() {
        return (!sourceString.equals(entryString));
    }

    public String getSourceString() {
        return sourceString;
    }

    private void setSourceString(String sourceString) {
        this.sourceString = sourceString;
    }

    public String getArticleString() {
        return articleString;
    }

    private void setArticleString(String articleString) {
        this.articleString = articleString;
    }

    public String getEntryString() {
        return entryString;
    }

    private void setEntryString(String entryString) {
        this.entryString = entryString;
    }

    public boolean isWithSeparationSpace() {
        return withSeparationSpace;
    }

    private void setWithSeparationSpace(boolean withSeparationSpace) {
        this.withSeparationSpace = withSeparationSpace;
    }

    public String getInitialChar() {
        return initialChar;
    }

    private void setInitialChar(String initialChar) {
        this.initialChar = initialChar;
    }

    private static String getInitialChar(String entryString, int langInteger) {
        InitialInfo initialInfo = getInitialInfo(langInteger);
        return initialInfo.getInitiale(entryString);
    }

    private static InitialInfo getInitialInfo(int langInteger) {
        InitialInfo initialInfo = initialInfoMap.get(langInteger);
        if (initialInfo == null) {
            synchronized (initialInfoMap) {
                RuleBasedCollator collator = (RuleBasedCollator) Collator.getInstance(LangInteger.getLocale(langInteger));
                collator.setStrength(Collator.PRIMARY);
                initialInfo = new InitialInfo(collator);
                initialInfoMap.put(langInteger, initialInfo);
            }
        }
        return initialInfo;
    }


    private static class Article {

        private String articleString;
        private boolean withEspace;
        private String trimedArticleString;

        Article(String articleString) {
            this.articleString = articleString;
            if (articleString.charAt(articleString.length() - 1) == ' ') {
                trimedArticleString = articleString.substring(0, articleString.length() - 1);
                withEspace = true;
            } else {
                withEspace = false;
                trimedArticleString = articleString;
            }
        }

        public String getArticleString() {
            return articleString;
        }

        public boolean isWithEspace() {
            return withEspace;
        }

        public String getTrimedArticleString() {
            return trimedArticleString;
        }

        public int getArticleLength() {
            return articleString.length();
        }

    }


    private static class InitialInfo {

        private RuleBasedCollator collator;
        private Map<Character, String> initialStringMap = new HashMap<Character, String>();

        private InitialInfo(RuleBasedCollator collator) {
            this.collator = collator;
            for (char c = '0'; c <= '9'; c++) {
                addInitialString(c);
            }
            for (char c = 'A'; c <= 'Z'; c++) {
                addInitialString(c);
            }
        }

        private void addInitialString(char c) {
            char[] array = {c};
            String initialString = new String(array);
            String collatedKey = CollationUnit.collate(initialString, collator, true);
            initialStringMap.put(new Character(collatedKey.charAt(0)), initialString);
        }

        private String getInitiale(String entryString) {
            if (entryString.length() == 0) {
                return null;
            }
            String collatedKey = CollationUnit.collate(entryString, collator, true);
            Character initial = new Character(collatedKey.charAt(0));
            String result = initialStringMap.get(initial);
            if (result == null) {
                return "_";
            }
            return result;
        }

    }

}
